@@ -2,6 +2,10 @@
 
 set -e
 
+# If run outside of a deqp-runner invoction (e.g. piglit trace replay), then act
+# the same as the first thread in its threadpool.
+THREAD=${DEQP_RUNNER_THREAD:-0}
+
 #
 # Helper to generate CIDs for virtio-vsock based communication with processes
 # running inside crosvm guests.
@@ -17,7 +21,7 @@ set -e
 #  - VSOCK_STDOUT, VSOCK_STDERR: the port numbers the guest should accept
 #    vsock connections on in order to transfer output messages
 #
-#  - VSOCK_TEMP_DIR: the temporary directory path used to pass additional
+#  - VM_TEMP_DIR: the temporary directory path used to pass additional
 #    context data towards the guest
 #
 set_vsock_context() {
@@ -26,19 +30,12 @@ set_vsock_context() {
         exit 1
     }
 
-    local dir_prefix="/tmp-vsock."
-    local cid_prefix=0
-    unset VSOCK_TEMP_DIR
+    VM_TEMP_DIR="/tmp-vm.${THREAD}"
+    # Clear out any leftover files from a previous run.
+    rm -rf $VM_TEMP_DIR
+    mkdir $VM_TEMP_DIR || return 1
 
-    while [ ${cid_prefix} -lt 128 ]; do
-        VSOCK_TEMP_DIR=${dir_prefix}${cid_prefix}
-        mkdir "${VSOCK_TEMP_DIR}" >/dev/null 2>&1 && break || unset VSOCK_TEMP_DIR
-        cid_prefix=$((cid_prefix + 1))
-    done
-
-    [ -n "${VSOCK_TEMP_DIR}" ] || return 1
-
-    VSOCK_CID=$(((CI_JOB_ID & 0x1ffffff) | ((cid_prefix & 0x7f) << 25)))
+    VSOCK_CID=$(((CI_JOB_ID & 0x1ffffff) | ((${THREAD} & 0x7f) << 25)))
     VSOCK_STDOUT=5001
     VSOCK_STDERR=5002
 
@@ -51,19 +48,26 @@ if [ -n "${1##*.sh}" ] && [ -z "${1##*"deqp"*}" ]; then
     export DEQP_BIN_DIR
 fi
 
-set_vsock_context || { echo "Could not generate crosvm vsock CID" >&2; exit 1; }
+VM_SOCKET=crosvm-${THREAD}.sock
 
-# Ensure cleanup on script exit
-trap 'exit ${exit_code}' INT TERM
-trap 'exit_code=$?; [ -z "${CROSVM_PID}${SOCAT_PIDS}" ] || kill ${CROSVM_PID} ${SOCAT_PIDS} >/dev/null 2>&1 || true; rm -rf ${VSOCK_TEMP_DIR}' EXIT
+# Terminate any existing crosvm, if a previous invocation of this shell script
+# was terminated due to timeouts.  This "vm stop" may fail if the crosvm died
+# without cleaning itself up.
+if [ -e $VM_SOCKET ]; then
+   crosvm stop $VM_SOCKET || rm -rf $VM_SOCKET
+   # Wait for socats from that invocation to drain
+   sleep 5
+fi
+
+set_vsock_context || { echo "Could not generate crosvm vsock CID" >&2; exit 1; }
 
 # Securely pass the current variables to the crosvm environment
 echo "Variables passed through:"
 SCRIPT_DIR=$(readlink -en "${0%/*}")
-${SCRIPT_DIR}/common/generate-env.sh | tee ${VSOCK_TEMP_DIR}/crosvm-env.sh
+${SCRIPT_DIR}/common/generate-env.sh | tee ${VM_TEMP_DIR}/crosvm-env.sh
 
 # Set the crosvm-script as the arguments of the current script
-echo "$@" > ${VSOCK_TEMP_DIR}/crosvm-script.sh
+echo "$@" > ${VM_TEMP_DIR}/crosvm-script.sh
 
 # Setup networking
 /usr/sbin/iptables-legacy -w -t nat -A POSTROUTING -o eth0 -j MASQUERADE
@@ -71,16 +75,14 @@ echo 1 > /proc/sys/net/ipv4/ip_forward
 
 # Start background processes to receive output from guest
 socat -u vsock-connect:${VSOCK_CID}:${VSOCK_STDERR},retry=200,interval=0.1 stderr &
-SOCAT_PIDS=$!
 socat -u vsock-connect:${VSOCK_CID}:${VSOCK_STDOUT},retry=200,interval=0.1 stdout &
-SOCAT_PIDS="${SOCAT_PIDS} $!"
 
 # Prepare to start crosvm
 unset DISPLAY
 unset XDG_RUNTIME_DIR
 
 CROSVM_KERN_ARGS="quiet console=null root=my_root rw rootfstype=virtiofs ip=192.168.30.2::192.168.30.1:255.255.255.0:crosvm:eth0"
-CROSVM_KERN_ARGS="${CROSVM_KERN_ARGS} init=${SCRIPT_DIR}/crosvm-init.sh -- ${VSOCK_STDOUT} ${VSOCK_STDERR} ${VSOCK_TEMP_DIR}"
+CROSVM_KERN_ARGS="${CROSVM_KERN_ARGS} init=${SCRIPT_DIR}/crosvm-init.sh -- ${VSOCK_STDOUT} ${VSOCK_STDERR} ${VM_TEMP_DIR}"
 
 [ "${CROSVM_GALLIUM_DRIVER}" = "llvmpipe" ] && \
     CROSVM_LIBGL_ALWAYS_SOFTWARE=true || CROSVM_LIBGL_ALWAYS_SOFTWARE=false
@@ -95,21 +97,15 @@ crosvm run \
     --gpu "${CROSVM_GPU_ARGS}" -m 4096 -c 2 --disable-sandbox \
     --shared-dir /:my_root:type=fs:writeback=true:timeout=60:cache=always \
     --host_ip "192.168.30.1" --netmask "255.255.255.0" --mac "AA:BB:CC:00:00:12" \
+    -s $VM_SOCKET \
     --cid ${VSOCK_CID} -p "${CROSVM_KERN_ARGS}" \
-    /lava-files/${KERNEL_IMAGE_NAME:-bzImage} > ${VSOCK_TEMP_DIR}/crosvm 2>&1 &
+    /lava-files/${KERNEL_IMAGE_NAME:-bzImage} > ${VM_TEMP_DIR}/crosvm 2>&1
 
-# Wait for crosvm process to terminate
-CROSVM_PID=$!
-wait ${CROSVM_PID}
 CROSVM_RET=$?
-unset CROSVM_PID
 
 [ ${CROSVM_RET} -eq 0 ] && {
-    # socat background processes terminate gracefully on remote peers exit
-    wait
-    unset SOCAT_PIDS
     # The actual return code is the crosvm guest script's exit code
-    CROSVM_RET=$(cat ${VSOCK_TEMP_DIR}/exit_code 2>/dev/null)
+    CROSVM_RET=$(cat ${VM_TEMP_DIR}/exit_code 2>/dev/null)
     # Force error when the guest script's exit code is not available
     CROSVM_RET=${CROSVM_RET:-1}
 }
@@ -118,7 +114,7 @@ unset CROSVM_PID
 [ ${CROSVM_RET} -eq 0 ] || {
     set +x
     echo "Dumping crosvm output.." >&2
-    cat ${VSOCK_TEMP_DIR}/crosvm >&2
+    cat ${VM_TEMP_DIR}/crosvm >&2
     set -x
 }
 
