@@ -21,30 +21,56 @@
  * IN THE SOFTWARE.
  */
 
+#ifndef RADV_SHADER_ARGS_H
+#define RADV_SHADER_ARGS_H
+
 #include "compiler/shader_enums.h"
 #include "util/list.h"
 #include "util/macros.h"
 #include "ac_shader_args.h"
 #include "amd_family.h"
 #include "radv_constants.h"
+#include "radv_shader.h"
 
 struct radv_shader_args {
    struct ac_shader_args ac;
-   struct radv_shader_info *shader_info;
-   const struct radv_nir_compiler_options *options;
 
    struct ac_arg descriptor_sets[MAX_SETS];
+   /* User data 0/1. GFX: descriptor list, Compute: scratch BO */
    struct ac_arg ring_offsets;
+   /* User data 2/3. same as the descriptor list above but for task shaders. */
+   struct ac_arg task_ring_offsets;
 
    /* Streamout */
    struct ac_arg streamout_buffers;
 
+   /* NGG */
+   struct ac_arg ngg_query_state;
+
    /* NGG GS */
-   struct ac_arg ngg_gs_state;
    struct ac_arg ngg_culling_settings;
    struct ac_arg ngg_viewport_scale[2];
    struct ac_arg ngg_viewport_translate[2];
 
+   /* Task shaders */
+   struct ac_arg task_ib_addr;
+   struct ac_arg task_ib_stride;
+
+   /* Fragment shaders */
+   struct ac_arg ps_epilog_pc;
+
+   struct ac_arg prolog_inputs;
+   struct ac_arg vs_inputs[MAX_VERTEX_ATTRIBS];
+
+   /* PS epilogs */
+   struct ac_arg ps_epilog_inputs[MAX_RTS];
+
+   struct radv_userdata_locations user_sgprs_locs;
+   unsigned num_user_sgprs;
+
+   bool explicit_scratch_args;
+   bool remap_spi_ps_input;
+   bool load_grid_size_from_user_sgpr;
    bool is_gs_copy_shader;
    bool is_trap_handler_shader;
 };
@@ -55,5 +81,15 @@ radv_shader_args_from_ac(struct ac_shader_args *args)
    return container_of(args, struct radv_shader_args, ac);
 }
 
-void radv_declare_shader_args(struct radv_shader_args *args, gl_shader_stage stage,
-                              bool has_previous_stage, gl_shader_stage previous_stage);
+struct radv_pipeline_key;
+struct radv_shader_info;
+
+void radv_declare_shader_args(enum amd_gfx_level gfx_level, const struct radv_pipeline_key *key,
+                              const struct radv_shader_info *info, gl_shader_stage stage,
+                              bool has_previous_stage, gl_shader_stage previous_stage,
+                              struct radv_shader_args *args);
+
+void radv_declare_ps_epilog_args(enum amd_gfx_level gfx_level, const struct radv_ps_epilog_key *key,
+                                 struct radv_shader_args *args);
+
+#endif
