@@ -81,23 +81,25 @@ agx_is_fmov(agx_instr *def)
 static agx_index
 agx_compose_float_src(agx_index to, agx_index from)
 {
-   if (to.abs)
+   if (to.abs) {
       from.neg = false;
+      from.abs = true;
+   }
 
-   from.abs |= to.abs;
-   from.neg |= to.neg;
+   from.neg ^= to.neg;
 
    return from;
 }
 
 static void
-agx_optimizer_fmov(agx_instr **defs, agx_instr *ins, unsigned srcs)
+agx_optimizer_fmov(agx_instr **defs, agx_instr *ins)
 {
-   for (unsigned s = 0; s < srcs; ++s) {
+   agx_foreach_src(ins, s) {
       agx_index src = ins->src[s];
       if (src.type != AGX_INDEX_NORMAL) continue;
       
       agx_instr *def = defs[src.value];
+      if (def == NULL) continue; /* happens for phis in loops */
       if (!agx_is_fmov(def)) continue;
       if (def->saturate) continue;
 
@@ -151,6 +153,30 @@ agx_optimizer_fmov_rev(agx_instr *I, agx_instr *use)
    return true;
 }
 
+static void
+agx_optimizer_copyprop(agx_instr **defs, agx_instr *I)
+{
+   agx_foreach_src(I, s) {
+      agx_index src = I->src[s];
+      if (src.type != AGX_INDEX_NORMAL) continue;
+
+      agx_instr *def = defs[src.value];
+      if (def == NULL) continue; /* happens for phis in loops */
+      if (def->op != AGX_OPCODE_MOV) continue;
+
+      /* At the moment, not all instructions support size conversions. Notably
+       * RA pseudo instructions don't handle size conversions. This should be
+       * refined in the future.
+       */
+      if (def->src[0].size != src.size) continue;
+
+      /* Immediate inlining happens elsewhere */
+      if (def->src[0].type == AGX_INDEX_IMMEDIATE) continue;
+
+      I->src[s] = agx_replace_index(src, def->src[0]);
+   }
+}
+
 static void
 agx_optimizer_forward(agx_context *ctx)
 {
@@ -159,14 +185,17 @@ agx_optimizer_forward(agx_context *ctx)
    agx_foreach_instr_global(ctx, I) {
       struct agx_opcode_info info = agx_opcodes_info[I->op];
 
-      for (unsigned d = 0; d < info.nr_dests; ++d) {
+      agx_foreach_dest(I, d) {
          if (I->dest[d].type == AGX_INDEX_NORMAL)
             defs[I->dest[d].value] = I;
       }
 
+      /* Optimize moves */
+      agx_optimizer_copyprop(defs, I);
+
       /* Propagate fmov down */
       if (info.is_float)
-         agx_optimizer_fmov(defs, I, info.nr_srcs);
+         agx_optimizer_fmov(defs, I);
 
       /* Inline immediates if we can. TODO: systematic */
       if (I->op != AGX_OPCODE_ST_VARY && I->op != AGX_OPCODE_ST_TILE && I->op != AGX_OPCODE_P_EXTRACT && I->op != AGX_OPCODE_P_COMBINE)
