@@ -1,5 +1,5 @@
 /*
- * Copyright © 2019 Raspberry Pi
+ * Copyright © 2019 Raspberry Pi Ltd
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -132,20 +132,11 @@ bo_free(struct v3dv_device *device,
       return true;
 
    assert(p_atomic_read(&bo->refcnt) == 0);
-
-   if (bo->map)
-      v3dv_bo_unmap(device, bo);
-
-   /* Our BO structs are stored in a sparse array in the physical device,
-    * so we don't want to free the BO pointer, instead we want to reset it
-    * to 0, to signal that array entry as being free.
-    */
-   uint32_t handle = bo->handle;
-   memset(bo, 0, sizeof(*bo));
+   assert(bo->map == NULL);
 
    struct drm_gem_close c;
    memset(&c, 0, sizeof(c));
-   c.handle = handle;
+   c.handle = bo->handle;
    int ret = v3dv_ioctl(device->pdevice->render_fd, DRM_IOCTL_GEM_CLOSE, &c);
    if (ret != 0)
       fprintf(stderr, "close object %d: %s\n", bo->handle, strerror(errno));
@@ -161,6 +152,12 @@ bo_free(struct v3dv_device *device,
       bo_dump_stats(device);
    }
 
+   /* Our BO structs are stored in a sparse array in the physical device,
+    * so we don't want to free the BO pointer, instead we want to reset it
+    * to 0, to signal that array entry as being free.
+    */
+   memset(bo, 0, sizeof(*bo));
+
    return ret == 0;
 }
 
@@ -460,6 +457,9 @@ v3dv_bo_free(struct v3dv_device *device,
    if (!p_atomic_dec_zero(&bo->refcnt))
       return true;
 
+   if (bo->map)
+      v3dv_bo_unmap(device, bo);
+
    struct timespec time;
    struct v3dv_bo_cache *cache = &device->bo_cache;
    uint32_t page_index = bo->size / 4096 - 1;
