@@ -43,7 +43,7 @@ descriptor_bo_map(struct v3dv_device *device,
     */
    uint32_t bo_size = v3dv_X(device, descriptor_bo_size)(binding_layout->type);
    assert(bo_size > 0 ||
-          binding_layout->type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT);
+          binding_layout->type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK);
 
    return set->pool->bo->map +
       set->base_offset + binding_layout->descriptor_offset +
@@ -132,7 +132,7 @@ v3dv_descriptor_map_get_descriptor_bo(struct v3dv_device *device,
    const struct v3dv_descriptor_set_binding_layout *binding_layout =
       &set->layout->binding[binding_number];
 
-   assert(binding_layout->type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT ||
+   assert(binding_layout->type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK ||
           v3dv_X(device, descriptor_bo_size)(binding_layout->type) > 0);
    if (out_type)
       *out_type = binding_layout->type;
@@ -284,6 +284,36 @@ v3dv_descriptor_map_get_texture_shader_state(struct v3dv_device *device,
    return reloc;
 }
 
+#define SHA1_UPDATE_VALUE(ctx, x) _mesa_sha1_update(ctx, &(x), sizeof(x));
+
+static void
+sha1_update_descriptor_set_binding_layout(struct mesa_sha1 *ctx,
+                                          const struct v3dv_descriptor_set_binding_layout *layout)
+{
+   SHA1_UPDATE_VALUE(ctx, layout->type);
+   SHA1_UPDATE_VALUE(ctx, layout->array_size);
+   SHA1_UPDATE_VALUE(ctx, layout->descriptor_index);
+   SHA1_UPDATE_VALUE(ctx, layout->dynamic_offset_count);
+   SHA1_UPDATE_VALUE(ctx, layout->dynamic_offset_index);
+   SHA1_UPDATE_VALUE(ctx, layout->descriptor_offset);
+   SHA1_UPDATE_VALUE(ctx, layout->immutable_samplers_offset);
+}
+
+static void
+sha1_update_descriptor_set_layout(struct mesa_sha1 *ctx,
+                                  const struct v3dv_descriptor_set_layout *layout)
+{
+   SHA1_UPDATE_VALUE(ctx, layout->flags);
+   SHA1_UPDATE_VALUE(ctx, layout->binding_count);
+   SHA1_UPDATE_VALUE(ctx, layout->shader_stages);
+   SHA1_UPDATE_VALUE(ctx, layout->descriptor_count);
+   SHA1_UPDATE_VALUE(ctx, layout->dynamic_offset_count);
+
+   for (uint16_t i = 0; i < layout->binding_count; i++)
+      sha1_update_descriptor_set_binding_layout(ctx, &layout->binding[i]);
+}
+
+
 /*
  * As anv and tu already points:
  *
@@ -336,6 +366,16 @@ v3dv_CreatePipelineLayout(VkDevice _device,
 
    layout->dynamic_offset_count = dynamic_offset_count;
 
+   struct mesa_sha1 ctx;
+   _mesa_sha1_init(&ctx);
+   for (unsigned s = 0; s < layout->num_sets; s++) {
+      sha1_update_descriptor_set_layout(&ctx, layout->set[s].layout);
+      _mesa_sha1_update(&ctx, &layout->set[s].dynamic_offset_start,
+                        sizeof(layout->set[s].dynamic_offset_start));
+   }
+   _mesa_sha1_update(&ctx, &layout->num_sets, sizeof(layout->num_sets));
+   _mesa_sha1_final(&ctx, layout->sha1);
+
    *pPipelineLayout = v3dv_pipeline_layout_to_handle(layout);
 
    return VK_SUCCESS;
@@ -381,7 +421,6 @@ v3dv_CreateDescriptorPool(VkDevice _device,
       vk_find_struct_const(pCreateInfo->pNext,
                            DESCRIPTOR_POOL_INLINE_UNIFORM_BLOCK_CREATE_INFO);
 
-   assert(pCreateInfo->poolSizeCount > 0);
    for (unsigned i = 0; i < pCreateInfo->poolSizeCount; ++i) {
       /* Verify supported descriptor type */
       switch(pCreateInfo->pPoolSizes[i].type) {
@@ -396,7 +435,7 @@ v3dv_CreateDescriptorPool(VkDevice _device,
       case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE:
       case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
       case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
-      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT:
+      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
          break;
       default:
          unreachable("Unimplemented descriptor type");
@@ -405,7 +444,7 @@ v3dv_CreateDescriptorPool(VkDevice _device,
 
       assert(pCreateInfo->pPoolSizes[i].descriptorCount > 0);
       if (pCreateInfo->pPoolSizes[i].type ==
-          VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT) {
+          VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK) {
          /* Inline uniform blocks are specified to use the descriptor array
           * size as the size in bytes of the block.
           */
@@ -465,6 +504,8 @@ v3dv_CreateDescriptorPool(VkDevice _device,
       pool->bo = NULL;
    }
 
+   list_inithead(&pool->set_list);
+
    *pDescriptorPool = v3dv_descriptor_pool_to_handle(pool);
 
    return VK_SUCCESS;
@@ -482,8 +523,6 @@ descriptor_set_destroy(struct v3dv_device *device,
 {
    assert(!pool->host_memory_base);
 
-   v3dv_descriptor_set_layout_unref(device, set->layout);
-
    if (free_bo && !pool->host_memory_base) {
       for (uint32_t i = 0; i < pool->entry_count; i++) {
          if (pool->entries[i].set == set) {
@@ -508,6 +547,11 @@ v3dv_DestroyDescriptorPool(VkDevice _device,
    if (!pool)
       return;
 
+   list_for_each_entry_safe(struct v3dv_descriptor_set, set,
+                            &pool->set_list, pool_link) {
+      v3dv_descriptor_set_layout_unref(device, set->layout);
+   }
+
    if (!pool->host_memory_base) {
       for(int i = 0; i < pool->entry_count; ++i) {
          descriptor_set_destroy(device, pool, pool->entries[i].set, false);
@@ -530,6 +574,12 @@ v3dv_ResetDescriptorPool(VkDevice _device,
    V3DV_FROM_HANDLE(v3dv_device, device, _device);
    V3DV_FROM_HANDLE(v3dv_descriptor_pool, pool, descriptorPool);
 
+   list_for_each_entry_safe(struct v3dv_descriptor_set, set,
+                            &pool->set_list, pool_link) {
+      v3dv_descriptor_set_layout_unref(device, set->layout);
+   }
+   list_inithead(&pool->set_list);
+
    if (!pool->host_memory_base) {
       for(int i = 0; i < pool->entry_count; ++i) {
          descriptor_set_destroy(device, pool, pool->entries[i].set, false);
@@ -655,7 +705,7 @@ v3dv_CreateDescriptorSetLayout(VkDevice _device,
       case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE:
       case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
       case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
-      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT:
+      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
          /* Nothing here, just to keep the descriptor type filtering below */
          break;
       default:
@@ -683,7 +733,7 @@ v3dv_CreateDescriptorSetLayout(VkDevice _device,
 
       set_layout->shader_stages |= binding->stageFlags;
 
-      if (binding->descriptorType != VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT) {
+      if (binding->descriptorType != VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK) {
          dynamic_offset_count += binding->descriptorCount *
             set_layout->binding[binding_number].dynamic_offset_count;
 
@@ -859,6 +909,8 @@ descriptor_set_create(struct v3dv_device *device,
    }
 
    v3dv_descriptor_set_layout_ref(layout);
+   list_addtail(&set->pool_link, &pool->set_list);
+
    *out_set = set;
 
    return VK_SUCCESS;
@@ -909,8 +961,13 @@ v3dv_FreeDescriptorSets(VkDevice _device,
 
    for (uint32_t i = 0; i < count; i++) {
       V3DV_FROM_HANDLE(v3dv_descriptor_set, set, pDescriptorSets[i]);
-      if (set && !pool->host_memory_base)
-         descriptor_set_destroy(device, pool, set, true);
+
+      if (set) {
+         v3dv_descriptor_set_layout_unref(device, set->layout);
+         list_del(&set->pool_link);
+         if (!pool->host_memory_base)
+            descriptor_set_destroy(device, pool, set, true);
+      }
    }
 
    return VK_SUCCESS;
@@ -1018,8 +1075,8 @@ write_inline_uniform_descriptor(struct v3dv_device *device,
                                 size_t offset,
                                 size_t size)
 {
-   assert(binding_layout->type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT);
-   descriptor->type = VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT;
+   assert(binding_layout->type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK);
+   descriptor->type = VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK;
    descriptor->buffer = NULL;
 
    void *desc_map = descriptor_bo_map(device, set, binding_layout, 0);
@@ -1058,7 +1115,7 @@ v3dv_UpdateDescriptorSets(VkDevice  _device,
        * to specify the size (in bytes) of the update.
        */
       uint32_t descriptor_count;
-      if (writeset->descriptorType != VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT) {
+      if (writeset->descriptorType != VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK) {
          descriptor += writeset->dstArrayElement;
          descriptor_count = writeset->descriptorCount;
       } else {
@@ -1120,7 +1177,7 @@ v3dv_UpdateDescriptorSets(VkDevice  _device,
                                          writeset->dstArrayElement + j);
             break;
          }
-         case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT: {
+         case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK: {
             const VkWriteDescriptorSetInlineUniformBlock *inline_write =
                vk_find_struct_const(writeset->pNext,
                                     WRITE_DESCRIPTOR_SET_INLINE_UNIFORM_BLOCK);
@@ -1365,7 +1422,7 @@ v3dv_UpdateDescriptorSetWithTemplate(
          }
          break;
 
-      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT: {
+      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK: {
          write_inline_uniform_descriptor(device, descriptor, set,
                                          binding_layout,
                                          pData + entry->offset,
