@@ -224,7 +224,7 @@ get_deref_node(nir_deref_instr *deref, struct lower_variables_state *state)
    /* This pass only works on local variables.  Just ignore any derefs with
     * a non-local mode.
     */
-   if (deref->mode != nir_var_function_temp)
+   if (!nir_deref_mode_must_be(deref, nir_var_function_temp))
       return NULL;
 
    struct deref_node *node = get_deref_node_recur(deref, state);
@@ -460,7 +460,7 @@ register_variable_uses(nir_function_impl *impl,
             nir_deref_instr *deref = nir_instr_as_deref(instr);
 
             if (deref->deref_type == nir_deref_type_var &&
-                nir_deref_instr_has_complex_use(deref))
+                nir_deref_instr_has_complex_use(deref, 0))
                register_complex_use(deref, state);
 
             break;
@@ -555,7 +555,7 @@ rename_variables(struct lower_variables_state *state)
          switch (intrin->intrinsic) {
          case nir_intrinsic_load_deref: {
             nir_deref_instr *deref = nir_src_as_deref(intrin->src[0]);
-            if (deref->mode != nir_var_function_temp)
+            if (!nir_deref_mode_must_be(deref, nir_var_function_temp))
                continue;
 
             struct deref_node *node = get_deref_node(deref, state);
@@ -577,7 +577,7 @@ rename_variables(struct lower_variables_state *state)
                nir_instr_remove(&intrin->instr);
 
                nir_ssa_def_rewrite_uses(&intrin->dest.ssa,
-                                        nir_src_for_ssa(&undef->def));
+                                        &undef->def);
                continue;
             }
 
@@ -602,13 +602,13 @@ rename_variables(struct lower_variables_state *state)
             nir_instr_remove(&intrin->instr);
 
             nir_ssa_def_rewrite_uses(&intrin->dest.ssa,
-                                     nir_src_for_ssa(&mov->dest.dest.ssa));
+                                     &mov->dest.dest.ssa);
             break;
          }
 
          case nir_intrinsic_store_deref: {
             nir_deref_instr *deref = nir_src_as_deref(intrin->src[0]);
-            if (deref->mode != nir_var_function_temp)
+            if (!nir_deref_mode_must_be(deref, nir_var_function_temp))
                continue;
 
             struct deref_node *node = get_deref_node(deref, state);
@@ -653,15 +653,15 @@ rename_variables(struct lower_variables_state *state)
                 * written values with the existing contents of unwritten
                 * channels, creating a new SSA value for the whole vector.
                 */
-               nir_ssa_def *srcs[NIR_MAX_VEC_COMPONENTS];
+               nir_ssa_scalar srcs[NIR_MAX_VEC_COMPONENTS];
                for (unsigned i = 0; i < intrin->num_components; i++) {
                   if (wrmask & (1 << i)) {
-                     srcs[i] = nir_channel(&b, value, i);
+                     srcs[i] = nir_get_ssa_scalar(value, i);
                   } else {
-                     srcs[i] = nir_channel(&b, old_def, i);
+                     srcs[i] = nir_get_ssa_scalar(old_def, i);
                   }
                }
-               new_def = nir_vec(&b, srcs, intrin->num_components);
+               new_def = nir_vec_scalars(&b, srcs, intrin->num_components);
             }
 
             assert(new_def->num_components == intrin->num_components);
@@ -750,9 +750,7 @@ nir_lower_vars_to_ssa_impl(nir_function_impl *impl)
    }
 
    if (!progress) {
-#ifndef NDEBUG
-      impl->valid_metadata &= ~nir_metadata_not_properly_reset;
-#endif
+      nir_metadata_preserve(impl, nir_metadata_all);
       return false;
    }
 
