@@ -112,6 +112,7 @@ struct u_vbuf_elements {
     * its vertex data must be translated to native_format[i]. */
    enum pipe_format native_format[PIPE_MAX_ATTRIBS];
    unsigned native_format_size[PIPE_MAX_ATTRIBS];
+   unsigned component_size[PIPE_MAX_ATTRIBS];
 
    /* Which buffers are used by the vertex element state. */
    uint32_t used_vb_mask;
@@ -127,6 +128,7 @@ struct u_vbuf_elements {
    /* Which buffer has at least one vertex element referencing it
     * compatible. */
    uint32_t compatible_vb_mask_any;
+   uint32_t vb_align_mask[2]; //which buffers require 2/4 byte alignments
    /* Which buffer has all vertex elements referencing it compatible. */
    uint32_t compatible_vb_mask_all;
 
@@ -163,6 +165,8 @@ struct u_vbuf {
    struct pipe_vertex_buffer vertex_buffer[PIPE_MAX_ATTRIBS];
    uint32_t enabled_vb_mask;
 
+   uint32_t unaligned_vb_mask[2]; //16/32bit
+
    /* Vertex buffers for the driver.
     * There are usually no user buffers. */
    struct pipe_vertex_buffer real_vertex_buffer[PIPE_MAX_ATTRIBS];
@@ -303,6 +307,11 @@ void u_vbuf_get_caps(struct pipe_screen *screen, struct u_vbuf_caps *caps,
    caps->velem_src_offset_unaligned =
       !screen->get_param(screen,
                          PIPE_CAP_VERTEX_ELEMENT_SRC_OFFSET_4BYTE_ALIGNED_ONLY);
+   caps->attrib_component_unaligned =
+      !screen->get_param(screen,
+                         PIPE_CAP_VERTEX_ATTRIB_ELEMENT_ALIGNED_ONLY);
+   assert(caps->attrib_component_unaligned ||
+          (caps->velem_src_offset_unaligned && caps->buffer_stride_unaligned && caps->buffer_offset_unaligned));
    caps->user_vertex_buffers =
       screen->get_param(screen, PIPE_CAP_USER_VERTEX_BUFFERS);
    caps->max_vertex_buffers =
@@ -330,6 +339,7 @@ void u_vbuf_get_caps(struct pipe_screen *screen, struct u_vbuf_caps *caps,
 
    if (!caps->buffer_offset_unaligned ||
        !caps->buffer_stride_unaligned ||
+       !caps->attrib_component_unaligned ||
        !caps->velem_src_offset_unaligned)
       caps->fallback_always = true;
 
@@ -491,9 +501,10 @@ u_vbuf_translate_buffers(struct u_vbuf *mgr, struct translate_key *key,
              * themselves, meaning that if stride < element_size, the mapped size will
              * be too small and conversion will overrun the map buffer
              *
-             * instead, add the size of the largest possible attribute to ensure the map is large enough
+             * instead, add the size of the largest possible attribute to the final attribute's offset
+             * in order to ensure the map is large enough
              */
-            unsigned last_offset = offset + size - vb->stride;
+            unsigned last_offset = size - vb->stride;
             size = MAX2(size, last_offset + sizeof(double)*4);
          }
 
@@ -668,13 +679,14 @@ u_vbuf_translate_begin(struct u_vbuf *mgr,
                        const struct pipe_draw_info *info,
                        const struct pipe_draw_start_count_bias *draw,
                        int start_vertex, unsigned num_vertices,
-                       int min_index, boolean unroll_indices)
+                       int min_index, boolean unroll_indices,
+                       uint32_t misaligned)
 {
    unsigned mask[VB_NUM] = {0};
    struct translate_key key[VB_NUM];
    unsigned elem_index[VB_NUM][PIPE_MAX_ATTRIBS]; /* ... into key.elements */
    unsigned i, type;
-   const unsigned incompatible_vb_mask = mgr->incompatible_vb_mask &
+   const unsigned incompatible_vb_mask = (misaligned | mgr->incompatible_vb_mask) &
                                          mgr->ve->used_vb_mask;
 
    const int start[VB_NUM] = {
@@ -726,6 +738,7 @@ u_vbuf_translate_begin(struct u_vbuf *mgr,
       return FALSE;
    }
 
+   unsigned min_alignment[VB_NUM] = {0};
    /* Initialize the translate keys. */
    for (i = 0; i < mgr->ve->count; i++) {
       struct translate_key *k;
@@ -764,15 +777,25 @@ u_vbuf_translate_begin(struct u_vbuf *mgr,
       te->input_offset = mgr->ve->ve[i].src_offset;
       te->output_format = output_format;
       te->output_offset = k->output_stride;
+      unsigned adjustment = 0;
+      if (!mgr->caps.attrib_component_unaligned &&
+          te->output_offset % mgr->ve->component_size[i] != 0) {
+         unsigned aligned = align(te->output_offset, mgr->ve->component_size[i]);
+         adjustment = aligned - te->output_offset;
+         te->output_offset = aligned;
+      }
 
-      k->output_stride += mgr->ve->native_format_size[i];
+      k->output_stride += mgr->ve->native_format_size[i] + adjustment;
       k->nr_elements++;
+      min_alignment[type] = MAX2(min_alignment[type], mgr->ve->component_size[i]);
    }
 
    /* Translate buffers. */
    for (type = 0; type < VB_NUM; type++) {
       if (key[type].nr_elements) {
          enum pipe_error err;
+         if (!mgr->caps.attrib_component_unaligned)
+            key[type].output_stride = align(key[type].output_stride, min_alignment[type]);
          err = u_vbuf_translate_buffers(mgr, &key[type], info, draw,
                                         mask[type], mgr->fallback_vbs[type],
                                         start[type], num[type], min_index,
@@ -880,13 +903,27 @@ u_vbuf_create_vertex_elements(struct u_vbuf *mgr, unsigned count,
       ve->native_format_size[i] =
             util_format_get_blocksize(ve->native_format[i]);
 
+      const struct util_format_description *desc = util_format_description(format);
+      bool is_packed = false;
+      for (unsigned c = 0; c < desc->nr_channels; c++)
+         is_packed |= desc->channel[c].size != desc->channel[0].size || desc->channel[c].size % 8 != 0;
+      unsigned component_size = is_packed ?
+                                ve->native_format_size[i] : (ve->native_format_size[i] / desc->nr_channels);
+      ve->component_size[i] = component_size;
+
       if (ve->ve[i].src_format != format ||
           (!mgr->caps.velem_src_offset_unaligned &&
-           ve->ve[i].src_offset % 4 != 0)) {
+           ve->ve[i].src_offset % 4 != 0) ||
+          (!mgr->caps.attrib_component_unaligned &&
+           ve->ve[i].src_offset % component_size != 0)) {
          ve->incompatible_elem_mask |= 1 << i;
          ve->incompatible_vb_mask_any |= vb_index_bit;
       } else {
          ve->compatible_vb_mask_any |= vb_index_bit;
+         if (component_size == 2)
+            ve->vb_align_mask[0] |= vb_index_bit;
+         else if (component_size == 4)
+            ve->vb_align_mask[1] |= vb_index_bit;
       }
    }
 
@@ -950,21 +987,25 @@ void u_vbuf_set_vertex_buffers(struct u_vbuf *mgr,
    uint32_t incompatible_vb_mask = 0;
    /* which buffers have a non-zero stride */
    uint32_t nonzero_stride_vb_mask = 0;
-   const uint32_t mask =
+   /* which buffers are unaligned to 2/4 bytes */
+   uint32_t unaligned_vb_mask[2] = {0};
+   uint32_t mask =
       ~(((1ull << (count + unbind_num_trailing_slots)) - 1) << start_slot);
 
-   /* Zero out the bits we are going to rewrite completely. */
-   mgr->user_vb_mask &= mask;
-   mgr->incompatible_vb_mask &= mask;
-   mgr->nonzero_stride_vb_mask &= mask;
-   mgr->enabled_vb_mask &= mask;
-
    if (!bufs) {
       struct pipe_context *pipe = mgr->pipe;
       /* Unbind. */
       unsigned total_count = count + unbind_num_trailing_slots;
       mgr->dirty_real_vb_mask &= mask;
 
+      /* Zero out the bits we are going to rewrite completely. */
+      mgr->user_vb_mask &= mask;
+      mgr->incompatible_vb_mask &= mask;
+      mgr->nonzero_stride_vb_mask &= mask;
+      mgr->enabled_vb_mask &= mask;
+      mgr->unaligned_vb_mask[0] &= mask;
+      mgr->unaligned_vb_mask[1] &= mask;
+
       for (i = 0; i < total_count; i++) {
          unsigned dst_index = start_slot + i;
 
@@ -989,6 +1030,21 @@ void u_vbuf_set_vertex_buffers(struct u_vbuf *mgr,
          continue;
       }
 
+      bool not_user = !vb->is_user_buffer && vb->is_user_buffer == orig_vb->is_user_buffer;
+      /* struct isn't tightly packed: do not use memcmp */
+      if (not_user && orig_vb->stride == vb->stride &&
+          orig_vb->buffer_offset == vb->buffer_offset && orig_vb->buffer.resource == vb->buffer.resource) {
+         mask |= BITFIELD_BIT(dst_index);
+         if (take_ownership) {
+             pipe_vertex_buffer_unreference(orig_vb);
+             /* the pointer was unset in the line above, so copy it back */
+             orig_vb->buffer.resource = vb->buffer.resource;
+         }
+         if (mask == UINT32_MAX)
+            return;
+         continue;
+      }
+
       if (take_ownership) {
          pipe_vertex_buffer_unreference(orig_vb);
          memcpy(orig_vb, vb, sizeof(*vb));
@@ -1011,6 +1067,13 @@ void u_vbuf_set_vertex_buffers(struct u_vbuf *mgr,
          continue;
       }
 
+      if (!mgr->caps.attrib_component_unaligned) {
+         if (vb->buffer_offset % 2 != 0 || vb->stride % 2 != 0)
+            unaligned_vb_mask[0] |= BITFIELD_BIT(dst_index);
+         if (vb->buffer_offset % 4 != 0 || vb->stride % 4 != 0)
+            unaligned_vb_mask[1] |= BITFIELD_BIT(dst_index);
+      }
+
       if (!mgr->caps.user_vertex_buffers && vb->is_user_buffer) {
          user_vb_mask |= 1 << dst_index;
          real_vb->buffer_offset = vb->buffer_offset;
@@ -1030,10 +1093,21 @@ void u_vbuf_set_vertex_buffers(struct u_vbuf *mgr,
       pipe_vertex_buffer_unreference(&mgr->real_vertex_buffer[dst_index]);
    }
 
+
+   /* Zero out the bits we are going to rewrite completely. */
+   mgr->user_vb_mask &= mask;
+   mgr->incompatible_vb_mask &= mask;
+   mgr->nonzero_stride_vb_mask &= mask;
+   mgr->enabled_vb_mask &= mask;
+   mgr->unaligned_vb_mask[0] &= mask;
+   mgr->unaligned_vb_mask[1] &= mask;
+
    mgr->user_vb_mask |= user_vb_mask;
    mgr->incompatible_vb_mask |= incompatible_vb_mask;
    mgr->nonzero_stride_vb_mask |= nonzero_stride_vb_mask;
    mgr->enabled_vb_mask |= enabled_vb_mask;
+   mgr->unaligned_vb_mask[0] |= unaligned_vb_mask[0];
+   mgr->unaligned_vb_mask[1] |= unaligned_vb_mask[1];
 
    /* All changed buffers are marked as dirty, even the NULL ones,
     * which will cause the NULL buffers to be unbound in the driver later. */
@@ -1183,7 +1257,7 @@ u_vbuf_upload_buffers(struct u_vbuf *mgr,
    return PIPE_OK;
 }
 
-static boolean u_vbuf_need_minmax_index(const struct u_vbuf *mgr)
+static boolean u_vbuf_need_minmax_index(const struct u_vbuf *mgr, uint32_t misaligned)
 {
    /* See if there are any per-vertex attribs which will be uploaded or
     * translated. Use bitmasks to get the info instead of looping over vertex
@@ -1191,12 +1265,13 @@ static boolean u_vbuf_need_minmax_index(const struct u_vbuf *mgr)
    return (mgr->ve->used_vb_mask &
            ((mgr->user_vb_mask |
              mgr->incompatible_vb_mask |
+             misaligned |
              mgr->ve->incompatible_vb_mask_any) &
             mgr->ve->noninstance_vb_mask_any &
             mgr->nonzero_stride_vb_mask)) != 0;
 }
 
-static boolean u_vbuf_mapping_vertex_buffer_blocks(const struct u_vbuf *mgr)
+static boolean u_vbuf_mapping_vertex_buffer_blocks(const struct u_vbuf *mgr, uint32_t misaligned)
 {
    /* Return true if there are hw buffers which don't need to be translated.
     *
@@ -1205,6 +1280,7 @@ static boolean u_vbuf_mapping_vertex_buffer_blocks(const struct u_vbuf *mgr)
    return (mgr->ve->used_vb_mask &
            (~mgr->user_vb_mask &
             ~mgr->incompatible_vb_mask &
+            ~misaligned &
             mgr->ve->compatible_vb_mask_all &
             mgr->ve->noninstance_vb_mask_any &
             mgr->nonzero_stride_vb_mask)) != 0;
@@ -1374,14 +1450,15 @@ u_vbuf_split_indexed_multidraw(struct u_vbuf *mgr, struct pipe_draw_info *info,
       draw.index_bias = indirect_data[offset + 3];
       info->start_instance = indirect_data[offset + 4];
 
-      u_vbuf_draw_vbo(mgr, info, drawid_offset, NULL, draw);
+      u_vbuf_draw_vbo(mgr, info, drawid_offset, NULL, &draw, 1);
    }
 }
 
 void u_vbuf_draw_vbo(struct u_vbuf *mgr, const struct pipe_draw_info *info,
                      unsigned drawid_offset,
                      const struct pipe_draw_indirect_info *indirect,
-                     const struct pipe_draw_start_count_bias draw)
+                     const struct pipe_draw_start_count_bias *draws,
+                     unsigned num_draws)
 {
    struct pipe_context *pipe = mgr->pipe;
    int start_vertex;
@@ -1390,12 +1467,19 @@ void u_vbuf_draw_vbo(struct u_vbuf *mgr, const struct pipe_draw_info *info,
    boolean unroll_indices = FALSE;
    const uint32_t used_vb_mask = mgr->ve->used_vb_mask;
    uint32_t user_vb_mask = mgr->user_vb_mask & used_vb_mask;
-   const uint32_t incompatible_vb_mask =
-      mgr->incompatible_vb_mask & used_vb_mask;
    struct pipe_draw_info new_info;
    struct pipe_draw_start_count_bias new_draw;
    unsigned fixed_restart_index = info->index_size ? util_prim_restart_index_from_size(info->index_size) : 0;
 
+   uint32_t misaligned = 0;
+   if (!mgr->caps.attrib_component_unaligned) {
+      for (unsigned i = 0; i < ARRAY_SIZE(mgr->unaligned_vb_mask); i++) {
+         misaligned |= mgr->ve->vb_align_mask[i] & mgr->unaligned_vb_mask[i];
+      }
+   }
+   const uint32_t incompatible_vb_mask =
+      (mgr->incompatible_vb_mask | misaligned) & used_vb_mask;
+
    /* Normal draw. No fallback and no user buffers. */
    if (!incompatible_vb_mask &&
        !mgr->ve->incompatible_elem_mask &&
@@ -1412,288 +1496,298 @@ void u_vbuf_draw_vbo(struct u_vbuf *mgr, const struct pipe_draw_info *info,
          u_vbuf_set_driver_vertex_buffers(mgr);
       }
 
-      pipe->draw_vbo(pipe, info, drawid_offset, indirect, &draw, 1);
+      pipe->draw_vbo(pipe, info, drawid_offset, indirect, draws, num_draws);
       return;
    }
 
+   /* Increase refcount to be able to use take_index_buffer_ownership with
+    * all draws.
+    */
+   if (num_draws > 1 && info->take_index_buffer_ownership)
+      p_atomic_add(&info->index.resource->reference.count, num_draws - 1);
    new_info = *info;
-   new_draw = draw;
 
-   /* Handle indirect (multi)draws. */
-   if (indirect && indirect->buffer) {
-      unsigned draw_count = 0;
+   for (unsigned d = 0; d < num_draws; d++) {
+      new_draw = draws[d];
+      if (info->increment_draw_id)
+         drawid_offset++;
 
-      /* Get the number of draws. */
-      if (indirect->indirect_draw_count) {
-         pipe_buffer_read(pipe, indirect->indirect_draw_count,
-                          indirect->indirect_draw_count_offset,
-                          4, &draw_count);
-      } else {
-         draw_count = indirect->draw_count;
-      }
+      /* Handle indirect (multi)draws. */
+      if (indirect && indirect->buffer) {
+         unsigned draw_count = 0;
 
-      if (!draw_count)
-         goto cleanup;
-
-      unsigned data_size = (draw_count - 1) * indirect->stride +
-                           (new_info.index_size ? 20 : 16);
-      unsigned *data = malloc(data_size);
-      if (!data)
-         goto cleanup; /* report an error? */
-
-      /* Read the used buffer range only once, because the read can be
-       * uncached.
-       */
-      pipe_buffer_read(pipe, indirect->buffer, indirect->offset, data_size,
-                       data);
-
-      if (info->index_size) {
-         /* Indexed multidraw. */
-         unsigned index_bias0 = data[3];
-         bool index_bias_same = true;
-
-         /* If we invoke the translate path, we have to split the multidraw. */
-         if (incompatible_vb_mask ||
-             mgr->ve->incompatible_elem_mask) {
-            u_vbuf_split_indexed_multidraw(mgr, &new_info, drawid_offset, data,
-                                           indirect->stride, draw_count);
-            free(data);
-            return;
+         /* Get the number of draws. */
+         if (indirect->indirect_draw_count) {
+            pipe_buffer_read(pipe, indirect->indirect_draw_count,
+                             indirect->indirect_draw_count_offset,
+                             4, &draw_count);
+         } else {
+            draw_count = indirect->draw_count;
          }
 
-         /* See if index_bias is the same for all draws. */
-         for (unsigned i = 1; i < draw_count; i++) {
-            if (data[i * indirect->stride / 4 + 3] != index_bias0) {
-               index_bias_same = false;
-               break;
+         if (!draw_count)
+            goto cleanup;
+
+         unsigned data_size = (draw_count - 1) * indirect->stride +
+                              (new_info.index_size ? 20 : 16);
+         unsigned *data = malloc(data_size);
+         if (!data)
+            goto cleanup; /* report an error? */
+
+         /* Read the used buffer range only once, because the read can be
+          * uncached.
+          */
+         pipe_buffer_read(pipe, indirect->buffer, indirect->offset, data_size,
+                          data);
+
+         if (info->index_size) {
+            /* Indexed multidraw. */
+            unsigned index_bias0 = data[3];
+            bool index_bias_same = true;
+
+            /* If we invoke the translate path, we have to split the multidraw. */
+            if (incompatible_vb_mask ||
+                mgr->ve->incompatible_elem_mask) {
+               u_vbuf_split_indexed_multidraw(mgr, &new_info, drawid_offset, data,
+                                              indirect->stride, draw_count);
+               free(data);
+               return;
             }
-         }
 
-         /* Split the multidraw if index_bias is different. */
-         if (!index_bias_same) {
-            u_vbuf_split_indexed_multidraw(mgr, &new_info, drawid_offset, data,
-                                           indirect->stride, draw_count);
+            /* See if index_bias is the same for all draws. */
+            for (unsigned i = 1; i < draw_count; i++) {
+               if (data[i * indirect->stride / 4 + 3] != index_bias0) {
+                  index_bias_same = false;
+                  break;
+               }
+            }
+
+            /* Split the multidraw if index_bias is different. */
+            if (!index_bias_same) {
+               u_vbuf_split_indexed_multidraw(mgr, &new_info, drawid_offset, data,
+                                              indirect->stride, draw_count);
+               free(data);
+               return;
+            }
+
+            /* If we don't need to use the translate path and index_bias is
+             * the same, we can process the multidraw with the time complexity
+             * equal to 1 draw call (except for the index range computation).
+             * We only need to compute the index range covering all draw calls
+             * of the multidraw.
+             *
+             * The driver will not look at these values because indirect != NULL.
+             * These values determine the user buffer bounds to upload.
+             */
+            new_draw.index_bias = index_bias0;
+            new_info.index_bounds_valid = true;
+            new_info.min_index = ~0u;
+            new_info.max_index = 0;
+            new_info.start_instance = ~0u;
+            unsigned end_instance = 0;
+
+            struct pipe_transfer *transfer = NULL;
+            const uint8_t *indices;
+
+            if (info->has_user_indices) {
+               indices = (uint8_t*)info->index.user;
+            } else {
+               indices = (uint8_t*)pipe_buffer_map(pipe, info->index.resource,
+                                                   PIPE_MAP_READ, &transfer);
+            }
+
+            for (unsigned i = 0; i < draw_count; i++) {
+               unsigned offset = i * indirect->stride / 4;
+               unsigned start = data[offset + 2];
+               unsigned count = data[offset + 0];
+               unsigned start_instance = data[offset + 4];
+               unsigned instance_count = data[offset + 1];
+
+               if (!count || !instance_count)
+                  continue;
+
+               /* Update the ranges of instances. */
+               new_info.start_instance = MIN2(new_info.start_instance,
+                                              start_instance);
+               end_instance = MAX2(end_instance, start_instance + instance_count);
+
+               /* Update the index range. */
+               unsigned min, max;
+               u_vbuf_get_minmax_index_mapped(&new_info, count,
+                                              indices +
+                                              new_info.index_size * start,
+                                              &min, &max);
+
+               new_info.min_index = MIN2(new_info.min_index, min);
+               new_info.max_index = MAX2(new_info.max_index, max);
+            }
             free(data);
-            return;
-         }
 
-         /* If we don't need to use the translate path and index_bias is
-          * the same, we can process the multidraw with the time complexity
-          * equal to 1 draw call (except for the index range computation).
-          * We only need to compute the index range covering all draw calls
-          * of the multidraw.
-          *
-          * The driver will not look at these values because indirect != NULL.
-          * These values determine the user buffer bounds to upload.
-          */
-         new_draw.index_bias = index_bias0;
-         new_info.index_bounds_valid = true;
-         new_info.min_index = ~0u;
-         new_info.max_index = 0;
-         new_info.start_instance = ~0u;
-         unsigned end_instance = 0;
+            if (transfer)
+               pipe_buffer_unmap(pipe, transfer);
 
-         struct pipe_transfer *transfer = NULL;
-         const uint8_t *indices;
+            /* Set the final instance count. */
+            new_info.instance_count = end_instance - new_info.start_instance;
 
-         if (info->has_user_indices) {
-            indices = (uint8_t*)info->index.user;
+            if (new_info.start_instance == ~0u || !new_info.instance_count)
+               goto cleanup;
          } else {
-            indices = (uint8_t*)pipe_buffer_map(pipe, info->index.resource,
-                                                PIPE_MAP_READ, &transfer);
+            /* Non-indexed multidraw.
+             *
+             * Keep the draw call indirect and compute minimums & maximums,
+             * which will determine the user buffer bounds to upload, but
+             * the driver will not look at these values because indirect != NULL.
+             *
+             * This efficiently processes the multidraw with the time complexity
+             * equal to 1 draw call.
+             */
+            new_draw.start = ~0u;
+            new_info.start_instance = ~0u;
+            unsigned end_vertex = 0;
+            unsigned end_instance = 0;
+
+            for (unsigned i = 0; i < draw_count; i++) {
+               unsigned offset = i * indirect->stride / 4;
+               unsigned start = data[offset + 2];
+               unsigned count = data[offset + 0];
+               unsigned start_instance = data[offset + 3];
+               unsigned instance_count = data[offset + 1];
+
+               new_draw.start = MIN2(new_draw.start, start);
+               new_info.start_instance = MIN2(new_info.start_instance,
+                                              start_instance);
+
+               end_vertex = MAX2(end_vertex, start + count);
+               end_instance = MAX2(end_instance, start_instance + instance_count);
+            }
+            free(data);
+
+            /* Set the final counts. */
+            new_draw.count = end_vertex - new_draw.start;
+            new_info.instance_count = end_instance - new_info.start_instance;
+
+            if (new_draw.start == ~0u || !new_draw.count || !new_info.instance_count)
+               goto cleanup;
          }
-
-         for (unsigned i = 0; i < draw_count; i++) {
-            unsigned offset = i * indirect->stride / 4;
-            unsigned start = data[offset + 2];
-            unsigned count = data[offset + 0];
-            unsigned start_instance = data[offset + 4];
-            unsigned instance_count = data[offset + 1];
-
-            if (!count || !instance_count)
-               continue;
-
-            /* Update the ranges of instances. */
-            new_info.start_instance = MIN2(new_info.start_instance,
-                                           start_instance);
-            end_instance = MAX2(end_instance, start_instance + instance_count);
-
-            /* Update the index range. */
-            unsigned min, max;
-            u_vbuf_get_minmax_index_mapped(&new_info, count,
-                                           indices +
-                                           new_info.index_size * start,
-                                           &min, &max);
-
-            new_info.min_index = MIN2(new_info.min_index, min);
-            new_info.max_index = MAX2(new_info.max_index, max);
-         }
-         free(data);
-
-         if (transfer)
-            pipe_buffer_unmap(pipe, transfer);
-
-         /* Set the final instance count. */
-         new_info.instance_count = end_instance - new_info.start_instance;
-
-         if (new_info.start_instance == ~0u || !new_info.instance_count)
-            goto cleanup;
       } else {
-         /* Non-indexed multidraw.
-          *
-          * Keep the draw call indirect and compute minimums & maximums,
-          * which will determine the user buffer bounds to upload, but
-          * the driver will not look at these values because indirect != NULL.
-          *
-          * This efficiently processes the multidraw with the time complexity
-          * equal to 1 draw call.
-          */
-         new_draw.start = ~0u;
-         new_info.start_instance = ~0u;
-         unsigned end_vertex = 0;
-         unsigned end_instance = 0;
-
-         for (unsigned i = 0; i < draw_count; i++) {
-            unsigned offset = i * indirect->stride / 4;
-            unsigned start = data[offset + 2];
-            unsigned count = data[offset + 0];
-            unsigned start_instance = data[offset + 3];
-            unsigned instance_count = data[offset + 1];
-
-            new_draw.start = MIN2(new_draw.start, start);
-            new_info.start_instance = MIN2(new_info.start_instance,
-                                           start_instance);
-
-            end_vertex = MAX2(end_vertex, start + count);
-            end_instance = MAX2(end_instance, start_instance + instance_count);
-         }
-         free(data);
-
-         /* Set the final counts. */
-         new_draw.count = end_vertex - new_draw.start;
-         new_info.instance_count = end_instance - new_info.start_instance;
-
-         if (new_draw.start == ~0u || !new_draw.count || !new_info.instance_count)
+         if ((!indirect && !new_draw.count) || !new_info.instance_count)
             goto cleanup;
       }
-   } else {
-      if ((!indirect && !new_draw.count) || !new_info.instance_count)
-         goto cleanup;
-   }
 
-   if (new_info.index_size) {
-      /* See if anything needs to be done for per-vertex attribs. */
-      if (u_vbuf_need_minmax_index(mgr)) {
-         unsigned max_index;
+      if (new_info.index_size) {
+         /* See if anything needs to be done for per-vertex attribs. */
+         if (u_vbuf_need_minmax_index(mgr, misaligned)) {
+            unsigned max_index;
 
-         if (new_info.index_bounds_valid) {
-            min_index = new_info.min_index;
-            max_index = new_info.max_index;
+            if (new_info.index_bounds_valid) {
+               min_index = new_info.min_index;
+               max_index = new_info.max_index;
+            } else {
+               u_vbuf_get_minmax_index(mgr->pipe, &new_info, &new_draw,
+                                       &min_index, &max_index);
+            }
+
+            assert(min_index <= max_index);
+
+            start_vertex = min_index + new_draw.index_bias;
+            num_vertices = max_index + 1 - min_index;
+
+            /* Primitive restart doesn't work when unrolling indices.
+             * We would have to break this drawing operation into several ones. */
+            /* Use some heuristic to see if unrolling indices improves
+             * performance. */
+            if (!indirect &&
+                !new_info.primitive_restart &&
+                util_is_vbo_upload_ratio_too_large(new_draw.count, num_vertices) &&
+                !u_vbuf_mapping_vertex_buffer_blocks(mgr, misaligned)) {
+               unroll_indices = TRUE;
+               user_vb_mask &= ~(mgr->nonzero_stride_vb_mask &
+                                 mgr->ve->noninstance_vb_mask_any);
+            }
          } else {
-            u_vbuf_get_minmax_index(mgr->pipe, &new_info, &new_draw,
-                                    &min_index, &max_index);
-         }
-
-         assert(min_index <= max_index);
-
-         start_vertex = min_index + new_draw.index_bias;
-         num_vertices = max_index + 1 - min_index;
-
-         /* Primitive restart doesn't work when unrolling indices.
-          * We would have to break this drawing operation into several ones. */
-         /* Use some heuristic to see if unrolling indices improves
-          * performance. */
-         if (!indirect &&
-             !new_info.primitive_restart &&
-             util_is_vbo_upload_ratio_too_large(new_draw.count, num_vertices) &&
-             !u_vbuf_mapping_vertex_buffer_blocks(mgr)) {
-            unroll_indices = TRUE;
-            user_vb_mask &= ~(mgr->nonzero_stride_vb_mask &
-                              mgr->ve->noninstance_vb_mask_any);
+            /* Nothing to do for per-vertex attribs. */
+            start_vertex = 0;
+            num_vertices = 0;
+            min_index = 0;
          }
       } else {
-         /* Nothing to do for per-vertex attribs. */
-         start_vertex = 0;
-         num_vertices = 0;
+         start_vertex = new_draw.start;
+         num_vertices = new_draw.count;
          min_index = 0;
       }
-   } else {
-      start_vertex = new_draw.start;
-      num_vertices = new_draw.count;
-      min_index = 0;
-   }
 
-   /* Translate vertices with non-native layouts or formats. */
-   if (unroll_indices ||
-       incompatible_vb_mask ||
-       mgr->ve->incompatible_elem_mask) {
-      if (!u_vbuf_translate_begin(mgr, &new_info, &new_draw,
-                                  start_vertex, num_vertices,
-                                  min_index, unroll_indices)) {
-         debug_warn_once("u_vbuf_translate_begin() failed");
-         goto cleanup;
+      /* Translate vertices with non-native layouts or formats. */
+      if (unroll_indices ||
+          incompatible_vb_mask ||
+          mgr->ve->incompatible_elem_mask) {
+         if (!u_vbuf_translate_begin(mgr, &new_info, &new_draw,
+                                     start_vertex, num_vertices,
+                                     min_index, unroll_indices, misaligned)) {
+            debug_warn_once("u_vbuf_translate_begin() failed");
+            goto cleanup;
+         }
+
+         if (unroll_indices) {
+            new_info.index_size = 0;
+            new_draw.index_bias = 0;
+            new_info.index_bounds_valid = true;
+            new_info.min_index = 0;
+            new_info.max_index = new_draw.count - 1;
+            new_draw.start = 0;
+         }
+
+         user_vb_mask &= ~(incompatible_vb_mask |
+                           mgr->ve->incompatible_vb_mask_all);
       }
 
+      /* Upload user buffers. */
+      if (user_vb_mask) {
+         if (u_vbuf_upload_buffers(mgr, start_vertex, num_vertices,
+                                   new_info.start_instance,
+                                   new_info.instance_count) != PIPE_OK) {
+            debug_warn_once("u_vbuf_upload_buffers() failed");
+            goto cleanup;
+         }
+
+         mgr->dirty_real_vb_mask |= user_vb_mask;
+      }
+
+      /*
       if (unroll_indices) {
-         new_info.index_size = 0;
-         new_draw.index_bias = 0;
-         new_info.index_bounds_valid = true;
-         new_info.min_index = 0;
-         new_info.max_index = new_draw.count - 1;
-         new_draw.start = 0;
+         printf("unrolling indices: start_vertex = %i, num_vertices = %i\n",
+                start_vertex, num_vertices);
+         util_dump_draw_info(stdout, info);
+         printf("\n");
       }
 
-      user_vb_mask &= ~(incompatible_vb_mask |
-                        mgr->ve->incompatible_vb_mask_all);
-   }
-
-   /* Upload user buffers. */
-   if (user_vb_mask) {
-      if (u_vbuf_upload_buffers(mgr, start_vertex, num_vertices,
-                                new_info.start_instance,
-                                new_info.instance_count) != PIPE_OK) {
-         debug_warn_once("u_vbuf_upload_buffers() failed");
-         goto cleanup;
+      unsigned i;
+      for (i = 0; i < mgr->nr_vertex_buffers; i++) {
+         printf("input %i: ", i);
+         util_dump_vertex_buffer(stdout, mgr->vertex_buffer+i);
+         printf("\n");
       }
+      for (i = 0; i < mgr->nr_real_vertex_buffers; i++) {
+         printf("real %i: ", i);
+         util_dump_vertex_buffer(stdout, mgr->real_vertex_buffer+i);
+         printf("\n");
+      }
+      */
 
-      mgr->dirty_real_vb_mask |= user_vb_mask;
+      u_upload_unmap(pipe->stream_uploader);
+      if (mgr->dirty_real_vb_mask)
+         u_vbuf_set_driver_vertex_buffers(mgr);
+
+      if ((new_info.index_size == 1 && mgr->caps.rewrite_ubyte_ibs) ||
+          (new_info.primitive_restart &&
+           ((new_info.restart_index != fixed_restart_index && mgr->caps.rewrite_restart_index) ||
+           !(mgr->caps.supported_restart_modes & BITFIELD_BIT(new_info.mode)))) ||
+          !(mgr->caps.supported_prim_modes & BITFIELD_BIT(new_info.mode))) {
+         util_primconvert_save_flatshade_first(mgr->pc, mgr->flatshade_first);
+         util_primconvert_draw_vbo(mgr->pc, &new_info, drawid_offset, indirect, &new_draw, 1);
+      } else
+         pipe->draw_vbo(pipe, &new_info, drawid_offset, indirect, &new_draw, 1);
    }
 
-   /*
-   if (unroll_indices) {
-      printf("unrolling indices: start_vertex = %i, num_vertices = %i\n",
-             start_vertex, num_vertices);
-      util_dump_draw_info(stdout, info);
-      printf("\n");
-   }
-
-   unsigned i;
-   for (i = 0; i < mgr->nr_vertex_buffers; i++) {
-      printf("input %i: ", i);
-      util_dump_vertex_buffer(stdout, mgr->vertex_buffer+i);
-      printf("\n");
-   }
-   for (i = 0; i < mgr->nr_real_vertex_buffers; i++) {
-      printf("real %i: ", i);
-      util_dump_vertex_buffer(stdout, mgr->real_vertex_buffer+i);
-      printf("\n");
-   }
-   */
-
-   u_upload_unmap(pipe->stream_uploader);
-   if (mgr->dirty_real_vb_mask)
-      u_vbuf_set_driver_vertex_buffers(mgr);
-
-   if ((new_info.index_size == 1 && mgr->caps.rewrite_ubyte_ibs) ||
-       (new_info.primitive_restart &&
-        ((new_info.restart_index != fixed_restart_index && mgr->caps.rewrite_restart_index) ||
-        !(mgr->caps.supported_restart_modes & BITFIELD_BIT(new_info.mode)))) ||
-       !(mgr->caps.supported_prim_modes & BITFIELD_BIT(new_info.mode))) {
-      util_primconvert_save_flatshade_first(mgr->pc, mgr->flatshade_first);
-      util_primconvert_draw_vbo(mgr->pc, &new_info, drawid_offset, indirect, &new_draw, 1);
-   } else
-      pipe->draw_vbo(pipe, &new_info, drawid_offset, indirect, &new_draw, 1);
-
    if (mgr->using_translate) {
       u_vbuf_translate_end(mgr);
    }
