@@ -28,7 +28,7 @@
 #ifndef LP_RAST_PRIV_H
 #define LP_RAST_PRIV_H
 
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_thread.h"
 #include "gallivm/lp_bld_debug.h"
 #include "lp_memory.h"
@@ -129,9 +129,17 @@ struct lp_rasterizer
 
    /** For synchronizing the rasterization threads */
    util_barrier barrier;
+
+   struct lp_fence *last_fence;
 };
 
 
+void
+lp_rast_shade_quads_mask_sample(struct lp_rasterizer_task *task,
+                                const struct lp_rast_shader_inputs *inputs,
+                                unsigned x, unsigned y,
+                                uint64_t mask);
+
 void
 lp_rast_shade_quads_mask(struct lp_rasterizer_task *task,
                          const struct lp_rast_shader_inputs *inputs,
@@ -148,15 +156,11 @@ lp_rast_get_color_block_pointer(struct lp_rasterizer_task *task,
                                 unsigned buf, unsigned x, unsigned y,
                                 unsigned layer)
 {
-   unsigned px, py, pixel_offset;
-   uint8_t *color;
-
    assert(x < task->scene->tiles_x * TILE_SIZE);
    assert(y < task->scene->tiles_y * TILE_SIZE);
    assert((x % TILE_VECTOR_WIDTH) == 0);
    assert((y % TILE_VECTOR_HEIGHT) == 0);
    assert(buf < task->scene->fb.nr_cbufs);
-
    assert(task->color_tiles[buf]);
 
    /*
@@ -164,14 +168,15 @@ lp_rast_get_color_block_pointer(struct lp_rasterizer_task *task,
     * it's just extra work - the mul/add would be exactly the same anyway.
     * Fortunately the extra work (modulo) here is very cheap at least...
     */
-   px = x % TILE_SIZE;
-   py = y % TILE_SIZE;
+   unsigned px = x % TILE_SIZE;
+   unsigned py = y % TILE_SIZE;
 
-   pixel_offset = px * task->scene->cbufs[buf].format_bytes +
-                  py * task->scene->cbufs[buf].stride;
-   color = task->color_tiles[buf] + pixel_offset;
+   unsigned pixel_offset = px * task->scene->cbufs[buf].format_bytes +
+                           py * task->scene->cbufs[buf].stride;
+   uint8_t *color = task->color_tiles[buf] + pixel_offset;
 
    if (layer) {
+      assert(layer <= task->scene->fb_max_layer);
       color += layer * task->scene->cbufs[buf].layer_stride;
    }
 
@@ -188,22 +193,18 @@ static inline uint8_t *
 lp_rast_get_depth_block_pointer(struct lp_rasterizer_task *task,
                                 unsigned x, unsigned y, unsigned layer)
 {
-   unsigned px, py, pixel_offset;
-   uint8_t *depth;
-
    assert(x < task->scene->tiles_x * TILE_SIZE);
    assert(y < task->scene->tiles_y * TILE_SIZE);
    assert((x % TILE_VECTOR_WIDTH) == 0);
    assert((y % TILE_VECTOR_HEIGHT) == 0);
-
    assert(task->depth_tile);
 
-   px = x % TILE_SIZE;
-   py = y % TILE_SIZE;
+   unsigned px = x % TILE_SIZE;
+   unsigned py = y % TILE_SIZE;
 
-   pixel_offset = px * task->scene->zsbuf.format_bytes +
-                  py * task->scene->zsbuf.stride;
-   depth = task->depth_tile + pixel_offset;
+   unsigned pixel_offset = px * task->scene->zsbuf.format_bytes +
+                           py * task->scene->zsbuf.stride;
+   uint8_t *depth = task->depth_tile + pixel_offset;
 
    if (layer) {
       depth += layer * task->scene->zsbuf.layer_stride;
@@ -230,28 +231,36 @@ lp_rast_shade_quads_all( struct lp_rasterizer_task *task,
    struct lp_fragment_shader_variant *variant = state->variant;
    uint8_t *color[PIPE_MAX_COLOR_BUFS];
    unsigned stride[PIPE_MAX_COLOR_BUFS];
+   unsigned sample_stride[PIPE_MAX_COLOR_BUFS];
    uint8_t *depth = NULL;
    unsigned depth_stride = 0;
-   unsigned i;
+   unsigned depth_sample_stride = 0;
 
    /* color buffer */
-   for (i = 0; i < scene->fb.nr_cbufs; i++) {
+   for (unsigned i = 0; i < scene->fb.nr_cbufs; i++) {
       if (scene->fb.cbufs[i]) {
          stride[i] = scene->cbufs[i].stride;
+         sample_stride[i] = scene->cbufs[i].sample_stride;
          color[i] = lp_rast_get_color_block_pointer(task, i, x, y,
-                                                    inputs->layer);
+                                                    inputs->layer + inputs->view_index);
       }
       else {
          stride[i] = 0;
+         sample_stride[i] = 0;
          color[i] = NULL;
       }
    }
 
    if (scene->zsbuf.map) {
-      depth = lp_rast_get_depth_block_pointer(task, x, y, inputs->layer);
+      depth = lp_rast_get_depth_block_pointer(task, x, y, inputs->layer + inputs->view_index);
+      depth_sample_stride = scene->zsbuf.sample_stride;
       depth_stride = scene->zsbuf.stride;
    }
 
+   uint64_t mask = 0;
+   for (unsigned i = 0; i < scene->fb_max_samples; i++)
+      mask |= (uint64_t)0xffff << (16 * i);
+
    /*
     * The rasterizer may produce fragments outside our
     * allocated 4x4 blocks hence need to filter them out here.
@@ -259,6 +268,7 @@ lp_rast_shade_quads_all( struct lp_rasterizer_task *task,
    if ((x % TILE_SIZE) < task->width && (y % TILE_SIZE) < task->height) {
       /* Propagate non-interpolated raster state. */
       task->thread_data.raster_state.viewport_index = inputs->viewport_index;
+      task->thread_data.raster_state.view_index = inputs->view_index;
 
       /* run shader on 4x4 block */
       BEGIN_JIT_CALL(state, task);
@@ -270,72 +280,195 @@ lp_rast_shade_quads_all( struct lp_rasterizer_task *task,
                                          GET_DADY(inputs),
                                          color,
                                          depth,
-                                         0xffff,
+                                         mask,
                                          &task->thread_data,
                                          stride,
-                                         depth_stride);
+                                         depth_stride,
+                                         sample_stride,
+                                         depth_sample_stride);
       END_JIT_CALL();
    }
 }
 
-void lp_rast_triangle_1( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_2( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_3( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_4( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_5( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_6( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_7( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_8( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
+void
+lp_rast_triangle_1(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
 
-void lp_rast_triangle_3_4(struct lp_rasterizer_task *,
-			  const union lp_rast_cmd_arg );
+void
+lp_rast_triangle_2(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
 
-void lp_rast_triangle_3_16( struct lp_rasterizer_task *, 
-                            const union lp_rast_cmd_arg );
+void
+lp_rast_triangle_3(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
 
-void lp_rast_triangle_4_16( struct lp_rasterizer_task *, 
-                            const union lp_rast_cmd_arg );
+void
+lp_rast_triangle_4(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_5(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_6(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_7(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_8(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_3_4(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_3_16(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_4_16(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_1(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_2(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_3(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_4(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_5(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_6(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_7(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_8(struct lp_rasterizer_task *, const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_3_4(struct lp_rasterizer_task *,
+                        const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_3_16(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_32_4_16(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_rectangle(struct lp_rasterizer_task *,
+                  const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_1(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_2(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_3(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_4(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_5(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_6(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_7(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_8(struct lp_rasterizer_task *,
+                      const union lp_rast_cmd_arg);
 
 
-void lp_rast_triangle_32_1( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_32_2( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_32_3( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_32_4( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_32_5( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_32_6( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_32_7( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
-void lp_rast_triangle_32_8( struct lp_rasterizer_task *, 
-                         const union lp_rast_cmd_arg );
+void
+lp_rast_triangle_ms_3_4(struct lp_rasterizer_task *,
+                        const union lp_rast_cmd_arg);
 
-void lp_rast_triangle_32_3_4(struct lp_rasterizer_task *,
-			  const union lp_rast_cmd_arg );
 
-void lp_rast_triangle_32_3_16( struct lp_rasterizer_task *, 
-                            const union lp_rast_cmd_arg );
+void
+lp_rast_triangle_ms_3_16(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
 
-void lp_rast_triangle_32_4_16( struct lp_rasterizer_task *, 
-                            const union lp_rast_cmd_arg );
+
+void
+lp_rast_triangle_ms_4_16(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+
+void
+lp_rast_triangle_ms_32_1(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_2(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_3(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_4(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_5(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_6(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_7(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_8(struct lp_rasterizer_task *,
+                         const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_3_4(struct lp_rasterizer_task *,
+                           const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_3_16(struct lp_rasterizer_task *,
+                            const union lp_rast_cmd_arg);
+
+void
+lp_rast_triangle_ms_32_4_16(struct lp_rasterizer_task *,
+                            const union lp_rast_cmd_arg);
 
 void
 lp_rast_set_state(struct lp_rasterizer_task *task,
                   const union lp_rast_cmd_arg arg);
- 
+
 void
-lp_debug_bin( const struct cmd_bin *bin, int x, int y );
+lp_debug_bin(const struct cmd_bin *bin, int x, int y);
+
+void
+lp_linear_rasterize_bin(struct lp_rasterizer_task *task,
+                        const struct cmd_bin *bin);
+
+void
+lp_rast_linear_rect_fallback(struct lp_rasterizer_task *task,
+                             const struct lp_rast_shader_inputs *inputs,
+                             const struct u_rect *box);
 
 #endif
