@@ -30,6 +30,7 @@
 #define LP_STATE_FS_H_
 
 
+#include "util/list.h"
 #include "pipe/p_compiler.h"
 #include "pipe/p_state.h"
 #include "tgsi/tgsi_scan.h" /* for tgsi_shader_info */
@@ -103,6 +104,7 @@ struct lp_fragment_shader_variant_key
    unsigned depth_clamp:1;
    unsigned multisample:1;
    unsigned no_ms_sample_mask_out:1;
+   unsigned restrict_depth_values:1;
 
    enum pipe_format zsbuf_format;
    enum pipe_format cbuf_format[PIPE_MAX_COLOR_BUFS];
@@ -145,14 +147,15 @@ static inline struct lp_image_static_state *
 lp_fs_variant_key_images(struct lp_fragment_shader_variant_key *key)
 {
    return (struct lp_image_static_state *)
-      &(lp_fs_variant_key_samplers(key)[MAX2(key->nr_samplers, key->nr_sampler_views)]);
+      &(lp_fs_variant_key_samplers(key)[MAX2(key->nr_samplers,
+                                             key->nr_sampler_views)]);
 }
 
 /** doubly-linked list item */
 struct lp_fs_variant_list_item
 {
+   struct list_head list;
    struct lp_fragment_shader_variant *base;
-   struct lp_fs_variant_list_item *next, *prev;
 };
 
 
@@ -163,10 +166,10 @@ struct lp_fragment_shader_variant
     */
    unsigned potentially_opaque:1;
 
+   unsigned opaque:1;
    unsigned blit:1;
    unsigned linear_input_mask:16;
    struct pipe_reference reference;
-   boolean opaque;
 
    struct gallivm_state *gallivm;
 
@@ -174,9 +177,9 @@ struct lp_fragment_shader_variant
    LLVMTypeRef jit_thread_data_ptr_type;
    LLVMTypeRef jit_linear_context_ptr_type;
 
-   LLVMValueRef function[2];
+   LLVMValueRef function[2]; // [RAST_WHOLE], [RAST_EDGE_TEST]
 
-   lp_jit_frag_func jit_function[2];
+   lp_jit_frag_func jit_function[2]; // [RAST_WHOLE], [RAST_EDGE_TEST]
 
    lp_jit_linear_func jit_linear;
    lp_jit_linear_func jit_linear_blit;
@@ -211,13 +214,9 @@ struct lp_fragment_shader
    struct pipe_reference reference;
    struct lp_tgsi_info info;
 
-   /*
-    * Analysis results
-    */
-
+   /* Analysis results */
    enum lp_fs_kind kind;
 
-
    struct lp_fs_variant_list_item variants;
 
    struct draw_fragment_shader *draw_data;
@@ -235,6 +234,7 @@ struct lp_fragment_shader
 
 void
 llvmpipe_fs_analyse_nir(struct lp_fragment_shader *shader);
+
 void
 llvmpipe_fs_analyse(struct lp_fragment_shader *shader,
                     const struct tgsi_token *tokens);
@@ -256,7 +256,6 @@ lp_debug_fs_variant(struct lp_fragment_shader_variant *variant);
 const char *
 lp_debug_fs_kind(enum lp_fs_kind kind);
 
-
 void
 lp_linear_check_variant(struct lp_fragment_shader_variant *variant);
 
@@ -270,7 +269,8 @@ lp_fs_reference(struct llvmpipe_context *llvmpipe,
                 struct lp_fragment_shader *shader)
 {
    struct lp_fragment_shader *old_ptr = *ptr;
-   if (pipe_reference(old_ptr ? &(*ptr)->reference : NULL, shader ? &shader->reference : NULL)) {
+   if (pipe_reference(old_ptr ? &(*ptr)->reference : NULL,
+                      shader ? &shader->reference : NULL)) {
       llvmpipe_destroy_fs(llvmpipe, old_ptr);
    }
    *ptr = shader;
@@ -286,7 +286,8 @@ lp_fs_variant_reference(struct llvmpipe_context *llvmpipe,
                         struct lp_fragment_shader_variant *variant)
 {
    struct lp_fragment_shader_variant *old_ptr = *ptr;
-   if (pipe_reference(old_ptr ? &(*ptr)->reference : NULL, variant ? &variant->reference : NULL)) {
+   if (pipe_reference(old_ptr ? &(*ptr)->reference : NULL,
+                      variant ? &variant->reference : NULL)) {
       llvmpipe_destroy_shader_variant(llvmpipe, old_ptr);
    }
    *ptr = variant;
