@@ -33,7 +33,7 @@
 
 #include "virgl_context.h"
 #include "virgl_encode.h"
-#include "virgl_protocol.h"
+#include "virtio-gpu/virgl_protocol.h"
 #include "virgl_resource.h"
 #include "virgl_screen.h"
 
@@ -42,6 +42,7 @@
 #define CONV_FORMAT(f) [PIPE_FORMAT_##f] = VIRGL_FORMAT_##f,
 
 static const enum virgl_formats virgl_formats_conv_table[PIPE_FORMAT_COUNT] = {
+   CONV_FORMAT(NONE)
    CONV_FORMAT(B8G8R8A8_UNORM)
    CONV_FORMAT(B8G8R8X8_UNORM)
    CONV_FORMAT(A8R8G8B8_UNORM)
@@ -52,6 +53,7 @@ static const enum virgl_formats virgl_formats_conv_table[PIPE_FORMAT_COUNT] = {
    CONV_FORMAT(R10G10B10A2_UNORM)
    CONV_FORMAT(L8_UNORM)
    CONV_FORMAT(A8_UNORM)
+   CONV_FORMAT(I8_UNORM)
    CONV_FORMAT(L8A8_UNORM)
    CONV_FORMAT(L16_UNORM)
    CONV_FORMAT(Z16_UNORM)
@@ -171,9 +173,11 @@ static const enum virgl_formats virgl_formats_conv_table[PIPE_FORMAT_COUNT] = {
    CONV_FORMAT(L16_SNORM)
    CONV_FORMAT(L16A16_SNORM)
    CONV_FORMAT(A16_FLOAT)
+   CONV_FORMAT(I16_FLOAT)
    CONV_FORMAT(L16_FLOAT)
    CONV_FORMAT(L16A16_FLOAT)
    CONV_FORMAT(A32_FLOAT)
+   CONV_FORMAT(I32_FLOAT)
    CONV_FORMAT(L32_FLOAT)
    CONV_FORMAT(L32A32_FLOAT)
    CONV_FORMAT(YV12)
@@ -206,21 +210,27 @@ static const enum virgl_formats virgl_formats_conv_table[PIPE_FORMAT_COUNT] = {
    CONV_FORMAT(R32G32B32_SINT)
    CONV_FORMAT(R32G32B32A32_SINT)
    CONV_FORMAT(A8_UINT)
+   CONV_FORMAT(I8_UINT)
    CONV_FORMAT(L8_UINT)
    CONV_FORMAT(L8A8_UINT)
    CONV_FORMAT(A8_SINT)
    CONV_FORMAT(L8_SINT)
+   CONV_FORMAT(I8_SINT)
    CONV_FORMAT(L8A8_SINT)
    CONV_FORMAT(A16_UINT)
+   CONV_FORMAT(I16_UINT)
    CONV_FORMAT(L16_UINT)
    CONV_FORMAT(L16A16_UINT)
    CONV_FORMAT(A16_SINT)
+   CONV_FORMAT(I16_SINT)
    CONV_FORMAT(L16_SINT)
    CONV_FORMAT(L16A16_SINT)
    CONV_FORMAT(A32_UINT)
+   CONV_FORMAT(I32_UINT)
    CONV_FORMAT(L32_UINT)
    CONV_FORMAT(L32A32_UINT)
    CONV_FORMAT(A32_SINT)
+   CONV_FORMAT(I32_SINT)
    CONV_FORMAT(L32_SINT)
    CONV_FORMAT(L32A32_SINT)
    CONV_FORMAT(R10G10B10A2_SSCALED)
@@ -248,6 +258,46 @@ static const enum virgl_formats virgl_formats_conv_table[PIPE_FORMAT_COUNT] = {
    CONV_FORMAT(R10G10B10X2_UNORM)
    CONV_FORMAT(A4B4G4R4_UNORM)
    CONV_FORMAT(R8_SRGB)
+   CONV_FORMAT(R8G8_SRGB)
+   CONV_FORMAT(ETC1_RGB8)
+   CONV_FORMAT(ETC2_RGB8)
+   CONV_FORMAT(ETC2_SRGB8)
+   CONV_FORMAT(ETC2_RGB8A1)
+   CONV_FORMAT(ETC2_SRGB8A1)
+   CONV_FORMAT(ETC2_RGBA8)
+   CONV_FORMAT(ETC2_SRGBA8)
+   CONV_FORMAT(ETC2_R11_UNORM)
+   CONV_FORMAT(ETC2_R11_SNORM)
+   CONV_FORMAT(ETC2_RG11_UNORM)
+   CONV_FORMAT(ETC2_RG11_SNORM)
+   CONV_FORMAT(ASTC_4x4)
+   CONV_FORMAT(ASTC_5x4)
+   CONV_FORMAT(ASTC_5x5)
+   CONV_FORMAT(ASTC_6x5)
+   CONV_FORMAT(ASTC_6x6)
+   CONV_FORMAT(ASTC_8x5)
+   CONV_FORMAT(ASTC_8x6)
+   CONV_FORMAT(ASTC_8x8)
+   CONV_FORMAT(ASTC_10x5)
+   CONV_FORMAT(ASTC_10x6)
+   CONV_FORMAT(ASTC_10x8)
+   CONV_FORMAT(ASTC_10x10)
+   CONV_FORMAT(ASTC_12x10)
+   CONV_FORMAT(ASTC_12x12)
+   CONV_FORMAT(ASTC_4x4_SRGB)
+   CONV_FORMAT(ASTC_5x4_SRGB)
+   CONV_FORMAT(ASTC_5x5_SRGB)
+   CONV_FORMAT(ASTC_6x5_SRGB)
+   CONV_FORMAT(ASTC_6x6_SRGB)
+   CONV_FORMAT(ASTC_8x5_SRGB)
+   CONV_FORMAT(ASTC_8x6_SRGB)
+   CONV_FORMAT(ASTC_8x8_SRGB )
+   CONV_FORMAT(ASTC_10x5_SRGB)
+   CONV_FORMAT(ASTC_10x6_SRGB)
+   CONV_FORMAT(ASTC_10x8_SRGB)
+   CONV_FORMAT(ASTC_10x10_SRGB)
+   CONV_FORMAT(ASTC_12x10_SRGB)
+   CONV_FORMAT(ASTC_12x12_SRGB)
 };
 
 enum virgl_formats pipe_to_virgl_format(enum pipe_format format)
@@ -328,13 +378,19 @@ int virgl_encode_blend_state(struct virgl_context *ctx,
    virgl_encoder_write_dword(ctx->cbuf, tmp);
 
    for (i = 0; i < VIRGL_MAX_COLOR_BUFS; i++) {
+      /* We use alpha src factor to pass the advanced blend equation value
+       * to the host. By doing so, we don't have to change the protocol.
+       */
+      uint32_t alpha = (i == 0 && blend_state->advanced_blend_func)
+                        ? blend_state->advanced_blend_func
+                        : blend_state->rt[i].alpha_src_factor;
       tmp =
          VIRGL_OBJ_BLEND_S2_RT_BLEND_ENABLE(blend_state->rt[i].blend_enable) |
          VIRGL_OBJ_BLEND_S2_RT_RGB_FUNC(blend_state->rt[i].rgb_func) |
          VIRGL_OBJ_BLEND_S2_RT_RGB_SRC_FACTOR(blend_state->rt[i].rgb_src_factor) |
          VIRGL_OBJ_BLEND_S2_RT_RGB_DST_FACTOR(blend_state->rt[i].rgb_dst_factor)|
          VIRGL_OBJ_BLEND_S2_RT_ALPHA_FUNC(blend_state->rt[i].alpha_func) |
-         VIRGL_OBJ_BLEND_S2_RT_ALPHA_SRC_FACTOR(blend_state->rt[i].alpha_src_factor) |
+         VIRGL_OBJ_BLEND_S2_RT_ALPHA_SRC_FACTOR(alpha) |
          VIRGL_OBJ_BLEND_S2_RT_ALPHA_DST_FACTOR(blend_state->rt[i].alpha_dst_factor) |
          VIRGL_OBJ_BLEND_S2_RT_COLORMASK(blend_state->rt[i].colormask);
       virgl_encoder_write_dword(ctx->cbuf, tmp);
@@ -351,11 +407,11 @@ int virgl_encode_dsa_state(struct virgl_context *ctx,
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_CREATE_OBJECT, VIRGL_OBJECT_DSA, VIRGL_OBJ_DSA_SIZE));
    virgl_encoder_write_dword(ctx->cbuf, handle);
 
-   tmp = VIRGL_OBJ_DSA_S0_DEPTH_ENABLE(dsa_state->depth.enabled) |
-      VIRGL_OBJ_DSA_S0_DEPTH_WRITEMASK(dsa_state->depth.writemask) |
-      VIRGL_OBJ_DSA_S0_DEPTH_FUNC(dsa_state->depth.func) |
-      VIRGL_OBJ_DSA_S0_ALPHA_ENABLED(dsa_state->alpha.enabled) |
-      VIRGL_OBJ_DSA_S0_ALPHA_FUNC(dsa_state->alpha.func);
+   tmp = VIRGL_OBJ_DSA_S0_DEPTH_ENABLE(dsa_state->depth_enabled) |
+      VIRGL_OBJ_DSA_S0_DEPTH_WRITEMASK(dsa_state->depth_writemask) |
+      VIRGL_OBJ_DSA_S0_DEPTH_FUNC(dsa_state->depth_func) |
+      VIRGL_OBJ_DSA_S0_ALPHA_ENABLED(dsa_state->alpha_enabled) |
+      VIRGL_OBJ_DSA_S0_ALPHA_FUNC(dsa_state->alpha_func);
    virgl_encoder_write_dword(ctx->cbuf, tmp);
 
    for (i = 0; i < 2; i++) {
@@ -369,7 +425,7 @@ int virgl_encode_dsa_state(struct virgl_context *ctx,
       virgl_encoder_write_dword(ctx->cbuf, tmp);
    }
 
-   virgl_encoder_write_dword(ctx->cbuf, fui(dsa_state->alpha.ref_value));
+   virgl_encoder_write_dword(ctx->cbuf, fui(dsa_state->alpha_ref_value));
    return 0;
 }
 int virgl_encode_rasterizer_state(struct virgl_context *ctx,
@@ -506,6 +562,13 @@ int virgl_encode_shader_state(struct virgl_context *ctx,
    if (virgl_debug & VIRGL_DEBUG_TGSI)
       debug_printf("TGSI:\n---8<---\n%s\n---8<---\n", str);
 
+   /* virglrenderer before addbd9c5058dcc9d561b20ab747aed58c53499da mis-counts
+    * the tokens needed for a BARRIER, so ask it to allocate some more space.
+    */
+   const char *barrier = str;
+   while ((barrier = strstr(barrier + 1, "BARRIER")))
+      num_tokens++;
+
    shader_len = strlen(str) + 1;
 
    left_bytes = shader_len;
@@ -569,6 +632,38 @@ int virgl_encode_clear(struct virgl_context *ctx,
    return 0;
 }
 
+int virgl_encode_clear_texture(struct virgl_context *ctx,
+                               struct virgl_resource *res,
+                               unsigned int level,
+                               const struct pipe_box *box,
+                               const void *data)
+{
+   const struct util_format_description *desc = util_format_description(res->b.format);
+   unsigned block_bits = desc->block.bits;
+   uint32_t arr[4] = {0};
+   /* The spec describe <data> as a pointer to an array of between one
+    * and four components of texel data that will be used as the source
+    * for the constant fill value.
+    * Here, we are just copying the memory into <arr>. We do not try to
+    * re-create the data array. The host part will take care of interpreting
+    * the memory and applying the correct format to the clear call.
+    */
+   memcpy(&arr, data, block_bits / 8);
+
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_CLEAR_TEXTURE, 0, VIRGL_CLEAR_TEXTURE_SIZE));
+   virgl_encoder_write_res(ctx, res);
+   virgl_encoder_write_dword(ctx->cbuf, level);
+   virgl_encoder_write_dword(ctx->cbuf, box->x);
+   virgl_encoder_write_dword(ctx->cbuf, box->y);
+   virgl_encoder_write_dword(ctx->cbuf, box->z);
+   virgl_encoder_write_dword(ctx->cbuf, box->width);
+   virgl_encoder_write_dword(ctx->cbuf, box->height);
+   virgl_encoder_write_dword(ctx->cbuf, box->depth);
+   for (unsigned i = 0; i < 4; i++)
+      virgl_encoder_write_dword(ctx->cbuf, arr[i]);
+   return 0;
+}
+
 int virgl_encoder_set_framebuffer_state(struct virgl_context *ctx,
                                        const struct pipe_framebuffer_state *state)
 {
@@ -659,53 +754,55 @@ int virgl_encoder_set_index_buffer(struct virgl_context *ctx,
 }
 
 int virgl_encoder_draw_vbo(struct virgl_context *ctx,
-                          const struct pipe_draw_info *info)
+                           const struct pipe_draw_info *info,
+                           unsigned drawid_offset,
+                           const struct pipe_draw_indirect_info *indirect,
+                           const struct pipe_draw_start_count_bias *draw)
 {
    uint32_t length = VIRGL_DRAW_VBO_SIZE;
    if (info->mode == PIPE_PRIM_PATCHES)
       length = VIRGL_DRAW_VBO_SIZE_TESS;
-   if (info->indirect)
+   if (indirect && indirect->buffer)
       length = VIRGL_DRAW_VBO_SIZE_INDIRECT;
    virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_DRAW_VBO, 0, length));
-   virgl_encoder_write_dword(ctx->cbuf, info->start);
-   virgl_encoder_write_dword(ctx->cbuf, info->count);
+   virgl_encoder_write_dword(ctx->cbuf, draw->start);
+   virgl_encoder_write_dword(ctx->cbuf, draw->count);
    virgl_encoder_write_dword(ctx->cbuf, info->mode);
    virgl_encoder_write_dword(ctx->cbuf, !!info->index_size);
    virgl_encoder_write_dword(ctx->cbuf, info->instance_count);
-   virgl_encoder_write_dword(ctx->cbuf, info->index_bias);
+   virgl_encoder_write_dword(ctx->cbuf, info->index_size ? draw->index_bias : 0);
    virgl_encoder_write_dword(ctx->cbuf, info->start_instance);
    virgl_encoder_write_dword(ctx->cbuf, info->primitive_restart);
-   virgl_encoder_write_dword(ctx->cbuf, info->restart_index);
-   virgl_encoder_write_dword(ctx->cbuf, info->min_index);
-   virgl_encoder_write_dword(ctx->cbuf, info->max_index);
-   if (info->count_from_stream_output)
-      virgl_encoder_write_dword(ctx->cbuf, info->count_from_stream_output->buffer_size);
+   virgl_encoder_write_dword(ctx->cbuf, info->primitive_restart ? info->restart_index : 0);
+   virgl_encoder_write_dword(ctx->cbuf, info->index_bounds_valid ? info->min_index : 0);
+   virgl_encoder_write_dword(ctx->cbuf, info->index_bounds_valid ? info->max_index : ~0);
+   if (indirect && indirect->count_from_stream_output)
+      virgl_encoder_write_dword(ctx->cbuf, indirect->count_from_stream_output->buffer_size);
    else
       virgl_encoder_write_dword(ctx->cbuf, 0);
    if (length >= VIRGL_DRAW_VBO_SIZE_TESS) {
-      virgl_encoder_write_dword(ctx->cbuf, info->vertices_per_patch); /* vertices per patch */
-      virgl_encoder_write_dword(ctx->cbuf, info->drawid); /* drawid */
+      virgl_encoder_write_dword(ctx->cbuf, ctx->patch_vertices); /* vertices per patch */
+      virgl_encoder_write_dword(ctx->cbuf, drawid_offset); /* drawid */
    }
    if (length == VIRGL_DRAW_VBO_SIZE_INDIRECT) {
-      virgl_encoder_write_res(ctx, virgl_resource(info->indirect->buffer));
-      virgl_encoder_write_dword(ctx->cbuf, info->indirect->offset);
-      virgl_encoder_write_dword(ctx->cbuf, info->indirect->stride); /* indirect stride */
-      virgl_encoder_write_dword(ctx->cbuf, info->indirect->draw_count); /* indirect draw count */
-      virgl_encoder_write_dword(ctx->cbuf, info->indirect->indirect_draw_count_offset); /* indirect draw count offset */
-      if (info->indirect->indirect_draw_count)
-         virgl_encoder_write_res(ctx, virgl_resource(info->indirect->indirect_draw_count));
+      virgl_encoder_write_res(ctx, virgl_resource(indirect->buffer));
+      virgl_encoder_write_dword(ctx->cbuf, indirect->offset);
+      virgl_encoder_write_dword(ctx->cbuf, indirect->stride); /* indirect stride */
+      virgl_encoder_write_dword(ctx->cbuf, indirect->draw_count); /* indirect draw count */
+      virgl_encoder_write_dword(ctx->cbuf, indirect->indirect_draw_count_offset); /* indirect draw count offset */
+      if (indirect->indirect_draw_count)
+         virgl_encoder_write_res(ctx, virgl_resource(indirect->indirect_draw_count));
       else
          virgl_encoder_write_dword(ctx->cbuf, 0); /* indirect draw count handle */
    }
    return 0;
 }
 
-int virgl_encoder_create_surface(struct virgl_context *ctx,
-                                uint32_t handle,
-                                struct virgl_resource *res,
-                                const struct pipe_surface *templat)
+static int virgl_encoder_create_surface_common(struct virgl_context *ctx,
+                                               uint32_t handle,
+                                               struct virgl_resource *res,
+                                               const struct pipe_surface *templat)
 {
-   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_CREATE_OBJECT, VIRGL_OBJECT_SURFACE, VIRGL_OBJ_SURFACE_SIZE));
    virgl_encoder_write_dword(ctx->cbuf, handle);
    virgl_encoder_write_res(ctx, res);
    virgl_encoder_write_dword(ctx->cbuf, pipe_to_virgl_format(templat->format));
@@ -717,6 +814,26 @@ int virgl_encoder_create_surface(struct virgl_context *ctx,
    return 0;
 }
 
+int virgl_encoder_create_surface(struct virgl_context *ctx,
+                                 uint32_t handle,
+                                 struct virgl_resource *res,
+                                 const struct pipe_surface *templat)
+{
+   if (templat->nr_samples > 0) {
+      ASSERTED struct virgl_screen *rs = virgl_screen(ctx->base.screen);
+      assert(rs->caps.caps.v2.capability_bits_v2 & VIRGL_CAP_V2_IMPLICIT_MSAA);
+
+      virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_CREATE_OBJECT, VIRGL_OBJECT_MSAA_SURFACE, VIRGL_OBJ_MSAA_SURFACE_SIZE));
+      virgl_encoder_create_surface_common(ctx, handle, res, templat);
+      virgl_encoder_write_dword(ctx->cbuf, templat->nr_samples);
+   } else {
+      virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_CREATE_OBJECT, VIRGL_OBJECT_SURFACE, VIRGL_OBJ_SURFACE_SIZE));
+      virgl_encoder_create_surface_common(ctx, handle, res, templat);
+   }
+
+   return 0;
+}
+
 int virgl_encoder_create_so_target(struct virgl_context *ctx,
                                   uint32_t handle,
                                   struct virgl_resource *res,
@@ -753,7 +870,7 @@ static void virgl_encoder_transfer3d_common(struct virgl_screen *vs,
    if (encode_stride == virgl_transfer3d_explicit_stride) {
       stride = transfer->stride;
       layer_stride = transfer->layer_stride;
-   } else if (virgl_transfer3d_host_inferred_stride) {
+   } else if (encode_stride == virgl_transfer3d_host_inferred_stride) {
       stride = 0;
       layer_stride = 0;
    } else {
@@ -789,7 +906,7 @@ int virgl_encoder_inline_write(struct virgl_context *ctx,
    struct virgl_transfer transfer;
    struct virgl_screen *vs = virgl_screen(ctx->base.screen);
 
-   transfer.base.resource = &res->u.b;
+   transfer.base.resource = &res->b;
    transfer.hw_res = res->hw_res;
    transfer.base.level = level;
    transfer.base.usage = usage;
@@ -849,7 +966,8 @@ int virgl_encode_sampler_state(struct virgl_context *ctx,
       VIRGL_OBJ_SAMPLE_STATE_S0_MAG_IMG_FILTER(state->mag_img_filter) |
       VIRGL_OBJ_SAMPLE_STATE_S0_COMPARE_MODE(state->compare_mode) |
       VIRGL_OBJ_SAMPLE_STATE_S0_COMPARE_FUNC(state->compare_func) |
-      VIRGL_OBJ_SAMPLE_STATE_S0_SEAMLESS_CUBE_MAP(state->seamless_cube_map);
+      VIRGL_OBJ_SAMPLE_STATE_S0_SEAMLESS_CUBE_MAP(state->seamless_cube_map) |
+      VIRGL_OBJ_SAMPLE_STATE_S0_MAX_ANISOTROPY((int)(state->max_anisotropy));
 
    virgl_encoder_write_dword(ctx->cbuf, tmp);
    virgl_encoder_write_dword(ctx->cbuf, fui(state->lod_bias));
@@ -876,12 +994,12 @@ int virgl_encode_sampler_view(struct virgl_context *ctx,
    if (rs->caps.caps.v2.capability_bits & VIRGL_CAP_TEXTURE_VIEW)
      dword_fmt_target |= (state->target << 24);
    virgl_encoder_write_dword(ctx->cbuf, dword_fmt_target);
-   if (res->u.b.target == PIPE_BUFFER) {
+   if (res->b.target == PIPE_BUFFER) {
       virgl_encoder_write_dword(ctx->cbuf, state->u.buf.offset / elem_size);
       virgl_encoder_write_dword(ctx->cbuf, (state->u.buf.offset + state->u.buf.size) / elem_size - 1);
    } else {
       if (res->metadata.plane) {
-         debug_assert(state->u.tex.first_layer == 0 && state->u.tex.last_layer == 0);
+         assert(state->u.tex.first_layer == 0 && state->u.tex.last_layer == 0);
          virgl_encoder_write_dword(ctx->cbuf, res->metadata.plane);
       } else {
          virgl_encoder_write_dword(ctx->cbuf, state->u.tex.first_layer | state->u.tex.last_layer << 16);
@@ -1180,6 +1298,18 @@ int virgl_encoder_destroy_sub_ctx(struct virgl_context *ctx, uint32_t sub_ctx_id
    return 0;
 }
 
+int virgl_encode_link_shader(struct virgl_context *ctx, uint32_t *handles)
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_LINK_SHADER, 0, VIRGL_LINK_SHADER_SIZE));
+   virgl_encoder_write_dword(ctx->cbuf, handles[PIPE_SHADER_VERTEX]);
+   virgl_encoder_write_dword(ctx->cbuf, handles[PIPE_SHADER_FRAGMENT]);
+   virgl_encoder_write_dword(ctx->cbuf, handles[PIPE_SHADER_GEOMETRY]);
+   virgl_encoder_write_dword(ctx->cbuf, handles[PIPE_SHADER_TESS_CTRL]);
+   virgl_encoder_write_dword(ctx->cbuf, handles[PIPE_SHADER_TESS_EVAL]);
+   virgl_encoder_write_dword(ctx->cbuf, handles[PIPE_SHADER_COMPUTE]);
+   return 0;
+}
+
 int virgl_encode_bind_shader(struct virgl_context *ctx,
                              uint32_t handle, uint32_t type)
 {
@@ -1219,7 +1349,7 @@ int virgl_encode_set_shader_buffers(struct virgl_context *ctx,
          virgl_encoder_write_dword(ctx->cbuf, buffers[i].buffer_size);
          virgl_encoder_write_res(ctx, res);
 
-         util_range_add(&res->u.b, &res->valid_buffer_range, buffers[i].buffer_offset,
+         util_range_add(&res->b, &res->valid_buffer_range, buffers[i].buffer_offset,
                buffers[i].buffer_offset + buffers[i].buffer_size);
          virgl_resource_dirty(res, 0);
       } else {
@@ -1246,7 +1376,7 @@ int virgl_encode_set_hw_atomic_buffers(struct virgl_context *ctx,
          virgl_encoder_write_dword(ctx->cbuf, buffers[i].buffer_size);
          virgl_encoder_write_res(ctx, res);
 
-         util_range_add(&res->u.b, &res->valid_buffer_range, buffers[i].buffer_offset,
+         util_range_add(&res->b, &res->valid_buffer_range, buffers[i].buffer_offset,
                buffers[i].buffer_offset + buffers[i].buffer_size);
          virgl_resource_dirty(res, 0);
       } else {
@@ -1277,8 +1407,8 @@ int virgl_encode_set_shader_images(struct virgl_context *ctx,
          virgl_encoder_write_dword(ctx->cbuf, images[i].u.buf.size);
          virgl_encoder_write_res(ctx, res);
 
-         if (res->u.b.target == PIPE_BUFFER) {
-            util_range_add(&res->u.b, &res->valid_buffer_range, images[i].u.buf.offset,
+         if (res->b.target == PIPE_BUFFER) {
+            util_range_add(&res->b, &res->valid_buffer_range, images[i].u.buf.offset,
                   images[i].u.buf.offset + images[i].u.buf.size);
          }
          virgl_resource_dirty(res, images[i].u.tex.level);
@@ -1381,10 +1511,18 @@ void virgl_encode_transfer(struct virgl_screen *vs, struct virgl_cmd_buf *buf,
                            struct virgl_transfer *trans, uint32_t direction)
 {
    uint32_t command;
+   struct virgl_resource *vres = virgl_resource(trans->base.resource);
+   enum virgl_transfer3d_encode_stride stride_type =
+        virgl_transfer3d_host_inferred_stride;
+
+   if (trans->base.box.depth == 1 && trans->base.level == 0 &&
+       trans->base.resource->target == PIPE_TEXTURE_2D &&
+       vres->blob_mem == VIRGL_BLOB_MEM_HOST3D_GUEST)
+      stride_type = virgl_transfer3d_explicit_stride;
+
    command = VIRGL_CMD0(VIRGL_CCMD_TRANSFER3D, 0, VIRGL_TRANSFER3D_SIZE);
    virgl_encoder_write_dword(buf, command);
-   virgl_encoder_transfer3d_common(vs, buf, trans,
-                                   virgl_transfer3d_host_inferred_stride);
+   virgl_encoder_transfer3d_common(vs, buf, trans, stride_type);
    virgl_encoder_write_dword(buf, trans->offset);
    virgl_encoder_write_dword(buf, direction);
 }
@@ -1394,10 +1532,22 @@ void virgl_encode_copy_transfer(struct virgl_context *ctx,
 {
    uint32_t command;
    struct virgl_screen *vs = virgl_screen(ctx->base.screen);
+   // set always synchronized to 1, second bit is used for direction
+   uint32_t direction_and_synchronized = VIRGL_COPY_TRANSFER3D_FLAGS_SYNCHRONIZED;
 
+   if (vs->caps.caps.v2.capability_bits_v2 & VIRGL_CAP_V2_COPY_TRANSFER_BOTH_DIRECTIONS) {
+      if (trans->direction == VIRGL_TRANSFER_TO_HOST) {
+         // do nothing, as 0 means transfer to host
+      } else if (trans->direction == VIRGL_TRANSFER_FROM_HOST) {
+         direction_and_synchronized |= VIRGL_COPY_TRANSFER3D_FLAGS_READ_FROM_HOST;
+      } else {
+         // something wrong happened here
+         assert(0);
+      }
+   }
    assert(trans->copy_src_hw_res);
-
    command = VIRGL_CMD0(VIRGL_CCMD_COPY_TRANSFER3D, 0, VIRGL_COPY_TRANSFER3D_SIZE);
+   
    virgl_encoder_write_cmd_dword(ctx, command);
    /* Copy transfers need to explicitly specify the stride, since it may differ
     * from the image stride.
@@ -1405,8 +1555,7 @@ void virgl_encode_copy_transfer(struct virgl_context *ctx,
    virgl_encoder_transfer3d_common(vs, ctx->cbuf, trans, virgl_transfer3d_explicit_stride);
    vs->vws->emit_res(vs->vws, ctx->cbuf, trans->copy_src_hw_res, TRUE);
    virgl_encoder_write_dword(ctx->cbuf, trans->copy_src_offset);
-   /* At the moment all copy transfers are synchronized. */
-   virgl_encoder_write_dword(ctx->cbuf, 1);
+   virgl_encoder_write_dword(ctx->cbuf, direction_and_synchronized);
 }
 
 void virgl_encode_end_transfers(struct virgl_cmd_buf *buf)
@@ -1418,3 +1567,28 @@ void virgl_encode_end_transfers(struct virgl_cmd_buf *buf)
       virgl_encoder_write_dword(buf, command);
    }
 }
+
+void virgl_encode_get_memory_info(struct virgl_context *ctx, struct virgl_resource *res)
+{
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_GET_MEMORY_INFO, 0, 1));
+   virgl_encoder_write_res(ctx, res);
+}
+
+void virgl_encode_emit_string_marker(struct virgl_context *ctx,
+                                     const char *message, int len)
+{
+   /* len is guaranteed to be non-negative but be defensive */
+   assert(len >= 0);
+   if (len <= 0)
+      return;
+
+   if (len > 4 * 0xffff) {
+      debug_printf("VIRGL: host debug flag string too long, will be truncated\n");
+      len = 4 * 0xffff;
+   }
+
+   uint32_t buf_len = (uint32_t )(len + 3) / 4 + 1;
+   virgl_encoder_write_cmd_dword(ctx, VIRGL_CMD0(VIRGL_CCMD_EMIT_STRING_MARKER, 0, buf_len));
+   virgl_encoder_write_dword(ctx->cbuf, len);
+   virgl_encoder_write_block(ctx->cbuf, (const uint8_t *)message, len);
+}
