@@ -28,6 +28,7 @@
 #include "zink_resource.h"
 #include "zink_screen.h"
 
+#include "util/os_file.h"
 #include "util/set.h"
 #include "util/u_memory.h"
 
@@ -36,7 +37,8 @@ destroy_fence(struct zink_screen *screen, struct zink_tc_fence *mfence)
 {
    mfence->fence = NULL;
    tc_unflushed_batch_token_reference(&mfence->tc_token, NULL);
-   VKSCR(DestroySemaphore)(screen->dev, mfence->sem, NULL);
+   if (mfence->sem)
+      VKSCR(DestroySemaphore)(screen->dev, mfence->sem, NULL);
    FREE(mfence);
 }
 
@@ -117,8 +119,8 @@ tc_fence_finish(struct zink_context *ctx, struct zink_tc_fence *mfence, uint64_t
    return true;
 }
 
-bool
-zink_vkfence_wait(struct zink_screen *screen, struct zink_fence *fence, uint64_t timeout_ns)
+static bool
+fence_wait(struct zink_screen *screen, struct zink_fence *fence, uint64_t timeout_ns)
 {
    if (screen->device_lost)
       return true;
@@ -128,14 +130,7 @@ zink_vkfence_wait(struct zink_screen *screen, struct zink_fence *fence, uint64_t
    assert(fence->batch_id);
    assert(fence->submitted);
 
-   bool success = false;
-
-   VkResult ret;
-   if (timeout_ns)
-      ret = VKSCR(WaitForFences)(screen->dev, 1, &fence->fence, VK_TRUE, timeout_ns);
-   else
-      ret = VKSCR(GetFenceStatus)(screen->dev, fence->fence);
-   success = zink_screen_handle_vkresult(screen, ret);
+   bool success = zink_screen_timeline_wait(screen, fence->batch_id, timeout_ns);
 
    if (success) {
       p_atomic_set(&fence->completed, true);
@@ -186,7 +181,7 @@ zink_fence_finish(struct zink_screen *screen, struct pipe_context *pctx, struct
    if (fence->submitted && zink_screen_check_last_finished(screen, fence->batch_id))
       return true;
 
-   return zink_vkfence_wait(screen, fence, timeout_ns);
+   return fence_wait(screen, fence, timeout_ns);
 }
 
 static bool
@@ -204,10 +199,13 @@ zink_fence_server_signal(struct pipe_context *pctx, struct pipe_fence_handle *pf
    struct zink_tc_fence *mfence = (struct zink_tc_fence *)pfence;
 
    assert(!ctx->batch.state->signal_semaphore);
-   /* this is a deferred flush to reduce overhead */
    ctx->batch.state->signal_semaphore = mfence->sem;
    ctx->batch.has_work = true;
-   pctx->flush(pctx, NULL, PIPE_FLUSH_ASYNC);
+   struct zink_batch_state *bs = ctx->batch.state;
+   /* this must produce a synchronous flush that completes before the function returns */
+   pctx->flush(pctx, NULL, 0);
+   if (zink_screen(ctx->base.screen)->threaded)
+      util_queue_fence_wait(&bs->flush_completed);
 }
 
 void
@@ -224,12 +222,73 @@ zink_fence_server_sync(struct pipe_context *pctx, struct pipe_fence_handle *pfen
    VkPipelineStageFlags flag = VK_PIPELINE_STAGE_ALL_COMMANDS_BIT;
    util_dynarray_append(&ctx->batch.state->wait_semaphores, VkSemaphore, mfence->sem);
    util_dynarray_append(&ctx->batch.state->wait_semaphore_stages, VkPipelineStageFlags, flag);
+
+   /* transfer the external wait sempahore ownership to the next submit */
+   mfence->sem = VK_NULL_HANDLE;
 }
 
 void
 zink_create_fence_fd(struct pipe_context *pctx, struct pipe_fence_handle **pfence, int fd, enum pipe_fd_type type)
 {
    struct zink_screen *screen = zink_screen(pctx->screen);
+   VkResult result;
+
+   assert(fd >= 0);
+
+   struct zink_tc_fence *mfence = zink_create_tc_fence();
+   if (!mfence)
+      goto fail_tc_fence_create;
+
+   const VkSemaphoreCreateInfo sci = {
+      .sType = VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO,
+   };
+   result = VKSCR(CreateSemaphore)(screen->dev, &sci, NULL, &mfence->sem);
+   if (result != VK_SUCCESS) {
+      mesa_loge("ZINK: vkCreateSemaphore failed (%s)", vk_Result_to_str(result));
+      goto fail_sem_create;
+   }
+
+   int dup_fd = os_dupfd_cloexec(fd);
+   if (dup_fd < 0)
+      goto fail_fd_dup;
+
+   static const VkExternalSemaphoreHandleTypeFlagBits flags[] = {
+      [PIPE_FD_TYPE_NATIVE_SYNC] = VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_SYNC_FD_BIT,
+      [PIPE_FD_TYPE_SYNCOBJ] = VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_FD_BIT,
+   };
+   assert(type < ARRAY_SIZE(flags));
+
+   const VkImportSemaphoreFdInfoKHR sdi = {
+      .sType = VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_FD_INFO_KHR,
+      .semaphore = mfence->sem,
+      .flags = VK_SEMAPHORE_IMPORT_TEMPORARY_BIT,
+      .handleType = flags[type],
+      .fd = dup_fd,
+   };
+   result = VKSCR(ImportSemaphoreFdKHR)(screen->dev, &sdi);
+   if (!zink_screen_handle_vkresult(screen, result)) {
+      mesa_loge("ZINK: vkImportSemaphoreFdKHR failed (%s)", vk_Result_to_str(result));
+      goto fail_sem_import;
+   }
+
+   *pfence = (struct pipe_fence_handle *)mfence;
+   return;
+
+fail_sem_import:
+   close(dup_fd);
+fail_fd_dup:
+   VKSCR(DestroySemaphore)(screen->dev, mfence->sem, NULL);
+fail_sem_create:
+   FREE(mfence);
+fail_tc_fence_create:
+   *pfence = NULL;
+}
+
+#ifdef _WIN32
+void
+zink_create_fence_win32(struct pipe_screen *pscreen, struct pipe_fence_handle **pfence, void *handle, const void *name, enum pipe_fd_type type)
+{
+   struct zink_screen *screen = zink_screen(pscreen);
    VkResult ret = VK_ERROR_UNKNOWN;
    VkSemaphoreCreateInfo sci = {
       VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO,
@@ -238,25 +297,25 @@ zink_create_fence_fd(struct pipe_context *pctx, struct pipe_fence_handle **pfenc
    };
    struct zink_tc_fence *mfence = zink_create_tc_fence();
    VkExternalSemaphoreHandleTypeFlagBits flags[] = {
-      [PIPE_FD_TYPE_NATIVE_SYNC] = VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_SYNC_FD_BIT,
-      [PIPE_FD_TYPE_SYNCOBJ] = VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_FD_BIT,
+      [PIPE_FD_TYPE_NATIVE_SYNC] = VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_WIN32_BIT,
+      [PIPE_FD_TYPE_SYNCOBJ] = VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_WIN32_BIT,
    };
-   VkImportSemaphoreFdInfoKHR sdi = {0};
+   VkImportSemaphoreWin32HandleInfoKHR sdi = {0};
    assert(type < ARRAY_SIZE(flags));
 
    *pfence = NULL;
 
    if (VKSCR(CreateSemaphore)(screen->dev, &sci, NULL, &mfence->sem) != VK_SUCCESS) {
-      mesa_loge("ZINK: vkCreateSemaphore failed");
       FREE(mfence);
       return;
    }
 
-   sdi.sType = VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_FD_INFO_KHR;
+   sdi.sType = VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_WIN32_HANDLE_INFO_KHR;
    sdi.semaphore = mfence->sem;
    sdi.handleType = flags[type];
-   sdi.fd = fd;
-   ret = VKSCR(ImportSemaphoreFdKHR)(screen->dev, &sdi);
+   sdi.handle = handle;
+   sdi.name = (LPCWSTR)name;
+   ret = VKSCR(ImportSemaphoreWin32HandleKHR)(screen->dev, &sdi);
 
    if (!zink_screen_handle_vkresult(screen, ret))
       goto fail;
@@ -267,6 +326,7 @@ fail:
    VKSCR(DestroySemaphore)(screen->dev, mfence->sem, NULL);
    FREE(mfence);
 }
+#endif
 
 void
 zink_screen_fence_init(struct pipe_screen *pscreen)
