@@ -43,9 +43,9 @@ struct zink_bo;
 #include <vulkan/vulkan.h>
 
 #define ZINK_MAP_TEMPORARY (PIPE_MAP_DRV_PRV << 0)
-#define ZINK_BIND_DMABUF (1 << 29)
-#define ZINK_BIND_TRANSIENT (1 << 30) //transient fb attachment
-#define ZINK_BIND_VIDEO (1 << 31)
+#define ZINK_BIND_DMABUF (1u << 29)
+#define ZINK_BIND_TRANSIENT (1u << 30) //transient fb attachment
+#define ZINK_BIND_VIDEO (1u << 31)
 
 struct mem_key {
    unsigned seen_count;
@@ -60,7 +60,8 @@ struct zink_resource_object {
 
    VkPipelineStageFlagBits access_stage;
    VkAccessFlags access;
-   bool unordered_barrier;
+   bool unordered_read;
+   bool unordered_write;
 
    unsigned persistent_maps; //if nonzero, requires vkFlushMappedMemoryRanges during batch use
    struct zink_descriptor_refs desc_set_refs;
@@ -82,16 +83,14 @@ struct zink_resource_object {
    bool exportable;
 
    /* TODO: this should be a union */
+   int handle;
    struct zink_bo *bo;
    // struct {
-   void *dt;
+   struct kopper_displaytarget *dt;
    uint32_t dt_idx;
    uint32_t last_dt_idx;
-   VkSemaphore acquire;
    VkSemaphore present;
-   bool acquired;
    bool new_dt;
-   bool dt_has_data;
    bool indefinite_acquire;
    // }
 
@@ -99,13 +98,17 @@ struct zink_resource_object {
    VkDeviceSize offset, size, alignment;
    VkImageCreateFlags vkflags;
    VkImageUsageFlags vkusage;
+   VkFormatFeatureFlags vkfeats;
    uint64_t modifier;
    VkImageAspectFlags modifier_aspect;
-   VkSamplerYcbcrConversionKHR sampler_conversion;
-   unsigned plane_sizes[3];
+   VkSamplerYcbcrConversion sampler_conversion;
+   unsigned plane_offsets[3];
+   unsigned plane_strides[3];
+   unsigned plane_count;
 
    bool host_visible;
    bool coherent;
+   bool is_aux;
 };
 
 struct zink_resource {
@@ -119,6 +122,7 @@ struct zink_resource {
          struct util_range valid_buffer_range;
          uint32_t vbo_bind_mask : PIPE_MAX_ATTRIBS;
          uint8_t ubo_bind_count[2];
+         uint8_t ssbo_bind_count[2];
          uint8_t vbo_bind_count;
          uint8_t so_bind_count; //not counted in all_binds
          bool so_valid;
@@ -132,11 +136,13 @@ struct zink_resource {
          VkImageAspectFlags aspect;
          bool optimal_tiling;
          bool need_2D;
+         bool valid;
          uint8_t fb_binds; //not counted in all_binds
       };
    };
    uint32_t sampler_binds[PIPE_SHADER_TYPES];
    uint32_t image_binds[PIPE_SHADER_TYPES];
+   uint16_t sampler_bind_count[2]; //gfx, compute
    uint16_t image_bind_count[2]; //gfx, compute
    uint16_t write_bind_count[2]; //gfx, compute
    uint16_t bindless[2]; //tex, img
@@ -145,6 +151,9 @@ struct zink_resource {
       uint32_t all_binds;
    };
 
+   VkPipelineStageFlagBits gfx_barrier;
+   VkAccessFlagBits barrier_access[2]; //gfx, compute
+
    union {
       struct {
          struct hash_table bufferview_cache;
@@ -158,6 +167,7 @@ struct zink_resource {
 
    bool swapchain;
    bool dmabuf_acquire;
+   bool dmabuf;
    unsigned dt_stride;
 
    uint8_t modifiers_count;
