@@ -41,6 +41,7 @@
 #include "frontend/opencl_interop.h"
 #include "os/os_thread.h"
 #include "postprocess/filters.h"
+#include "kopper_interface.h"
 
 struct dri_context;
 struct dri_drawable;
@@ -52,9 +53,6 @@ struct dri_screen
    struct st_manager base;
    struct st_api *st_api;
 
-   /* on old libGL's invalidate doesn't get called as it should */
-   boolean broken_invalidate;
-
    /* dri */
    __DRIscreen *sPriv;
    boolean throttle;
@@ -81,6 +79,8 @@ struct dri_screen
 
    /* hooks filled in by dri2 & drisw */
    __DRIimage * (*lookup_egl_image)(struct dri_screen *ctx, void *handle);
+   boolean (*validate_egl_image)(struct dri_screen *ctx, void *handle);
+   __DRIimage * (*lookup_egl_image_validated)(struct dri_screen *ctx, void *handle);
 
    /* DRI exts that vary based on gallium pipe_screen caps. */
    __DRIimageExtension image_extension;
@@ -104,6 +104,12 @@ dri_screen(__DRIscreen * sPriv)
    return (struct dri_screen *)sPriv->driverPrivate;
 }
 
+static inline const __DRIkopperLoaderExtension *
+dri_screen_get_kopper(struct dri_screen *screen)
+{
+   return screen->sPriv->kopper_loader;
+}
+
 struct __DRIimageRec {
    struct pipe_resource *texture;
    unsigned level;
@@ -111,9 +117,15 @@ struct __DRIimageRec {
    uint32_t dri_format;
    uint32_t dri_fourcc;
    uint32_t dri_components;
+   /* Provided by eglCreateImageKHR if creating from a
+    * texture or a renderbuffer. 0 otherwise.
+    */
+   uint32_t internal_format;
    unsigned use;
    unsigned plane;
 
+   int in_fence_fd;
+
    void *loader_private;
 
    boolean imported_dmabuf;
@@ -157,12 +169,14 @@ dri_destroy_screen_helper(struct dri_screen * screen);
 void
 dri_destroy_screen(__DRIscreen * sPriv);
 
-extern const struct __DriverAPIRec dri_kms_driver_api;
-
+extern const struct __DriverAPIRec dri_swrast_kms_driver_api;
+extern const __DRIextension *dri_swrast_kms_driver_extensions[];
 extern const struct __DriverAPIRec galliumdrm_driver_api;
 extern const __DRIextension *galliumdrm_driver_extensions[];
 extern const struct __DriverAPIRec galliumsw_driver_api;
 extern const __DRIextension *galliumsw_driver_extensions[];
+extern const struct __DriverAPIRec galliumvk_driver_api;
+extern const __DRIextension *galliumvk_driver_extensions[];
 extern const __DRIconfigOptionsExtension gallium_config_options;
 
 #endif
