@@ -28,6 +28,7 @@
 #include <windows.h>
 
 #include "glapi/glapi.h"
+#include "util/debug.h"
 #include "util/u_debug.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
@@ -94,6 +95,7 @@ init_screen(const struct stw_winsys *stw_winsys, HDC hdc)
 
    stw_dev->smapi->screen = screen;
    stw_dev->screen = screen;
+   stw_dev->zink = !memcmp(screen->get_name(screen), "zink", 4);
 
    stw_dev->max_2d_length = screen->get_param(screen,
                                               PIPE_CAP_MAX_TEXTURE_2D_SIZE);
@@ -120,7 +122,8 @@ stw_init(const struct stw_winsys *stw_winsys)
 {
    static struct stw_device stw_dev_storage;
 
-   debug_disable_error_message_boxes();
+   if (env_var_as_boolean("WGL_DISABLE_ERROR_DIALOGS", false))
+      debug_disable_win32_error_dialogs();
 
    assert(!stw_dev);
 
