@@ -357,6 +357,9 @@ driFetchDrawable(struct glx_context *gc, GLXDrawable glxDrawable)
       return NULL;
 
    if (__glxHashLookup(priv->drawHash, glxDrawable, (void *) &pdraw) == 0) {
+      /* Resurrected, so remove from the alive-query-set if exist. */
+      _mesa_set_remove_key(priv->zombieGLXDrawable, pdraw);
+
       pdraw->refcount ++;
       return pdraw;
    }
@@ -414,41 +417,94 @@ driFetchDrawable(struct glx_context *gc, GLXDrawable glxDrawable)
       (*pdraw->destroyDrawable) (pdraw);
       return NULL;
    }
-   pdraw->refcount = 1;
+   /* This sure does look suspicious, doesn't it? We're on this path because
+    * this is a naked Window. GLX 1.3 drawables have an explicit creation
+    * step (setting refcount to 1), and those we would have found in the
+    * hash lookup above, bumped their refcount for the bind_context we're
+    * being called for, and then returned. But since we just created the
+    * internal naked-Window state, we need to account for both here.
+    */
+   pdraw->refcount = 2;
 
    return pdraw;
 }
 
+static int
+discardGLXBadDrawableHandler(Display *display, xError *err, XExtCodes *codes,
+                             int *ret_code)
+{
+   int code = codes->first_error + GLXBadDrawable;
+
+   /* Only discard error which is expected. */
+   if (err->majorCode == codes->major_opcode &&
+       err->minorCode == X_GLXGetDrawableAttributes &&
+       /* newer xserver use GLXBadDrawable, old one use BadDrawable */
+       (err->errorCode == code || err->errorCode == BadDrawable)) {
+      *ret_code = 1;
+      return 1;
+   }
+
+   return 0;
+}
+
+static void
+checkServerGLXDrawableAlive(const struct glx_display *priv)
+{
+   ErrorType old = XESetError(priv->dpy, priv->codes.extension,
+                              discardGLXBadDrawableHandler);
+
+   set_foreach(priv->zombieGLXDrawable, entry) {
+      __GLXDRIdrawable *pdraw = (__GLXDRIdrawable *)entry->key;
+      GLXDrawable drawable = pdraw->drawable;
+      unsigned int dummy;
+
+      /* Fail to query, so the window has been closed. Release the GLXDrawable. */
+      if (!__glXGetDrawableAttribute(priv->dpy, drawable, GLX_WIDTH, &dummy)) {
+         pdraw->destroyDrawable(pdraw);
+         __glxHashDelete(priv->drawHash, drawable);
+         _mesa_set_remove(priv->zombieGLXDrawable, entry);
+      }
+   }
+
+   XESetError(priv->dpy, priv->codes.extension, old);
+}
+
+static void
+releaseDrawable(const struct glx_display *priv, GLXDrawable drawable)
+{
+   __GLXDRIdrawable *pdraw;
+
+   if (__glxHashLookup(priv->drawHash, drawable, (void *) &pdraw) == 0) {
+      /* Only native window and pbuffer have same GLX and X11 drawable ID. */
+      if (pdraw->drawable == pdraw->xDrawable) {
+         pdraw->refcount --;
+         /* If pbuffer's refcount reaches 0, it must be imported from other
+          * display. Because pbuffer created from this display will always
+          * hold the last refcount until destroy the GLXPbuffer object.
+          */
+         if (pdraw->refcount == 0) {
+            if (pdraw->psc->keep_native_window_glx_drawable) {
+               checkServerGLXDrawableAlive(priv);
+               _mesa_set_add(priv->zombieGLXDrawable, pdraw);
+            } else {
+               pdraw->destroyDrawable(pdraw);
+               __glxHashDelete(priv->drawHash, drawable);
+            }
+         }
+      }
+   }
+}
+
 _X_HIDDEN void
 driReleaseDrawables(struct glx_context *gc)
 {
-   const struct glx_display *priv = gc->psc->display;
-   __GLXDRIdrawable *pdraw;
+   const struct glx_display *priv = (gc && gc->psc) ? gc->psc->display : NULL;
 
    if (priv == NULL)
       return;
 
-   if (__glxHashLookup(priv->drawHash,
-		       gc->currentDrawable, (void *) &pdraw) == 0) {
-      if (pdraw->drawable == pdraw->xDrawable) {
-	 pdraw->refcount --;
-	 if (pdraw->refcount == 0) {
-	    pdraw->destroyDrawable(pdraw);
-	    __glxHashDelete(priv->drawHash, gc->currentDrawable);
-	 }
-      }
-   }
-
-   if (__glxHashLookup(priv->drawHash,
-		       gc->currentReadable, (void *) &pdraw) == 0) {
-      if (pdraw->drawable == pdraw->xDrawable) {
-	 pdraw->refcount --;
-	 if (pdraw->refcount == 0) {
-	    pdraw->destroyDrawable(pdraw);
-	    __glxHashDelete(priv->drawHash, gc->currentReadable);
-	 }
-      }
-   }
+   releaseDrawable(priv, gc->currentDrawable);
+   releaseDrawable(priv, gc->currentReadable);
 
    gc->currentDrawable = None;
    gc->currentReadable = None;
@@ -460,7 +516,6 @@ dri_convert_glx_attribs(unsigned num_attribs, const uint32_t *attribs,
                         struct dri_ctx_attribs *dca)
 {
    unsigned i;
-   int no_error = 0;
    uint32_t profile = GLX_CONTEXT_CORE_PROFILE_BIT_ARB;
 
    dca->major_ver = 1;
@@ -470,6 +525,7 @@ dri_convert_glx_attribs(unsigned num_attribs, const uint32_t *attribs,
    dca->release = __DRI_CTX_RELEASE_BEHAVIOR_FLUSH;
    dca->flags = 0;
    dca->api = __DRI_API_OPENGL;
+   dca->no_error = 0;
 
    if (num_attribs == 0)
       return __DRI_CTX_ERROR_SUCCESS;
@@ -490,7 +546,7 @@ dri_convert_glx_attribs(unsigned num_attribs, const uint32_t *attribs,
 	 dca->flags = attribs[i * 2 + 1];
 	 break;
       case GLX_CONTEXT_OPENGL_NO_ERROR_ARB:
-	 no_error = attribs[i * 2 + 1];
+	 dca->no_error = attribs[i * 2 + 1];
 	 break;
       case GLX_CONTEXT_PROFILE_MASK_ARB:
 	 profile = attribs[i * 2 + 1];
@@ -533,10 +589,6 @@ dri_convert_glx_attribs(unsigned num_attribs, const uint32_t *attribs,
       }
    }
 
-   if (no_error) {
-      dca->flags |= __DRI_CTX_FLAG_NO_ERROR;
-   }
-
    switch (profile) {
    case GLX_CONTEXT_CORE_PROFILE_BIT_ARB:
       /* This is the default value, but there are no profiles before OpenGL
@@ -571,7 +623,7 @@ dri_convert_glx_attribs(unsigned num_attribs, const uint32_t *attribs,
    if (dca->flags & ~(__DRI_CTX_FLAG_DEBUG |
                       __DRI_CTX_FLAG_FORWARD_COMPATIBLE |
                       __DRI_CTX_FLAG_ROBUST_BUFFER_ACCESS |
-                      __DRI_CTX_FLAG_NO_ERROR))
+                      __DRI_CTX_FLAG_RESET_ISOLATION))
       return __DRI_CTX_ERROR_UNKNOWN_FLAG;
 
    /* There are no forward-compatible contexts before OpenGL 3.0.  The
@@ -586,34 +638,12 @@ dri_convert_glx_attribs(unsigned num_attribs, const uint32_t *attribs,
    if (dca->major_ver >= 3 && dca->render_type == GLX_COLOR_INDEX_TYPE)
       return __DRI_CTX_ERROR_BAD_FLAG;
 
-   return __DRI_CTX_ERROR_SUCCESS;
-}
-
-_X_HIDDEN bool
-dri2_check_no_error(uint32_t flags, struct glx_context *share_context,
-                    int major, unsigned *error)
-{
-   Bool noError = flags & __DRI_CTX_FLAG_NO_ERROR;
-
    /* The KHR_no_error specs say:
     *
     *    Requires OpenGL ES 2.0 or OpenGL 2.0.
     */
-   if (noError && major < 2) {
-      *error = __DRI_CTX_ERROR_UNKNOWN_ATTRIBUTE;
-      return false;
-   }
-
-   /* The GLX_ARB_create_context_no_error specs say:
-    *
-    *    BadMatch is generated if the value of GLX_CONTEXT_OPENGL_NO_ERROR_ARB
-    *    used to create <share_context> does not match the value of
-    *    GLX_CONTEXT_OPENGL_NO_ERROR_ARB for the context being created.
-    */
-   if (share_context && !!share_context->noError != !!noError) {
-      *error = __DRI_CTX_ERROR_BAD_FLAG;
-      return false;
-   }
+   if (dca->no_error && dca->major_ver < 2)
+      return __DRI_CTX_ERROR_UNKNOWN_ATTRIBUTE;
 
    /* The GLX_ARB_create_context_no_error specs say:
     *
@@ -621,13 +651,11 @@ dri2_check_no_error(uint32_t flags, struct glx_context *share_context,
     *    the same time as a debug or robustness context is specified.
     *
     */
-   if (noError && ((flags & __DRI_CTX_FLAG_DEBUG) ||
-                   (flags & __DRI_CTX_FLAG_ROBUST_BUFFER_ACCESS))) {
-      *error = __DRI_CTX_ERROR_BAD_FLAG;
-      return false;
-   }
+   if (dca->no_error && ((dca->flags & __DRI_CTX_FLAG_DEBUG) ||
+                         (dca->flags & __DRI_CTX_FLAG_ROBUST_BUFFER_ACCESS)))
+      return __DRI_CTX_ERROR_BAD_FLAG;
 
-   return true;
+   return __DRI_CTX_ERROR_SUCCESS;
 }
 
 struct glx_context *
