@@ -26,7 +26,32 @@
 #include <stdbool.h>
 #include "util/ralloc.h"
 #include "brw_eu.h"
-#include "brw_gen_enum.h"
+
+#include <gtest/gtest.h>
+
+struct CompactParams {
+   unsigned verx10;
+   unsigned align;
+};
+
+std::string
+get_compact_params_name(const testing::TestParamInfo<CompactParams> p)
+{
+   CompactParams params = p.param;
+   std::stringstream ss;
+   ss << params.verx10 << "_";
+   switch (params.align) {
+   case BRW_ALIGN_1:
+      ss << "Align_1";
+      break;
+   case BRW_ALIGN_16:
+      ss << "Align_16";
+      break;
+   default:
+      unreachable("invalid align");
+   }
+   return ss.str();
+}
 
 static bool
 test_compact_instruction(struct brw_codegen *p, brw_inst src)
@@ -34,12 +59,12 @@ test_compact_instruction(struct brw_codegen *p, brw_inst src)
    brw_compact_inst dst;
    memset(&dst, 0xd0, sizeof(dst));
 
-   if (brw_try_compact_instruction(p->devinfo, &dst, &src)) {
+   if (brw_try_compact_instruction(p->isa, &dst, &src)) {
       brw_inst uncompacted;
 
-      brw_uncompact_instruction(p->devinfo, &uncompacted, &dst);
+      brw_uncompact_instruction(p->isa, &uncompacted, &dst);
       if (memcmp(&uncompacted, &src, sizeof(src))) {
-	 brw_debug_compact_uncompact(p->devinfo, &src, &uncompacted);
+	 brw_debug_compact_uncompact(p->isa, &src, &uncompacted);
 	 return false;
       }
    } else {
@@ -49,7 +74,7 @@ test_compact_instruction(struct brw_codegen *p, brw_inst src)
       if (memcmp(&unchanged, &dst, sizeof(dst))) {
 	 fprintf(stderr, "Failed to compact, but dst changed\n");
 	 fprintf(stderr, "  Instruction: ");
-	 brw_disassemble_inst(stderr, p->devinfo, &src, false, 0, NULL);
+	 brw_disassemble_inst(stderr, p->isa, &src, false, 0, NULL);
 	 return false;
       }
    }
@@ -65,17 +90,19 @@ test_compact_instruction(struct brw_codegen *p, brw_inst src)
  * become meaningless once fuzzing twiddles a related bit.
  */
 static void
-clear_pad_bits(const struct gen_device_info *devinfo, brw_inst *inst)
+clear_pad_bits(const struct brw_isa_info *isa, brw_inst *inst)
 {
-   if (brw_inst_opcode(devinfo, inst) != BRW_OPCODE_SEND &&
-       brw_inst_opcode(devinfo, inst) != BRW_OPCODE_SENDC &&
+   const struct intel_device_info *devinfo = isa->devinfo;
+
+   if (brw_inst_opcode(isa, inst) != BRW_OPCODE_SEND &&
+       brw_inst_opcode(isa, inst) != BRW_OPCODE_SENDC &&
        brw_inst_src0_reg_file(devinfo, inst) != BRW_IMMEDIATE_VALUE &&
        brw_inst_src1_reg_file(devinfo, inst) != BRW_IMMEDIATE_VALUE) {
       brw_inst_set_bits(inst, 127, 111, 0);
    }
 
-   if (devinfo->ver == 8 && !devinfo->is_cherryview &&
-       is_3src(devinfo, brw_inst_opcode(devinfo, inst))) {
+   if (devinfo->ver == 8 && devinfo->platform != INTEL_PLATFORM_CHV &&
+       is_3src(isa, brw_inst_opcode(isa, inst))) {
       brw_inst_set_bits(inst, 105, 105, 0);
       brw_inst_set_bits(inst, 84, 84, 0);
       brw_inst_set_bits(inst, 36, 35, 0);
@@ -83,8 +110,10 @@ clear_pad_bits(const struct gen_device_info *devinfo, brw_inst *inst)
 }
 
 static bool
-skip_bit(const struct gen_device_info *devinfo, brw_inst *src, int bit)
+skip_bit(const struct brw_isa_info *isa, brw_inst *src, int bit)
 {
+   const struct intel_device_info *devinfo = isa->devinfo;
+
    /* pad bit */
    if (bit == 7)
       return true;
@@ -93,8 +122,8 @@ skip_bit(const struct gen_device_info *devinfo, brw_inst *src, int bit)
    if (bit == 29)
       return true;
 
-   if (is_3src(devinfo, brw_inst_opcode(devinfo, src))) {
-      if (devinfo->ver >= 9 || devinfo->is_cherryview) {
+   if (is_3src(isa, brw_inst_opcode(isa, src))) {
+      if (devinfo->ver >= 9 || devinfo->platform == INTEL_PLATFORM_CHV) {
          if (bit == 127)
             return true;
       } else {
@@ -130,8 +159,8 @@ skip_bit(const struct gen_device_info *devinfo, brw_inst *src, int bit)
    }
 
    /* sometimes these are pad bits. */
-   if (brw_inst_opcode(devinfo, src) != BRW_OPCODE_SEND &&
-       brw_inst_opcode(devinfo, src) != BRW_OPCODE_SENDC &&
+   if (brw_inst_opcode(isa, src) != BRW_OPCODE_SEND &&
+       brw_inst_opcode(isa, src) != BRW_OPCODE_SENDC &&
        brw_inst_src0_reg_file(devinfo, src) != BRW_IMMEDIATE_VALUE &&
        brw_inst_src1_reg_file(devinfo, src) != BRW_IMMEDIATE_VALUE &&
        bit >= 121) {
@@ -145,22 +174,22 @@ static bool
 test_fuzz_compact_instruction(struct brw_codegen *p, brw_inst src)
 {
    for (int bit0 = 0; bit0 < 128; bit0++) {
-      if (skip_bit(p->devinfo, &src, bit0))
+      if (skip_bit(p->isa, &src, bit0))
 	 continue;
 
       for (int bit1 = 0; bit1 < 128; bit1++) {
          brw_inst instr = src;
 	 uint64_t *bits = instr.data;
 
-         if (skip_bit(p->devinfo, &src, bit1))
+         if (skip_bit(p->isa, &src, bit1))
 	    continue;
 
 	 bits[bit0 / 64] ^= (1ull << (bit0 & 63));
 	 bits[bit1 / 64] ^= (1ull << (bit1 & 63));
 
-         clear_pad_bits(p->devinfo, &instr);
+         clear_pad_bits(p->isa, &instr);
 
-         if (!brw_validate_instruction(p->devinfo, &instr, 0, NULL))
+         if (!brw_validate_instruction(p->isa, &instr, 0, sizeof(brw_inst), NULL))
             continue;
 
 	 if (!test_compact_instruction(p, instr)) {
@@ -173,8 +202,68 @@ test_fuzz_compact_instruction(struct brw_codegen *p, brw_inst src)
    return true;
 }
 
-static void
-gen_ADD_GRF_GRF_GRF(struct brw_codegen *p)
+class CompactTestFixture : public testing::TestWithParam<CompactParams> {
+protected:
+   virtual void SetUp() {
+      CompactParams params = GetParam();
+      mem_ctx = ralloc_context(NULL);
+      devinfo = rzalloc(mem_ctx, intel_device_info);
+      p = rzalloc(mem_ctx, brw_codegen);
+
+      devinfo->verx10 = params.verx10;
+      devinfo->ver = devinfo->verx10 / 10;
+
+      brw_init_isa_info(&isa, devinfo);
+      brw_init_codegen(&isa, p, p);
+      brw_set_default_predicate_control(p, BRW_PREDICATE_NONE);
+      brw_set_default_access_mode(p, params.align);
+   };
+
+   virtual void TearDown() {
+      EXPECT_EQ(p->nr_insn, 1);
+      EXPECT_TRUE(test_compact_instruction(p, p->store[0]));
+      EXPECT_TRUE(test_fuzz_compact_instruction(p, p->store[0]));
+
+      ralloc_free(mem_ctx);
+   };
+
+   void *mem_ctx;
+   struct brw_isa_info isa;
+   intel_device_info *devinfo;
+   brw_codegen *p;
+};
+
+class Instructions : public CompactTestFixture {};
+
+INSTANTIATE_TEST_CASE_P(
+   CompactTest,
+   Instructions,
+   testing::Values(
+      CompactParams{ 50,  BRW_ALIGN_1 }, CompactParams{ 50, BRW_ALIGN_16 },
+      CompactParams{ 60,  BRW_ALIGN_1 }, CompactParams{ 60, BRW_ALIGN_16 },
+      CompactParams{ 70,  BRW_ALIGN_1 }, CompactParams{ 70, BRW_ALIGN_16 },
+      CompactParams{ 75,  BRW_ALIGN_1 }, CompactParams{ 75, BRW_ALIGN_16 },
+      CompactParams{ 80,  BRW_ALIGN_1 }, CompactParams{ 80, BRW_ALIGN_16 },
+      CompactParams{ 90,  BRW_ALIGN_1 }, CompactParams{ 90, BRW_ALIGN_16 },
+      CompactParams{ 110, BRW_ALIGN_1 },
+      CompactParams{ 120, BRW_ALIGN_1 },
+      CompactParams{ 125, BRW_ALIGN_1 }
+   ),
+   get_compact_params_name);
+
+class InstructionsBeforeIvyBridge : public CompactTestFixture {};
+
+INSTANTIATE_TEST_CASE_P(
+   CompactTest,
+   InstructionsBeforeIvyBridge,
+   testing::Values(
+      CompactParams{ 50,  BRW_ALIGN_1 }, CompactParams{ 50, BRW_ALIGN_16 },
+      CompactParams{ 60,  BRW_ALIGN_1 }, CompactParams{ 60, BRW_ALIGN_16 }
+   ),
+   get_compact_params_name);
+
+
+TEST_P(Instructions, ADD_GRF_GRF_GRF)
 {
    struct brw_reg g0 = brw_vec8_grf(0, 0);
    struct brw_reg g2 = brw_vec8_grf(2, 0);
@@ -183,8 +272,7 @@ gen_ADD_GRF_GRF_GRF(struct brw_codegen *p)
    brw_ADD(p, g0, g2, g4);
 }
 
-static void
-gen_ADD_GRF_GRF_IMM(struct brw_codegen *p)
+TEST_P(Instructions, ADD_GRF_GRF_IMM)
 {
    struct brw_reg g0 = brw_vec8_grf(0, 0);
    struct brw_reg g2 = brw_vec8_grf(2, 0);
@@ -192,8 +280,7 @@ gen_ADD_GRF_GRF_IMM(struct brw_codegen *p)
    brw_ADD(p, g0, g2, brw_imm_f(1.0));
 }
 
-static void
-gen_ADD_GRF_GRF_IMM_d(struct brw_codegen *p)
+TEST_P(Instructions, ADD_GRF_GRF_IMM_d)
 {
    struct brw_reg g0 = retype(brw_vec8_grf(0, 0), BRW_REGISTER_TYPE_D);
    struct brw_reg g2 = retype(brw_vec8_grf(2, 0), BRW_REGISTER_TYPE_D);
@@ -201,8 +288,7 @@ gen_ADD_GRF_GRF_IMM_d(struct brw_codegen *p)
    brw_ADD(p, g0, g2, brw_imm_d(1));
 }
 
-static void
-gen_MOV_GRF_GRF(struct brw_codegen *p)
+TEST_P(Instructions, MOV_GRF_GRF)
 {
    struct brw_reg g0 = brw_vec8_grf(0, 0);
    struct brw_reg g2 = brw_vec8_grf(2, 0);
@@ -210,8 +296,7 @@ gen_MOV_GRF_GRF(struct brw_codegen *p)
    brw_MOV(p, g0, g2);
 }
 
-static void
-gen_ADD_MRF_GRF_GRF(struct brw_codegen *p)
+TEST_P(InstructionsBeforeIvyBridge, ADD_MRF_GRF_GRF)
 {
    struct brw_reg m6 = brw_vec8_reg(BRW_MESSAGE_REGISTER_FILE, 6, 0);
    struct brw_reg g2 = brw_vec8_grf(2, 0);
@@ -220,8 +305,7 @@ gen_ADD_MRF_GRF_GRF(struct brw_codegen *p)
    brw_ADD(p, m6, g2, g4);
 }
 
-static void
-gen_ADD_vec1_GRF_GRF_GRF(struct brw_codegen *p)
+TEST_P(Instructions, ADD_vec1_GRF_GRF_GRF)
 {
    struct brw_reg g0 = brw_vec1_grf(0, 0);
    struct brw_reg g2 = brw_vec1_grf(2, 0);
@@ -230,8 +314,7 @@ gen_ADD_vec1_GRF_GRF_GRF(struct brw_codegen *p)
    brw_ADD(p, g0, g2, g4);
 }
 
-static void
-gen_PLN_MRF_GRF_GRF(struct brw_codegen *p)
+TEST_P(InstructionsBeforeIvyBridge, PLN_MRF_GRF_GRF)
 {
    struct brw_reg m6 = brw_vec8_reg(BRW_MESSAGE_REGISTER_FILE, 6, 0);
    struct brw_reg interp = brw_vec1_grf(2, 0);
@@ -240,8 +323,7 @@ gen_PLN_MRF_GRF_GRF(struct brw_codegen *p)
    brw_PLN(p, m6, interp, g4);
 }
 
-static void
-gen_f0_0_MOV_GRF_GRF(struct brw_codegen *p)
+TEST_P(Instructions, f0_0_MOV_GRF_GRF)
 {
    struct brw_reg g0 = brw_vec8_grf(0, 0);
    struct brw_reg g2 = brw_vec8_grf(2, 0);
@@ -256,8 +338,7 @@ gen_f0_0_MOV_GRF_GRF(struct brw_codegen *p)
  * it, so that we run the fuzzing can run over all the other bits that might
  * interact with it.
  */
-static void
-gen_f0_1_MOV_GRF_GRF(struct brw_codegen *p)
+TEST_P(Instructions, f0_1_MOV_GRF_GRF)
 {
    struct brw_reg g0 = brw_vec8_grf(0, 0);
    struct brw_reg g2 = brw_vec8_grf(2, 0);
@@ -268,79 +349,3 @@ gen_f0_1_MOV_GRF_GRF(struct brw_codegen *p)
    brw_inst_set_flag_subreg_nr(p->devinfo, mov, 1);
    brw_pop_insn_state(p);
 }
-
-struct {
-   void (*func)(struct brw_codegen *p);
-   int gens;
-} tests[] = {
-   { gen_MOV_GRF_GRF,          GFX_ALL      },
-   { gen_ADD_GRF_GRF_GRF,      GFX_ALL      },
-   { gen_ADD_GRF_GRF_IMM,      GFX_ALL      },
-   { gen_ADD_GRF_GRF_IMM_d,    GFX_ALL      },
-   { gen_ADD_MRF_GRF_GRF,      GFX_LE(GFX6) },
-   { gen_ADD_vec1_GRF_GRF_GRF, GFX_ALL      },
-   { gen_PLN_MRF_GRF_GRF,      GFX_LE(GFX6) },
-   { gen_f0_0_MOV_GRF_GRF,     GFX_ALL      },
-   { gen_f0_1_MOV_GRF_GRF,     GFX_ALL      },
-};
-
-static bool
-run_tests(const struct gen_device_info *devinfo)
-{
-   bool fail = false;
-
-   for (unsigned i = 0; i < ARRAY_SIZE(tests); i++) {
-      if ((tests[i].gens & gen_from_devinfo(devinfo)) == 0)
-         continue;
-
-      for (int align_16 = 0; align_16 <= 1; align_16++) {
-         /* Align16 support is not present on Gfx11+ */
-         if (devinfo->ver >= 11 && align_16)
-            continue;
-
-	 struct brw_codegen *p = rzalloc(NULL, struct brw_codegen);
-	 brw_init_codegen(devinfo, p, p);
-
-	 brw_set_default_predicate_control(p, BRW_PREDICATE_NONE);
-	 if (align_16)
-	    brw_set_default_access_mode(p, BRW_ALIGN_16);
-	 else
-	    brw_set_default_access_mode(p, BRW_ALIGN_1);
-
-	 tests[i].func(p);
-	 assert(p->nr_insn == 1);
-
-	 if (!test_compact_instruction(p, p->store[0])) {
-	    fail = true;
-	    continue;
-	 }
-
-	 if (!test_fuzz_compact_instruction(p, p->store[0])) {
-	    fail = true;
-	    continue;
-	 }
-
-	 ralloc_free(p);
-      }
-   }
-
-   return fail;
-}
-
-int
-main(UNUSED int argc, UNUSED char **argv)
-{
-   struct gen_device_info *devinfo = (struct gen_device_info *)calloc(1, sizeof(*devinfo));
-   bool fail = false;
-
-   for (devinfo->ver = 5; devinfo->ver <= 12; devinfo->ver++) {
-      if (devinfo->ver == 10)
-         continue;
-
-      devinfo->verx10 = devinfo->ver * 10;
-      fail |= run_tests(devinfo);
-   }
-
-   free(devinfo);
-   return fail;
-}
