@@ -34,27 +34,28 @@ new_upload_buffer(struct gl_context *ctx, GLsizeiptr size, uint8_t **ptr)
 {
    assert(ctx->GLThread.SupportsBufferUploads);
 
-   struct gl_buffer_object *obj = ctx->Driver.NewBufferObject(ctx, -1);
+   struct gl_buffer_object *obj =
+      _mesa_bufferobj_alloc(ctx, -1);
    if (!obj)
       return NULL;
 
    obj->Immutable = true;
 
-   if (!ctx->Driver.BufferData(ctx, GL_ARRAY_BUFFER, size, NULL,
-                               GL_WRITE_ONLY,
-                               GL_CLIENT_STORAGE_BIT | GL_MAP_WRITE_BIT,
-                               obj)) {
-      ctx->Driver.DeleteBuffer(ctx, obj);
+   if (!_mesa_bufferobj_data(ctx, GL_ARRAY_BUFFER, size, NULL,
+                          GL_WRITE_ONLY,
+                          GL_CLIENT_STORAGE_BIT | GL_MAP_WRITE_BIT,
+                          obj)) {
+      _mesa_delete_buffer_object(ctx, obj);
       return NULL;
    }
 
-   *ptr = ctx->Driver.MapBufferRange(ctx, 0, size,
-                                     GL_MAP_WRITE_BIT |
-                                     GL_MAP_UNSYNCHRONIZED_BIT |
-                                     MESA_MAP_THREAD_SAFE_BIT,
-                                     obj, MAP_GLTHREAD);
+   *ptr = _mesa_bufferobj_map_range(ctx, 0, size,
+                                 GL_MAP_WRITE_BIT |
+                                 GL_MAP_UNSYNCHRONIZED_BIT |
+                                 MESA_MAP_THREAD_SAFE_BIT,
+                                 obj, MAP_GLTHREAD);
    if (!*ptr) {
-      ctx->Driver.DeleteBuffer(ctx, obj);
+      _mesa_delete_buffer_object(ctx, obj);
       return NULL;
    }
 
@@ -195,6 +196,9 @@ _mesa_glthread_BindBuffer(struct gl_context *ctx, GLenum target, GLuint buffer)
    case GL_PIXEL_UNPACK_BUFFER:
       glthread->CurrentPixelUnpackBufferName = buffer;
       break;
+   case GL_QUERY_BUFFER:
+      glthread->CurrentQueryBufferName = buffer;
+      break;
    }
 }
 
@@ -204,7 +208,7 @@ _mesa_glthread_DeleteBuffers(struct gl_context *ctx, GLsizei n,
 {
    struct glthread_state *glthread = &ctx->GLThread;
 
-   if (!buffers)
+   if (!buffers || n < 0)
       return;
 
    for (unsigned i = 0; i < n; i++) {
@@ -237,9 +241,10 @@ struct marshal_cmd_BufferData
    /* Next size bytes are GLubyte data[size] */
 };
 
-void
+uint32_t
 _mesa_unmarshal_BufferData(struct gl_context *ctx,
-                           const struct marshal_cmd_BufferData *cmd)
+                           const struct marshal_cmd_BufferData *cmd,
+                           const uint64_t *last)
 {
    const GLuint target_or_name = cmd->target_or_name;
    const GLsizei size = cmd->size;
@@ -263,20 +268,25 @@ _mesa_unmarshal_BufferData(struct gl_context *ctx,
       CALL_BufferData(ctx->CurrentServerDispatch,
                       (target_or_name, size, data, usage));
    }
+   return cmd->cmd_base.cmd_size;
 }
 
-void
+uint32_t
 _mesa_unmarshal_NamedBufferData(struct gl_context *ctx,
-                                const struct marshal_cmd_NamedBufferData *cmd)
+                                const struct marshal_cmd_NamedBufferData *cmd,
+                                const uint64_t *last)
 {
    unreachable("never used - all BufferData variants use DISPATCH_CMD_BufferData");
+   return 0;
 }
 
-void
+uint32_t
 _mesa_unmarshal_NamedBufferDataEXT(struct gl_context *ctx,
-                                   const struct marshal_cmd_NamedBufferDataEXT *cmd)
+                                   const struct marshal_cmd_NamedBufferDataEXT *cmd,
+                                   const uint64_t *last)
 {
    unreachable("never used - all BufferData variants use DISPATCH_CMD_BufferData");
+   return 0;
 }
 
 static void
@@ -358,9 +368,10 @@ struct marshal_cmd_BufferSubData
    /* Next size bytes are GLubyte data[size] */
 };
 
-void
+uint32_t
 _mesa_unmarshal_BufferSubData(struct gl_context *ctx,
-                              const struct marshal_cmd_BufferSubData *cmd)
+                              const struct marshal_cmd_BufferSubData *cmd,
+                              const uint64_t *last)
 {
    const GLenum target_or_name = cmd->target_or_name;
    const GLintptr offset = cmd->offset;
@@ -377,20 +388,25 @@ _mesa_unmarshal_BufferSubData(struct gl_context *ctx,
       CALL_BufferSubData(ctx->CurrentServerDispatch,
                          (target_or_name, offset, size, data));
    }
+   return cmd->cmd_base.cmd_size;
 }
 
-void
+uint32_t
 _mesa_unmarshal_NamedBufferSubData(struct gl_context *ctx,
-                                   const struct marshal_cmd_NamedBufferSubData *cmd)
+                                   const struct marshal_cmd_NamedBufferSubData *cmd,
+                                   const uint64_t *last)
 {
    unreachable("never used - all BufferSubData variants use DISPATCH_CMD_BufferSubData");
+   return 0;
 }
 
-void
+uint32_t
 _mesa_unmarshal_NamedBufferSubDataEXT(struct gl_context *ctx,
-                                      const struct marshal_cmd_NamedBufferSubDataEXT *cmd)
+                                      const struct marshal_cmd_NamedBufferSubDataEXT *cmd,
+                                      const uint64_t *last)
 {
    unreachable("never used - all BufferSubData variants use DISPATCH_CMD_BufferSubData");
+   return 0;
 }
 
 static void
