@@ -59,6 +59,8 @@ _mesa_glthread_allocate_command(struct gl_context *ctx,
    struct glthread_state *glthread = &ctx->GLThread;
    const unsigned num_elements = align(size, 8) / 8;
 
+   assert (num_elements <= MARSHAL_MAX_CMD_SIZE / 8);
+
    if (unlikely(glthread->used + num_elements > MARSHAL_MAX_CMD_SIZE / 8))
       _mesa_glthread_flush_batch(ctx);
 
@@ -132,8 +134,8 @@ _mesa_glthread_has_non_vbo_vertices_or_indices_or_indirect(const struct gl_conte
 }
 
 
-struct _glapi_table *
-_mesa_create_marshal_table(const struct gl_context *ctx);
+bool
+_mesa_create_marshal_tables(struct gl_context *ctx);
 
 static inline unsigned
 _mesa_buffer_enum_to_count(GLenum buffer)
@@ -442,11 +444,21 @@ _mesa_glthread_Enable(struct gl_context *ctx, GLenum cap)
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
-   if (cap == GL_PRIMITIVE_RESTART ||
-       cap == GL_PRIMITIVE_RESTART_FIXED_INDEX)
+   switch (cap) {
+   case GL_PRIMITIVE_RESTART:
+   case GL_PRIMITIVE_RESTART_FIXED_INDEX:
       _mesa_glthread_set_prim_restart(ctx, cap, true);
-   else if (cap == GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB)
-      _mesa_glthread_disable(ctx, "Enable(DEBUG_OUTPUT_SYNCHRONOUS)");
+      break;
+   case GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB:
+      _mesa_glthread_destroy(ctx, "Enable(DEBUG_OUTPUT_SYNCHRONOUS)");
+      break;
+   case GL_DEPTH_TEST:
+      ctx->GLThread.DepthTest = true;
+      break;
+   case GL_CULL_FACE:
+      ctx->GLThread.CullFace = true;
+      break;
+   }
 }
 
 static inline void
@@ -455,9 +467,40 @@ _mesa_glthread_Disable(struct gl_context *ctx, GLenum cap)
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
-   if (cap == GL_PRIMITIVE_RESTART ||
-       cap == GL_PRIMITIVE_RESTART_FIXED_INDEX)
+   switch (cap) {
+   case GL_PRIMITIVE_RESTART:
+   case GL_PRIMITIVE_RESTART_FIXED_INDEX:
       _mesa_glthread_set_prim_restart(ctx, cap, false);
+      break;
+   case GL_CULL_FACE:
+      ctx->GLThread.CullFace = false;
+      break;
+   case GL_DEPTH_TEST:
+      ctx->GLThread.DepthTest = false;
+      break;
+   }
+}
+
+static inline int
+_mesa_glthread_IsEnabled(struct gl_context *ctx, GLenum cap)
+{
+   switch (cap) {
+   case GL_CULL_FACE:
+      return ctx->GLThread.CullFace;
+   case GL_DEPTH_TEST:
+      return ctx->GLThread.DepthTest;
+   case GL_VERTEX_ARRAY:
+      return !!(ctx->GLThread.CurrentVAO->UserEnabled & VERT_BIT_POS);
+   case GL_NORMAL_ARRAY:
+      return !!(ctx->GLThread.CurrentVAO->UserEnabled & VERT_BIT_NORMAL);
+   case GL_COLOR_ARRAY:
+      return !!(ctx->GLThread.CurrentVAO->UserEnabled & VERT_BIT_COLOR0);
+   case GL_TEXTURE_COORD_ARRAY:
+      return !!(ctx->GLThread.CurrentVAO->UserEnabled &
+                (1 << VERT_ATTRIB_TEX(ctx->GLThread.ClientActiveTexture)));
+   default:
+      return -1; /* sync and call _mesa_IsEnabled. */
+   }
 }
 
 static inline void
@@ -466,11 +509,20 @@ _mesa_glthread_PushAttrib(struct gl_context *ctx, GLbitfield mask)
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
+   if (ctx->GLThread.AttribStackDepth >= MAX_ATTRIB_STACK_DEPTH)
+      return;
+
    struct glthread_attrib_node *attr =
       &ctx->GLThread.AttribStack[ctx->GLThread.AttribStackDepth++];
 
    attr->Mask = mask;
 
+   if (mask & (GL_POLYGON_BIT | GL_ENABLE_BIT))
+      attr->CullFace = ctx->GLThread.CullFace;
+
+   if (mask & (GL_DEPTH_BUFFER_BIT | GL_ENABLE_BIT))
+      attr->DepthTest = ctx->GLThread.DepthTest;
+
    if (mask & GL_TEXTURE_BIT)
       attr->ActiveTexture = ctx->GLThread.ActiveTexture;
 
@@ -484,10 +536,19 @@ _mesa_glthread_PopAttrib(struct gl_context *ctx)
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
+   if (ctx->GLThread.AttribStackDepth == 0)
+      return;
+
    struct glthread_attrib_node *attr =
       &ctx->GLThread.AttribStack[--ctx->GLThread.AttribStackDepth];
    unsigned mask = attr->Mask;
 
+   if (mask & (GL_POLYGON_BIT | GL_ENABLE_BIT))
+      ctx->GLThread.CullFace = attr->CullFace;
+
+   if (mask & (GL_DEPTH_BUFFER_BIT | GL_ENABLE_BIT))
+      ctx->GLThread.DepthTest = attr->DepthTest;
+
    if (mask & GL_TEXTURE_BIT)
       ctx->GLThread.ActiveTexture = attr->ActiveTexture;
 
@@ -497,12 +558,36 @@ _mesa_glthread_PopAttrib(struct gl_context *ctx)
    }
 }
 
+static bool
+is_matrix_stack_full(struct gl_context *ctx, gl_matrix_index idx)
+{
+   int max_stack_depth = 0;
+   if (M_MODELVIEW == ctx->GLThread.MatrixIndex) {
+      max_stack_depth = MAX_MODELVIEW_STACK_DEPTH;
+   } else if (M_PROJECTION == ctx->GLThread.MatrixIndex) {
+      max_stack_depth = MAX_PROJECTION_STACK_DEPTH;
+   } else if (M_PROGRAM_LAST >= ctx->GLThread.MatrixIndex) {
+      max_stack_depth = MAX_PROGRAM_MATRIX_STACK_DEPTH;
+   } else if (M_TEXTURE_LAST >= ctx->GLThread.MatrixIndex) {
+      max_stack_depth = MAX_TEXTURE_STACK_DEPTH;
+   }
+   assert(max_stack_depth);
+
+   if (ctx->GLThread.MatrixStackDepth[idx] + 1 >= max_stack_depth)
+      return true;
+
+   return false;
+}
+
 static inline void
 _mesa_glthread_MatrixPushEXT(struct gl_context *ctx, GLenum matrixMode)
 {
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
+   if (is_matrix_stack_full(ctx, _mesa_get_matrix_index(ctx, matrixMode)))
+      return;
+
    ctx->GLThread.MatrixStackDepth[_mesa_get_matrix_index(ctx, matrixMode)]++;
 }
 
@@ -512,6 +597,9 @@ _mesa_glthread_MatrixPopEXT(struct gl_context *ctx, GLenum matrixMode)
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
+   if (ctx->GLThread.MatrixStackDepth[_mesa_get_matrix_index(ctx, matrixMode)] == 0)
+      return;
+
    ctx->GLThread.MatrixStackDepth[_mesa_get_matrix_index(ctx, matrixMode)]--;
 }
 
@@ -532,6 +620,9 @@ _mesa_glthread_PushMatrix(struct gl_context *ctx)
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
+   if (is_matrix_stack_full(ctx, ctx->GLThread.MatrixIndex))
+      return;
+
    ctx->GLThread.MatrixStackDepth[ctx->GLThread.MatrixIndex]++;
 }
 
@@ -541,6 +632,9 @@ _mesa_glthread_PopMatrix(struct gl_context *ctx)
    if (ctx->GLThread.ListMode == GL_COMPILE)
       return;
 
+   if (ctx->GLThread.MatrixStackDepth[ctx->GLThread.MatrixIndex] == 0)
+      return;
+
    ctx->GLThread.MatrixStackDepth[ctx->GLThread.MatrixIndex]--;
 }
 
@@ -579,6 +673,9 @@ _mesa_glthread_CallList(struct gl_context *ctx, GLuint list)
       p_atomic_set(&ctx->GLThread.LastDListChangeBatchIndex, -1);
    }
 
+   if (!ctx->Shared->DisplayListsAffectGLThread)
+      return;
+
    /* Clear GL_COMPILE_AND_EXECUTE if needed. We only execute here. */
    unsigned saved_mode = ctx->GLThread.ListMode;
    ctx->GLThread.ListMode = 0;
@@ -721,6 +818,20 @@ _mesa_glthread_DeleteLists(struct gl_context *ctx, GLsizei range)
    _mesa_glthread_flush_batch(ctx);
 }
 
+static inline void
+_mesa_glthread_DeleteFramebuffers(struct gl_context *ctx, GLsizei n,
+                                  const GLuint *ids)
+{
+   if (ctx->GLThread.CurrentDrawFramebuffer) {
+      for (int i = 0; i < n; i++) {
+         if (ctx->GLThread.CurrentDrawFramebuffer == ids[i]) {
+            ctx->GLThread.CurrentDrawFramebuffer = 0;
+            break;
+         }
+      }
+   }
+}
+
 struct marshal_cmd_CallList
 {
    struct marshal_cmd_base cmd_base;
