@@ -163,11 +163,13 @@ pan_blitter_emit_blend(const struct panfrost_device *dev,
                 } else {
 #if PAN_ARCH >= 6
                         cfg.internal.shader.pc = blend_shader;
+#if PAN_ARCH <= 7
                         if (blit_shader->blend_ret_offsets[rt]) {
                                 cfg.internal.shader.return_value =
                                         blit_shader->address +
                                         blit_shader->blend_ret_offsets[rt];
                         }
+#endif
 #else
                         cfg.blend_shader = true;
                         cfg.shader_pc = blend_shader;
@@ -177,35 +179,64 @@ pan_blitter_emit_blend(const struct panfrost_device *dev,
 }
 #endif
 
-static void
-pan_blitter_emit_rsd(const struct panfrost_device *dev,
-                     const struct pan_blit_shader_data *blit_shader,
-                     unsigned rt_count,
-                     const struct pan_image_view **rts,
-                     mali_ptr *blend_shaders,
-                     const struct pan_image_view *z,
-                     const struct pan_image_view *s,
-                     void *out)
-{
-        UNUSED bool zs = (z || s);
-        bool ms = false;
+struct pan_blitter_views {
+        unsigned rt_count;
+        const struct pan_image_view *src_rts[8];
+        const struct pan_image_view *dst_rts[8];
+        const struct pan_image_view *src_z;
+        const struct pan_image_view *dst_z;
+        const struct pan_image_view *src_s;
+        const struct pan_image_view *dst_s;
+};
 
-        for (unsigned i = 0; i < rt_count; i++) {
-                if (rts[i]) {
-                        if (rts[i]->nr_samples > 1)
-                                ms = true;
+static bool
+pan_blitter_is_ms(struct pan_blitter_views *views)
+{
+        for (unsigned i = 0; i < views->rt_count; i++) {
+                if (views->dst_rts[i]) {
+                        if (views->dst_rts[i]->image->layout.nr_samples > 1)
+                               return true;
                 }
         }
 
-        if (z) {
-                if (z->image->layout.nr_samples > 1)
-                        ms = true;
-        }
+        if (views->dst_z && views->dst_z->image->layout.nr_samples > 1)
+                return true;
 
-        if (s) {
-                if (s->image->layout.nr_samples > 1)
-                        ms = true;
+        if (views->dst_s && views->dst_s->image->layout.nr_samples > 1)
+                return true;
+
+        return false;
+}
+
+#if PAN_ARCH >= 5
+static void
+pan_blitter_emit_blends(const struct panfrost_device *dev,
+                        const struct pan_blit_shader_data *blit_shader,
+                        struct pan_blitter_views *views,
+                        mali_ptr *blend_shaders,
+                        void *out)
+{
+        for (unsigned i = 0; i < MAX2(views->rt_count, 1); ++i) {
+                void *dest = out + pan_size(BLEND) * i;
+                const struct pan_image_view *rt_view = views->dst_rts[i];
+                mali_ptr blend_shader = blend_shaders ? blend_shaders[i] : 0;
+
+                pan_blitter_emit_blend(dev, i, rt_view, blit_shader,
+                                       blend_shader, dest);
         }
+}
+#endif
+
+#if PAN_ARCH <= 7
+static void
+pan_blitter_emit_rsd(const struct panfrost_device *dev,
+                     const struct pan_blit_shader_data *blit_shader,
+                     struct pan_blitter_views *views,
+                     mali_ptr *blend_shaders,
+                     void *out)
+{
+        UNUSED bool zs = (views->dst_z || views->dst_s);
+        bool ms = pan_blitter_is_ms(views);
 
         pan_pack(out, RENDERER_STATE, cfg) {
                 assert(blit_shader->address);
@@ -214,10 +245,10 @@ pan_blitter_emit_rsd(const struct panfrost_device *dev,
                 cfg.multisample_misc.sample_mask = 0xFFFF;
                 cfg.multisample_misc.multisample_enable = ms;
                 cfg.multisample_misc.evaluate_per_sample = ms;
-                cfg.multisample_misc.depth_write_mask = z != NULL;
+                cfg.multisample_misc.depth_write_mask = views->dst_z != NULL;
                 cfg.multisample_misc.depth_function = MALI_FUNC_ALWAYS;
 
-                cfg.stencil_mask_misc.stencil_enable = s != NULL;
+                cfg.stencil_mask_misc.stencil_enable = views->dst_s != NULL;
                 cfg.stencil_mask_misc.stencil_mask_front = 0xFF;
                 cfg.stencil_mask_misc.stencil_mask_back = 0xFF;
                 cfg.stencil_front.compare_function = MALI_FUNC_ALWAYS;
@@ -228,8 +259,14 @@ pan_blitter_emit_rsd(const struct panfrost_device *dev,
                 cfg.stencil_back = cfg.stencil_front;
 
 #if PAN_ARCH >= 6
-                /* Skipping ATEST requires forcing Z/S */
-                if (!zs) {
+                if (zs) {
+                        /* Writing Z/S requires late updates */
+                        cfg.properties.zs_update_operation =
+                                MALI_PIXEL_KILL_FORCE_LATE;
+                        cfg.properties.pixel_kill_operation =
+                                MALI_PIXEL_KILL_FORCE_LATE;
+                } else {
+                        /* Skipping ATEST requires forcing Z/S */
                         cfg.properties.zs_update_operation =
                                 MALI_PIXEL_KILL_STRONG_EARLY;
                         cfg.properties.pixel_kill_operation =
@@ -246,7 +283,7 @@ pan_blitter_emit_rsd(const struct panfrost_device *dev,
 #else
 
                 mali_ptr blend_shader = blend_shaders ?
-                        panfrost_last_nonnull(blend_shaders, rt_count) : 0;
+                        panfrost_last_nonnull(blend_shaders, MAX2(views->rt_count, 1)) : 0;
 
                 cfg.properties.work_register_count = 4;
                 cfg.properties.force_early_z = !zs;
@@ -270,9 +307,9 @@ pan_blitter_emit_rsd(const struct panfrost_device *dev,
                         cfg.blend_equation.alpha.c = MALI_BLEND_OPERAND_C_ZERO;
                         cfg.blend_constant = 0;
 
-                        if (rts && rts[0]) {
+                        if (views->dst_rts[0] != NULL) {
                                 cfg.stencil_mask_misc.srgb =
-                                        util_format_is_srgb(rts[0]->format);
+                                        util_format_is_srgb(views->dst_rts[0]->format);
                                 cfg.blend_equation.color_mask = 0xf;
                         }
                }
@@ -281,16 +318,11 @@ pan_blitter_emit_rsd(const struct panfrost_device *dev,
         }
 
 #if PAN_ARCH >= 5
-        for (unsigned i = 0; i < MAX2(rt_count, 1); ++i) {
-                void *dest = out + pan_size(RENDERER_STATE) + pan_size(BLEND) * i;
-                const struct pan_image_view *rt_view = rts ? rts[i] : NULL;
-                mali_ptr blend_shader = blend_shaders ? blend_shaders[i] : 0;
-
-                pan_blitter_emit_blend(dev, i, rt_view, blit_shader,
-                                       blend_shader, dest);
-        }
+        pan_blitter_emit_blends(dev, blit_shader, views, blend_shaders,
+                                out + pan_size(RENDERER_STATE));
 #endif
 }
+#endif
 
 static void
 pan_blitter_get_blend_shaders(struct panfrost_device *dev,
@@ -434,7 +466,7 @@ pan_blitter_get_blit_shader(struct panfrost_device *dev,
                 nir_variable_create(b.shader, nir_var_shader_in,
                                     glsl_vector_type(GLSL_TYPE_FLOAT, coord_comps),
                                     "coord");
-        coord_var->data.location = VARYING_SLOT_TEX0;
+        coord_var->data.location = VARYING_SLOT_VAR0;
 
         nir_ssa_def *coord = nir_load_var(&b, coord_var);
 
@@ -572,6 +604,8 @@ pan_blitter_get_blit_shader(struct panfrost_device *dev,
         struct panfrost_compile_inputs inputs = {
                 .gpu_id = dev->gpu_id,
                 .is_blit = true,
+                .no_idvs = true,
+                .fixed_sysval_ubo = -1,
         };
         struct util_dynarray binary;
 
@@ -587,6 +621,9 @@ pan_blitter_get_blit_shader(struct panfrost_device *dev,
 
         GENX(pan_shader_compile)(b.shader, &inputs, &binary, &shader->info);
 
+        /* Blit shaders shouldn't have sysvals */
+        assert(shader->info.sysvals.sysval_count == 0);
+
         shader->key = *key;
         shader->address =
                 pan_pool_upload_aligned(dev->blitter.shaders.pool,
@@ -610,59 +647,92 @@ out:
         return shader;
 }
 
+static struct pan_blit_shader_key
+pan_blitter_get_key(struct pan_blitter_views *views)
+{
+        struct pan_blit_shader_key key = { 0 };
+
+        if (views->src_z) {
+                assert(views->dst_z);
+                key.surfaces[0].loc = FRAG_RESULT_DEPTH;
+                key.surfaces[0].type = nir_type_float32;
+                key.surfaces[0].src_samples = views->src_z->image->layout.nr_samples;
+                key.surfaces[0].dst_samples = views->dst_z->image->layout.nr_samples;
+                key.surfaces[0].dim = views->src_z->dim;
+                key.surfaces[0].array = views->src_z->first_layer != views->src_z->last_layer;
+        }
+
+        if (views->src_s) {
+                assert(views->dst_s);
+                key.surfaces[1].loc = FRAG_RESULT_STENCIL;
+                key.surfaces[1].type = nir_type_uint32;
+                key.surfaces[1].src_samples = views->src_s->image->layout.nr_samples;
+                key.surfaces[1].dst_samples = views->dst_s->image->layout.nr_samples;
+                key.surfaces[1].dim = views->src_s->dim;
+                key.surfaces[1].array = views->src_s->first_layer != views->src_s->last_layer;
+        }
+
+        for (unsigned i = 0; i < views->rt_count; i++) {
+                if (!views->src_rts[i])
+                        continue;
+
+                assert(views->dst_rts[i]);
+                key.surfaces[i].loc = FRAG_RESULT_DATA0 + i;
+                key.surfaces[i].type =
+                        util_format_is_pure_uint(views->src_rts[i]->format) ? nir_type_uint32 :
+                        util_format_is_pure_sint(views->src_rts[i]->format) ? nir_type_int32 :
+                        nir_type_float32;
+                key.surfaces[i].src_samples = views->src_rts[i]->image->layout.nr_samples;
+                key.surfaces[i].dst_samples = views->dst_rts[i]->image->layout.nr_samples;
+                key.surfaces[i].dim = views->src_rts[i]->dim;
+                key.surfaces[i].array = views->src_rts[i]->first_layer != views->src_rts[i]->last_layer;
+        }
+
+        return key;
+}
+
+#if PAN_ARCH <= 7
 static mali_ptr
 pan_blitter_get_rsd(struct panfrost_device *dev,
-                    unsigned rt_count,
-                    const struct pan_image_view **src_rts,
-                    const struct pan_image_view **dst_rts,
-                    const struct pan_image_view *src_z,
-                    const struct pan_image_view *dst_z,
-                    const struct pan_image_view *src_s,
-                    const struct pan_image_view *dst_s)
+                    struct pan_blitter_views *views)
 {
         struct pan_blit_rsd_key rsd_key = { 0 };
 
-        assert(!rt_count || (!src_z && !src_s));
+        assert(!views->rt_count || (!views->src_z && !views->src_s));
 
-        struct pan_blit_shader_key blit_key = { 0 };
+        struct pan_blit_shader_key blit_key = pan_blitter_get_key(views);
 
-        if (src_z) {
-                assert(dst_z);
-                rsd_key.z.format = dst_z->format;
-                blit_key.surfaces[0].loc = FRAG_RESULT_DEPTH;
-                rsd_key.z.type = blit_key.surfaces[0].type = nir_type_float32;
-                rsd_key.z.src_samples = blit_key.surfaces[0].src_samples = src_z->image->layout.nr_samples;
-                rsd_key.z.dst_samples = blit_key.surfaces[0].dst_samples = dst_z->image->layout.nr_samples;
-                rsd_key.z.dim = blit_key.surfaces[0].dim = src_z->dim;
-                rsd_key.z.array = blit_key.surfaces[0].array = src_z->first_layer != src_z->last_layer;
+        if (views->src_z) {
+                assert(views->dst_z);
+                rsd_key.z.format = views->dst_z->format;
+                rsd_key.z.type = blit_key.surfaces[0].type;
+                rsd_key.z.src_samples = blit_key.surfaces[0].src_samples;
+                rsd_key.z.dst_samples = blit_key.surfaces[0].dst_samples;
+                rsd_key.z.dim = blit_key.surfaces[0].dim;
+                rsd_key.z.array = blit_key.surfaces[0].array;
         }
 
-        if (src_s) {
-                assert(dst_s);
-                rsd_key.s.format = dst_s->format;
-                blit_key.surfaces[1].loc = FRAG_RESULT_STENCIL;
-                rsd_key.s.type = blit_key.surfaces[1].type = nir_type_uint32;
-                rsd_key.s.src_samples = blit_key.surfaces[1].src_samples = src_s->image->layout.nr_samples;
-                rsd_key.s.dst_samples = blit_key.surfaces[1].dst_samples = dst_s->image->layout.nr_samples;
-                rsd_key.s.dim = blit_key.surfaces[1].dim = src_s->dim;
-                rsd_key.s.array = blit_key.surfaces[1].array = src_s->first_layer != src_s->last_layer;
+        if (views->src_s) {
+                assert(views->dst_s);
+                rsd_key.s.format = views->dst_s->format;
+                rsd_key.s.type = blit_key.surfaces[1].type;
+                rsd_key.s.src_samples = blit_key.surfaces[1].src_samples;
+                rsd_key.s.dst_samples = blit_key.surfaces[1].dst_samples;
+                rsd_key.s.dim = blit_key.surfaces[1].dim;
+                rsd_key.s.array = blit_key.surfaces[1].array;
         }
 
-        for (unsigned i = 0; i < rt_count; i++) {
-                if (!src_rts[i])
+        for (unsigned i = 0; i < views->rt_count; i++) {
+                if (!views->src_rts[i])
                         continue;
 
-                assert(dst_rts[i]);
-                rsd_key.rts[i].format = dst_rts[i]->format;
-                blit_key.surfaces[i].loc = FRAG_RESULT_DATA0 + i;
-                rsd_key.rts[i].type = blit_key.surfaces[i].type =
-                        util_format_is_pure_uint(src_rts[i]->format) ? nir_type_uint32 :
-                        util_format_is_pure_sint(src_rts[i]->format) ? nir_type_int32 :
-                        nir_type_float32;
-                rsd_key.rts[i].src_samples = blit_key.surfaces[i].src_samples = src_rts[i]->image->layout.nr_samples;
-                rsd_key.rts[i].dst_samples = blit_key.surfaces[i].dst_samples = dst_rts[i]->image->layout.nr_samples;
-                rsd_key.rts[i].dim = blit_key.surfaces[i].dim = src_rts[i]->dim;
-                rsd_key.rts[i].array = blit_key.surfaces[i].array = src_rts[i]->first_layer != src_rts[i]->last_layer;
+                assert(views->dst_rts[i]);
+                rsd_key.rts[i].format = views->dst_rts[i]->format;
+                rsd_key.rts[i].type = blit_key.surfaces[i].type;
+                rsd_key.rts[i].src_samples = blit_key.surfaces[i].src_samples;
+                rsd_key.rts[i].dst_samples = blit_key.surfaces[i].dst_samples;
+                rsd_key.rts[i].dim = blit_key.surfaces[i].dim;
+                rsd_key.rts[i].array = blit_key.surfaces[i].array;
         }
 
         pthread_mutex_lock(&dev->blitter.rsds.lock);
@@ -675,7 +745,7 @@ pan_blitter_get_rsd(struct panfrost_device *dev,
         rsd = rzalloc(dev->blitter.rsds.rsds, struct pan_blit_rsd_data);
         rsd->key = rsd_key;
 
-        unsigned bd_count = PAN_ARCH >= 5 ? MAX2(rt_count, 1) : 0;
+        unsigned bd_count = PAN_ARCH >= 5 ? MAX2(views->rt_count, 1) : 0;
         struct panfrost_ptr rsd_ptr =
                 pan_pool_alloc_desc_aggregate(dev->blitter.rsds.pool,
                                               PAN_DESC(RENDERER_STATE),
@@ -686,12 +756,11 @@ pan_blitter_get_rsd(struct panfrost_device *dev,
         const struct pan_blit_shader_data *blit_shader =
                 pan_blitter_get_blit_shader(dev, &blit_key);
 
-        pan_blitter_get_blend_shaders(dev, rt_count, dst_rts,
+        pan_blitter_get_blend_shaders(dev, views->rt_count, views->dst_rts,
                                       blit_shader, blend_shaders);
 
-        pan_blitter_emit_rsd(dev, blit_shader,
-                             MAX2(rt_count, 1), dst_rts, blend_shaders,
-                             dst_z, dst_s, rsd_ptr.cpu);
+        pan_blitter_emit_rsd(dev, blit_shader, views, blend_shaders,
+                             rsd_ptr.cpu);
         rsd->address = rsd_ptr.gpu;
         _mesa_hash_table_insert(dev->blitter.rsds.rsds, &rsd->key, rsd);
 
@@ -701,18 +770,46 @@ out:
 }
 
 static mali_ptr
-pan_preload_get_rsd(struct panfrost_device *dev,
-                    const struct pan_fb_info *fb,
-                    bool zs)
+pan_blit_get_rsd(struct panfrost_device *dev,
+                 const struct pan_image_view *src_views,
+                 const struct pan_image_view *dst_view)
 {
-        const struct pan_image_view *rts[8] = { NULL };
-        const struct pan_image_view *z = NULL, *s = NULL;
-        struct pan_image_view patched_s_view;
-        unsigned rt_count = 0;
+        const struct util_format_description *desc =
+                util_format_description(src_views[0].format);
+
+        struct pan_blitter_views views = { };
+
+        if (util_format_has_depth(desc)) {
+                views.src_z = &src_views[0];
+                views.dst_z = dst_view;
+        }
+
+        if (src_views[1].format) {
+                views.src_s = &src_views[1];
+                views.dst_s = dst_view;
+        } else if (util_format_has_stencil(desc)) {
+                views.src_s = &src_views[0];
+                views.dst_s = dst_view;
+        }
+
+        if (!views.src_z && !views.src_s) {
+                views.rt_count = 1;
+                views.src_rts[0] = src_views;
+                views.dst_rts[0] = dst_view;
+        }
+
+        return pan_blitter_get_rsd(dev, &views);
+}
+#endif
+
+static struct pan_blitter_views
+pan_preload_get_views(const struct pan_fb_info *fb, bool zs, struct pan_image_view *patched_s)
+{
+        struct pan_blitter_views views = { 0 };
 
         if (zs) {
                 if (fb->zs.preload.z)
-                        z = fb->zs.view.zs;
+                        views.src_z = views.dst_z = fb->zs.view.zs;
 
                 if (fb->zs.preload.s) {
                         const struct pan_image_view *view = fb->zs.view.s ? : fb->zs.view.zs;
@@ -725,56 +822,25 @@ pan_preload_get_rsd(struct panfrost_device *dev,
                         }
 
                         if (fmt != view->format) {
-                                patched_s_view = *view;
-                                patched_s_view.format = fmt;
-                                s = &patched_s_view;
+                                *patched_s = *view;
+                                patched_s->format = fmt;
+                                views.src_s = views.dst_s = patched_s;
                         } else {
-                                s = view;
+                                views.src_s = views.dst_s = view;
                         }
                 }
         } else {
                 for (unsigned i = 0; i < fb->rt_count; i++) {
-                        if (fb->rts[i].preload)
-                                rts[i] = fb->rts[i].view;
+                        if (fb->rts[i].preload) {
+                                views.src_rts[i] = fb->rts[i].view;
+                                views.dst_rts[i] = fb->rts[i].view;
+                        }
                 }
 
-                rt_count = fb->rt_count;
+                views.rt_count = fb->rt_count;
         }
 
-        return pan_blitter_get_rsd(dev, rt_count, rts, rts, z, z, s, s);
-}
-
-static mali_ptr
-pan_blit_get_rsd(struct panfrost_device *dev,
-                 const struct pan_image_view *src_views,
-                 const struct pan_image_view *dst_view)
-{
-        const struct util_format_description *desc =
-                util_format_description(src_views[0].format);
-        const struct pan_image_view *src_rt = NULL, *dst_rt = NULL;
-        const struct pan_image_view *src_z = NULL, *dst_z = NULL;
-        const struct pan_image_view *src_s = NULL, *dst_s = NULL;
-
-        if (util_format_has_depth(desc)) {
-                src_z = &src_views[0];
-                dst_z = dst_view;
-        }
-
-        if (src_views[1].format) {
-                src_s = &src_views[1];
-                dst_s = dst_view;
-        } else if (util_format_has_stencil(desc)) {
-                src_s = &src_views[0];
-                dst_s = dst_view;
-        }
-
-        if (!src_z && !src_s) {
-                src_rt = &src_views[0];
-                dst_rt = dst_view;
-        }
-
-        return pan_blitter_get_rsd(dev, src_rt ? 1 : 0, &src_rt, &dst_rt,
-                                   src_z, dst_z, src_s, dst_s);
+        return views;
 }
 
 static bool
@@ -793,16 +859,41 @@ pan_preload_needed(const struct pan_fb_info *fb, bool zs)
         return false;
 }
 
-static void
-pan_blitter_emit_varying(struct pan_pool *pool,
-                         mali_ptr coordinates,
-                         struct MALI_DRAW *draw)
+static mali_ptr
+pan_blitter_emit_varying(struct pan_pool *pool)
 {
+        struct panfrost_ptr varying = pan_pool_alloc_desc(pool, ATTRIBUTE);
+
+        pan_pack(varying.cpu, ATTRIBUTE, cfg) {
+                cfg.buffer_index = 0;
+                cfg.offset_enable = PAN_ARCH <= 5;
+                cfg.format = pool->dev->formats[PIPE_FORMAT_R32G32B32_FLOAT].hw;
+
+#if PAN_ARCH >= 9
+                cfg.attribute_type = MALI_ATTRIBUTE_TYPE_1D;
+                cfg.table = PAN_TABLE_ATTRIBUTE_BUFFER;
+                cfg.frequency = MALI_ATTRIBUTE_FREQUENCY_VERTEX;
+                cfg.stride = 4 * sizeof(float);
+#endif
+        }
+
+        return varying.gpu;
+}
+
+static mali_ptr
+pan_blitter_emit_varying_buffer(struct pan_pool *pool, mali_ptr coordinates)
+{
+#if PAN_ARCH >= 9
+        struct panfrost_ptr varying_buffer = pan_pool_alloc_desc(pool, BUFFER);
+
+        pan_pack(varying_buffer.cpu, BUFFER, cfg) {
+                cfg.address = coordinates;
+                cfg.size = 4 * sizeof(float) * 4;
+        }
+#else
         /* Bifrost needs an empty desc to mark end of prefetching */
         bool padding_buffer = PAN_ARCH >= 6;
 
-        struct panfrost_ptr varying =
-                pan_pool_alloc_desc(pool, ATTRIBUTE);
         struct panfrost_ptr varying_buffer =
                 pan_pool_alloc_desc_array(pool, (padding_buffer ? 2 : 1),
                                           ATTRIBUTE_BUFFER);
@@ -817,15 +908,9 @@ pan_blitter_emit_varying(struct pan_pool *pool,
                 pan_pack(varying_buffer.cpu + pan_size(ATTRIBUTE_BUFFER),
                          ATTRIBUTE_BUFFER, cfg);
         }
+#endif
 
-        pan_pack(varying.cpu, ATTRIBUTE, cfg) {
-                cfg.buffer_index = 0;
-                cfg.offset_enable = PAN_ARCH <= 5;
-                cfg.format = pool->dev->formats[PIPE_FORMAT_R32G32B32_FLOAT].hw;
-        }
-
-        draw->varyings = varying.gpu;
-        draw->varying_buffers = varying_buffer.gpu;
+        return varying_buffer.gpu;
 }
 
 static mali_ptr
@@ -859,8 +944,7 @@ pan_blitter_emit_textures(struct pan_pool *pool,
                 size_t payload_size =
                         GENX(panfrost_estimate_texture_payload_size)(views[i]);
                 struct panfrost_ptr surfaces =
-                        pan_pool_alloc_aligned(pool, payload_size,
-                                               pan_alignment(SURFACE_WITH_STRIDE));
+                        pan_pool_alloc_aligned(pool, payload_size, 64);
 
                 GENX(panfrost_new_texture)(pool->dev, views[i], texture, &surfaces);
         }
@@ -889,10 +973,10 @@ pan_blitter_emit_textures(struct pan_pool *pool,
 #endif
 }
 
-static void
+static mali_ptr
 pan_preload_emit_textures(struct pan_pool *pool,
                           const struct pan_fb_info *fb, bool zs,
-                          struct MALI_DRAW *draw)
+                          unsigned *tex_count_out)
 {
         const struct pan_image_view *views[8];
         struct pan_image_view patched_s_view;
@@ -927,9 +1011,48 @@ pan_preload_emit_textures(struct pan_pool *pool,
 
         }
 
-        draw->textures = pan_blitter_emit_textures(pool, tex_count, views);
+        *tex_count_out = tex_count;
+
+        return pan_blitter_emit_textures(pool, tex_count, views);
 }
 
+#if PAN_ARCH >= 8
+/* TODO: cache */
+static mali_ptr
+pan_blitter_emit_zs(struct pan_pool *pool, bool z, bool s)
+{
+        struct panfrost_ptr zsd = pan_pool_alloc_desc(pool, DEPTH_STENCIL);
+
+        pan_pack(zsd.cpu, DEPTH_STENCIL, cfg) {
+                cfg.depth_function = MALI_FUNC_ALWAYS;
+                cfg.depth_write_enable = z;
+
+                if (z)
+                        cfg.depth_source = MALI_DEPTH_SOURCE_SHADER;
+
+                cfg.stencil_test_enable = s;
+                cfg.stencil_from_shader = s;
+
+                cfg.front_compare_function = MALI_FUNC_ALWAYS;
+                cfg.front_stencil_fail = MALI_STENCIL_OP_REPLACE;
+                cfg.front_depth_fail = MALI_STENCIL_OP_REPLACE;
+                cfg.front_depth_pass = MALI_STENCIL_OP_REPLACE;
+                cfg.front_write_mask = 0xFF;
+                cfg.front_value_mask = 0xFF;
+
+                cfg.back_compare_function = MALI_FUNC_ALWAYS;
+                cfg.back_stencil_fail = MALI_STENCIL_OP_REPLACE;
+                cfg.back_depth_fail = MALI_STENCIL_OP_REPLACE;
+                cfg.back_depth_pass = MALI_STENCIL_OP_REPLACE;
+                cfg.back_write_mask = 0xFF;
+                cfg.back_value_mask = 0xFF;
+
+                cfg.depth_cull_enable = false;
+        }
+
+        return zsd.gpu;
+}
+#else
 static mali_ptr
 pan_blitter_emit_viewport(struct pan_pool *pool,
                           uint16_t minx, uint16_t miny,
@@ -946,49 +1069,139 @@ pan_blitter_emit_viewport(struct pan_pool *pool,
 
         return vp.gpu;
 }
+#endif
 
 static void
 pan_preload_emit_dcd(struct pan_pool *pool,
                      struct pan_fb_info *fb, bool zs,
                      mali_ptr coordinates,
-                     mali_ptr tsd, mali_ptr rsd,
-                     void *out, bool always_write)
+                     mali_ptr tsd, void *out, bool always_write)
 {
-        pan_pack(out, DRAW, cfg) {
-                cfg.thread_storage = tsd;
-                cfg.state = rsd;
+        unsigned tex_count = 0;
+        mali_ptr textures = pan_preload_emit_textures(pool, fb, zs, &tex_count);
+        mali_ptr samplers = pan_blitter_emit_sampler(pool, true);
+        mali_ptr varyings = pan_blitter_emit_varying(pool);
+        mali_ptr varying_buffers = pan_blitter_emit_varying_buffer(pool, coordinates);
 
-                cfg.position = coordinates;
-                pan_blitter_emit_varying(pool, coordinates, &cfg);
+        /* Tiles updated by blit shaders are still considered clean (separate
+         * for colour and Z/S), allowing us to suppress unnecessary writeback
+         */
+        UNUSED bool clean_fragment_write = !always_write;
+
+        /* Image view used when patching stencil formats for combined
+         * depth/stencil preloads.
+         */
+        struct pan_image_view patched_s;
+
+        struct pan_blitter_views views = pan_preload_get_views(fb, zs, &patched_s);
+
+#if PAN_ARCH <= 7
+        pan_pack(out, DRAW, cfg) {
                 uint16_t minx = 0, miny = 0, maxx, maxy;
 
-#if PAN_ARCH == 4
-                maxx = fb->width - 1;
-                maxy = fb->height - 1;
-#else
-                /* Align on 32x32 tiles */
-                minx = fb->extent.minx & ~31;
-                miny = fb->extent.miny & ~31;
-                maxx = MIN2(ALIGN_POT(fb->extent.maxx + 1, 32), fb->width) - 1;
-                maxy = MIN2(ALIGN_POT(fb->extent.maxy + 1, 32), fb->height) - 1;
-#endif
+                if (PAN_ARCH == 4) {
+                        maxx = fb->width - 1;
+                        maxy = fb->height - 1;
+                } else {
+                        /* Align on 32x32 tiles */
+                        minx = fb->extent.minx & ~31;
+                        miny = fb->extent.miny & ~31;
+                        maxx = MIN2(ALIGN_POT(fb->extent.maxx + 1, 32), fb->width) - 1;
+                        maxy = MIN2(ALIGN_POT(fb->extent.maxy + 1, 32), fb->height) - 1;
+                }
 
+                cfg.thread_storage = tsd;
+                cfg.state = pan_blitter_get_rsd(pool->dev, &views);
+
+                cfg.position = coordinates;
                 cfg.viewport =
                         pan_blitter_emit_viewport(pool, minx, miny, maxx, maxy);
 
-                pan_preload_emit_textures(pool, fb, zs, &cfg);
-
-                cfg.samplers = pan_blitter_emit_sampler(pool, true);
+                cfg.varyings = varyings;
+                cfg.varying_buffers = varying_buffers;
+                cfg.textures = textures;
+                cfg.samplers = samplers;
 
 #if PAN_ARCH >= 6
-                /* Tiles updated by blit shaders are still considered
-                 * clean (separate for colour and Z/S), allowing us to
-                 * suppress unnecessary writeback */
-                cfg.clean_fragment_write = !always_write;
+                cfg.clean_fragment_write = clean_fragment_write;
 #endif
         }
+#else
+        struct panfrost_ptr T;
+        unsigned nr_tables = 12;
+
+        /* Although individual resources need only 16 byte alignment, the
+         * resource table as a whole must be 64-byte aligned.
+         */
+        T = pan_pool_alloc_aligned(pool, nr_tables * pan_size(RESOURCE), 64);
+        memset(T.cpu, 0, nr_tables * pan_size(RESOURCE));
+
+        panfrost_make_resource_table(T, PAN_TABLE_TEXTURE, textures, tex_count);
+        panfrost_make_resource_table(T, PAN_TABLE_SAMPLER, samplers, 1);
+        panfrost_make_resource_table(T, PAN_TABLE_ATTRIBUTE, varyings, 1);
+        panfrost_make_resource_table(T, PAN_TABLE_ATTRIBUTE_BUFFER, varying_buffers, 1);
+
+        struct pan_blit_shader_key key = pan_blitter_get_key(&views);
+        const struct pan_blit_shader_data *blit_shader =
+                pan_blitter_get_blit_shader(pool->dev, &key);
+
+        bool z = fb->zs.preload.z;
+        bool s = fb->zs.preload.s;
+        bool ms = pan_blitter_is_ms(&views);
+
+        struct panfrost_ptr spd = pan_pool_alloc_desc(pool, SHADER_PROGRAM);
+        pan_pack(spd.cpu, SHADER_PROGRAM, cfg) {
+                cfg.stage = MALI_SHADER_STAGE_FRAGMENT;
+                cfg.primary_shader = true;
+                cfg.register_allocation = MALI_SHADER_REGISTER_ALLOCATION_32_PER_THREAD;
+                cfg.binary = blit_shader->address;
+                cfg.preload.r48_r63 = blit_shader->info.preload >> 48;
+        }
+
+        unsigned bd_count = views.rt_count;
+        struct panfrost_ptr blend = pan_pool_alloc_desc_array(pool, bd_count, BLEND);
+        mali_ptr blend_shaders[8] = { 0 };
+
+        if (!zs) {
+                pan_blitter_get_blend_shaders(pool->dev, views.rt_count, views.dst_rts,
+                                              blit_shader, blend_shaders);
+
+                pan_blitter_emit_blends(pool->dev, blit_shader, &views, blend_shaders,
+                                        blend.cpu);
+        }
+
+        pan_pack(out, DRAW, cfg) {
+                if (zs) {
+                        /* ZS_EMIT requires late update/kill */
+                        cfg.zs_update_operation = MALI_PIXEL_KILL_FORCE_LATE;
+                        cfg.pixel_kill_operation = MALI_PIXEL_KILL_FORCE_LATE;
+                        cfg.blend_count = 0;
+                } else {
+                        /* Skipping ATEST requires forcing Z/S */
+                        cfg.zs_update_operation = MALI_PIXEL_KILL_STRONG_EARLY;
+                        cfg.pixel_kill_operation = MALI_PIXEL_KILL_FORCE_EARLY;
+
+                        cfg.blend = blend.gpu;
+                        cfg.blend_count = bd_count;
+                        cfg.render_target_mask = 0x1;
+                }
+
+                cfg.allow_forward_pixel_to_kill = !zs;
+                cfg.allow_forward_pixel_to_be_killed = true;
+                cfg.depth_stencil = pan_blitter_emit_zs(pool, z, s);
+                cfg.sample_mask = 0xFFFF;
+                cfg.multisample_enable = ms;
+                cfg.evaluate_per_sample = ms;
+                cfg.maximum_z = 1.0;
+                cfg.clean_fragment_write = clean_fragment_write;
+                cfg.shader.resources = T.gpu | nr_tables;
+                cfg.shader.shader = spd.gpu;
+                cfg.shader.thread_storage = tsd;
+        }
+#endif
 }
 
+#if PAN_ARCH <= 7
 static void *
 pan_blit_emit_tiler_job(struct pan_pool *pool,
                         struct pan_scoreboard *scoreboard,
@@ -1021,6 +1234,7 @@ pan_blit_emit_tiler_job(struct pan_pool *pool,
                          false, false, 0, 0, job, false);
         return pan_section_ptr(job->cpu, TILER_JOB, DRAW);
 }
+#endif
 
 #if PAN_ARCH >= 6
 static void
@@ -1037,10 +1251,9 @@ pan_preload_fb_alloc_pre_post_dcds(struct pan_pool *desc_pool,
 static void
 pan_preload_emit_pre_frame_dcd(struct pan_pool *desc_pool,
                                struct pan_fb_info *fb, bool zs,
-                               mali_ptr coords, mali_ptr rsd,
-                               mali_ptr tsd)
+                               mali_ptr coords, mali_ptr tsd)
 {
-        unsigned dcd_idx = zs ? 0 : 1;
+        unsigned dcd_idx = zs ? 1 : 0;
         pan_preload_fb_alloc_pre_post_dcds(desc_pool, fb);
         assert(fb->bifrost.pre_post.dcds.cpu);
         void *dcd = fb->bifrost.pre_post.dcds.cpu +
@@ -1065,7 +1278,7 @@ pan_preload_emit_pre_frame_dcd(struct pan_pool *desc_pool,
                         always_write = true;
         }
 
-        pan_preload_emit_dcd(desc_pool, fb, zs, coords, tsd, rsd, dcd, always_write);
+        pan_preload_emit_dcd(desc_pool, fb, zs, coords, tsd, dcd, always_write);
         if (zs) {
                 enum pipe_format fmt = fb->zs.view.zs ?
                                        fb->zs.view.zs->image->layout.format :
@@ -1105,12 +1318,12 @@ static struct panfrost_ptr
 pan_preload_emit_tiler_job(struct pan_pool *desc_pool,
                            struct pan_scoreboard *scoreboard,
                            struct pan_fb_info *fb, bool zs,
-                           mali_ptr coords, mali_ptr rsd, mali_ptr tsd)
+                           mali_ptr coords, mali_ptr tsd)
 {
         struct panfrost_ptr job =
                 pan_pool_alloc_desc(desc_pool, TILER_JOB);
 
-        pan_preload_emit_dcd(desc_pool, fb, zs, coords, tsd, rsd,
+        pan_preload_emit_dcd(desc_pool, fb, zs, coords, tsd,
                              pan_section_ptr(job.cpu, TILER_JOB, DRAW),
                              false);
 
@@ -1142,16 +1355,12 @@ pan_preload_fb_part(struct pan_pool *pool,
                     struct pan_fb_info *fb, bool zs,
                     mali_ptr coords, mali_ptr tsd, mali_ptr tiler)
 {
-        struct panfrost_device *dev = pool->dev;
-        mali_ptr rsd = pan_preload_get_rsd(dev, fb, zs);
         struct panfrost_ptr job = { 0 };
 
 #if PAN_ARCH >= 6
-        pan_preload_emit_pre_frame_dcd(pool, fb, zs,
-                                       coords, rsd, tsd);
+        pan_preload_emit_pre_frame_dcd(pool, fb, zs, coords, tsd);
 #else
-        job = pan_preload_emit_tiler_job(pool, scoreboard,
-                                         fb, zs, coords, rsd, tsd);
+        job = pan_preload_emit_tiler_job(pool, scoreboard, fb, zs, coords, tsd);
 #endif
         return job;
 }
@@ -1200,6 +1409,7 @@ GENX(pan_preload_fb)(struct pan_pool *pool,
         return njobs;
 }
 
+#if PAN_ARCH <= 7
 void
 GENX(pan_blit_ctx_init)(struct panfrost_device *dev,
                         const struct pan_blit_info *info,
@@ -1370,7 +1580,8 @@ GENX(pan_blit)(struct pan_blit_context *ctx,
                 cfg.state = ctx->rsd;
 
                 cfg.position = ctx->position;
-                pan_blitter_emit_varying(pool, src_coords, &cfg);
+                cfg.varyings = pan_blitter_emit_varying(pool);
+                cfg.varying_buffers = pan_blitter_emit_varying_buffer(pool, src_coords);
                 cfg.viewport = ctx->vpd;
                 cfg.textures = ctx->textures;
                 cfg.samplers = ctx->samplers;
@@ -1378,6 +1589,7 @@ GENX(pan_blit)(struct pan_blit_context *ctx,
 
         return job;
 }
+#endif
 
 static uint32_t pan_blit_shader_key_hash(const void *key)
 {
