@@ -28,23 +28,38 @@
 #ifndef __PAN_TEXTURE_H
 #define __PAN_TEXTURE_H
 
+#include "genxml/gen_macros.h"
+
 #include <stdbool.h>
 #include "drm-uapi/drm_fourcc.h"
 #include "util/format/u_format.h"
 #include "compiler/shader_enums.h"
-#include "midgard_pack.h"
+#include "genxml/gen_macros.h"
 #include "pan_bo.h"
 #include "pan_device.h"
 #include "pan_util.h"
 #include "pan_format.h"
 
-#define PAN_MODIFIER_COUNT 4
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#define PAN_MODIFIER_COUNT 6
 extern uint64_t pan_best_modifiers[PAN_MODIFIER_COUNT];
 
 struct pan_image_slice_layout {
         unsigned offset;
-        unsigned line_stride;
+
+        /* For AFBC images, the number of bytes between two rows of AFBC
+         * headers.
+         *
+         * For non-AFBC images, the number of bytes between two rows of texels.
+         * For linear images, this will equal the logical stride. For
+         * images that are compressed or interleaved, this will be greater than
+         * the logical stride.
+         */
         unsigned row_stride;
+
         unsigned surface_stride;
 
         struct {
@@ -54,9 +69,6 @@ struct pan_image_slice_layout {
                 /* Size of the AFBC body */
                 unsigned body_size;
 
-                /* Stride between two rows of AFBC headers */
-                unsigned row_stride;
-
                 /* Stride between AFBC headers of two consecutive surfaces.
                  * For 3D textures, this must be set to header size since
                  * AFBC headers are allocated together, for 2D arrays this
@@ -83,17 +95,6 @@ enum pan_image_crc_mode {
       PAN_IMAGE_CRC_OOB,
 };
 
-#ifndef PAN_PACK_H
-/* Avoid the GenXML dependence */
-
-enum mali_texture_dimension {
-        MALI_TEXTURE_DIMENSION_CUBE = 0,
-        MALI_TEXTURE_DIMENSION_1D   = 1,
-        MALI_TEXTURE_DIMENSION_2D   = 2,
-        MALI_TEXTURE_DIMENSION_3D   = 3,
-};
-#endif
-
 struct pan_image_layout {
         uint64_t modifier;
         enum pipe_format format;
@@ -101,15 +102,20 @@ struct pan_image_layout {
         unsigned nr_samples;
         enum mali_texture_dimension dim;
         unsigned nr_slices;
-        struct pan_image_slice_layout slices[MAX_MIP_LEVELS];
         unsigned array_size;
-        unsigned array_stride;
-        unsigned data_size;
-
         enum pan_image_crc_mode crc_mode;
+
+        /* The remaining fields may be derived from the above by calling
+         * pan_image_layout_init
+         */
+
+        struct pan_image_slice_layout slices[MAX_MIP_LEVELS];
+
         /* crc_size != 0 only if crc_mode == OOB otherwise CRC words are
          * counted in data_size */
         unsigned crc_size;
+        unsigned data_size;
+        unsigned array_stride;
 };
 
 struct pan_image_mem {
@@ -157,26 +163,54 @@ bool
 panfrost_format_supports_afbc(const struct panfrost_device *dev,
                 enum pipe_format format);
 
-#define AFBC_HEADER_BYTES_PER_TILE 16
+enum pipe_format
+panfrost_afbc_format(unsigned arch, enum pipe_format format);
 
-unsigned
-panfrost_afbc_header_size(unsigned width, unsigned height);
+#define AFBC_HEADER_BYTES_PER_TILE 16
 
 bool
 panfrost_afbc_can_ytr(enum pipe_format format);
 
-unsigned
-panfrost_block_dim(uint64_t modifier, bool width, unsigned plane);
+bool
+panfrost_afbc_can_tile(const struct panfrost_device *dev);
 
+/*
+ * Represents the block size of a single plane. For AFBC, this represents the
+ * superblock size. For u-interleaving, this represents the tile size.
+ */
+struct pan_block_size {
+        /** Width of block */
+        unsigned width;
+
+        /** Height of blocks */
+        unsigned height;
+};
+
+struct pan_block_size panfrost_afbc_superblock_size(uint64_t modifier);
+
+unsigned panfrost_afbc_superblock_width(uint64_t modifier);
+
+unsigned panfrost_afbc_superblock_height(uint64_t modifier);
+
+bool panfrost_afbc_is_wide(uint64_t modifier);
+
+uint32_t pan_afbc_row_stride(uint64_t modifier, uint32_t width);
+
+uint32_t pan_afbc_stride_blocks(uint64_t modifier, uint32_t row_stride_bytes);
+
+struct pan_block_size
+panfrost_block_size(uint64_t modifier, enum pipe_format format);
+
+#ifdef PAN_ARCH
 unsigned
-panfrost_estimate_texture_payload_size(const struct panfrost_device *dev,
-                                       const struct pan_image_view *iview);
+GENX(panfrost_estimate_texture_payload_size)(const struct pan_image_view *iview);
 
 void
-panfrost_new_texture(const struct panfrost_device *dev,
-                     const struct pan_image_view *iview,
-                     void *out,
-                     const struct panfrost_ptr *payload);
+GENX(panfrost_new_texture)(const struct panfrost_device *dev,
+                           const struct pan_image_view *iview,
+                           void *out,
+                           const struct panfrost_ptr *payload);
+#endif
 
 unsigned
 panfrost_get_layer_stride(const struct pan_image_layout *layout,
@@ -196,38 +230,30 @@ struct pan_scoreboard;
         ((mod >> 52) == (DRM_FORMAT_MOD_ARM_TYPE_AFBC | \
                 (DRM_FORMAT_MOD_VENDOR_ARM << 4)))
 
-/* Map modifiers to mali_texture_layout for packing in a texture descriptor */
-
-static inline enum mali_texture_layout
-panfrost_modifier_to_layout(uint64_t modifier)
-{
-        if (drm_is_afbc(modifier))
-                return MALI_TEXTURE_LAYOUT_AFBC;
-        else if (modifier == DRM_FORMAT_MOD_ARM_16X16_BLOCK_U_INTERLEAVED)
-                return MALI_TEXTURE_LAYOUT_TILED;
-        else if (modifier == DRM_FORMAT_MOD_LINEAR)
-                return MALI_TEXTURE_LAYOUT_LINEAR;
-        else
-                unreachable("Invalid modifer");
-}
-
 struct pan_image_explicit_layout {
         unsigned offset;
-        unsigned line_stride;
+        unsigned row_stride;
 };
 
 bool
-pan_image_layout_init(const struct panfrost_device *dev,
-                      struct pan_image_layout *layout,
-                      uint64_t modifier,
-                      enum pipe_format format,
-                      enum mali_texture_dimension dim,
-                      unsigned width, unsigned height, unsigned depth,
-                      unsigned array_size, unsigned nr_samples,
-                      unsigned nr_slices,
-                      enum pan_image_crc_mode crc_mode,
+pan_image_layout_init(struct pan_image_layout *layout,
                       const struct pan_image_explicit_layout *explicit_layout);
 
+unsigned
+panfrost_get_legacy_stride(const struct pan_image_layout *layout,
+                           unsigned level);
+
+unsigned
+panfrost_from_legacy_stride(unsigned legacy_stride,
+                            enum pipe_format format,
+                            uint64_t modifier);
+
+uint32_t
+pan_stride_align_B(enum pipe_format format);
+
+bool
+pan_is_stride_aligned(enum pipe_format format, unsigned stride_B);
+
 struct pan_surface {
         union {
                 mali_ptr data;
@@ -243,4 +269,14 @@ pan_iview_get_surface(const struct pan_image_view *iview,
                       unsigned level, unsigned layer, unsigned sample,
                       struct pan_surface *surf);
 
+
+#if PAN_ARCH >= 9
+enum mali_afbc_compression_mode
+pan_afbc_compression_mode(enum pipe_format format);
+#endif
+
+#ifdef __cplusplus
+} /* extern C */
+#endif
+
 #endif
