@@ -62,6 +62,42 @@ vn_descriptor_set_destroy(struct vn_device *dev,
    vk_free(alloc, set);
 }
 
+/* Mapping VkDescriptorType to array index */
+static enum vn_descriptor_type
+vn_descriptor_type_index(VkDescriptorType type)
+{
+   switch (type) {
+   case VK_DESCRIPTOR_TYPE_SAMPLER:
+      return VN_DESCRIPTOR_TYPE_SAMPLER;
+   case VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER:
+      return VN_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER;
+   case VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE:
+      return VN_DESCRIPTOR_TYPE_SAMPLED_IMAGE;
+   case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE:
+      return VN_DESCRIPTOR_TYPE_STORAGE_IMAGE;
+   case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
+      return VN_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER;
+   case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
+      return VN_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER;
+   case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER:
+      return VN_DESCRIPTOR_TYPE_UNIFORM_BUFFER;
+   case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER:
+      return VN_DESCRIPTOR_TYPE_STORAGE_BUFFER;
+   case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
+      return VN_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC;
+   case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC:
+      return VN_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC;
+   case VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT:
+      return VN_DESCRIPTOR_TYPE_INPUT_ATTACHMENT;
+   case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
+      return VN_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK;
+   default:
+      break;
+   }
+
+   unreachable("bad VkDescriptorType");
+}
+
 /* descriptor set layout commands */
 
 void
@@ -240,10 +276,15 @@ vn_CreateDescriptorPool(VkDevice device,
                         const VkAllocationCallbacks *pAllocator,
                         VkDescriptorPool *pDescriptorPool)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    const VkAllocationCallbacks *alloc =
       pAllocator ? pAllocator : &dev->base.base.alloc;
 
+   const VkDescriptorPoolInlineUniformBlockCreateInfo *iub_info =
+      vk_find_struct_const(pCreateInfo->pNext,
+                           DESCRIPTOR_POOL_INLINE_UNIFORM_BLOCK_CREATE_INFO);
+
    struct vn_descriptor_pool *pool =
       vk_zalloc(alloc, sizeof(*pool), VN_DEFAULT_ALIGN,
                 VK_SYSTEM_ALLOCATION_SCOPE_OBJECT);
@@ -259,18 +300,27 @@ vn_CreateDescriptorPool(VkDevice device,
     * allocation must not fail due to a fragmented pool per spec. In this
     * case, set allocation can be asynchronous with pool resource tracking.
     */
-   pool->async_set_allocation = !(
-      pCreateInfo->flags & VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT);
+   pool->async_set_allocation =
+      !VN_PERF(NO_ASYNC_SET_ALLOC) &&
+      !(pCreateInfo->flags &
+        VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT);
 
    pool->max.set_count = pCreateInfo->maxSets;
 
+   if (iub_info)
+      pool->max.iub_binding_count = iub_info->maxInlineUniformBlockBindings;
+
    for (uint32_t i = 0; i < pCreateInfo->poolSizeCount; i++) {
       const VkDescriptorPoolSize *pool_size = &pCreateInfo->pPoolSizes[i];
+      const uint32_t type_index = vn_descriptor_type_index(pool_size->type);
 
-      assert(pool_size->type < VN_NUM_DESCRIPTOR_TYPES);
+      assert(type_index < VN_NUM_DESCRIPTOR_TYPES);
 
-      pool->max.descriptor_counts[pool_size->type] +=
-         pool_size->descriptorCount;
+      pool->max.descriptor_counts[type_index] += pool_size->descriptorCount;
+
+      assert((pCreateInfo->pPoolSizes[i].type !=
+              VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK) ||
+             iub_info);
    }
 
    list_inithead(&pool->descriptor_sets);
@@ -289,6 +339,7 @@ vn_DestroyDescriptorPool(VkDevice device,
                          VkDescriptorPool descriptorPool,
                          const VkAllocationCallbacks *pAllocator)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    struct vn_descriptor_pool *pool =
       vn_descriptor_pool_from_handle(descriptorPool);
@@ -339,10 +390,25 @@ vn_descriptor_pool_alloc_descriptors(
                                 ? last_binding_descriptor_count
                                 : layout->bindings[i].count;
 
-      pool->used.descriptor_counts[type] += count;
+      /* Allocation may fail if a call to vkAllocateDescriptorSets would cause
+       * the total number of inline uniform block bindings allocated from the
+       * pool to exceed the value of
+       * VkDescriptorPoolInlineUniformBlockCreateInfo::maxInlineUniformBlockBindings
+       * used to create the descriptor pool.
+       */
+      if (type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK) {
+         if (++pool->used.iub_binding_count > pool->max.iub_binding_count) {
+            /* restore pool state before this allocation */
+            pool->used = recovery;
+            return false;
+         }
+      }
 
-      if (pool->used.descriptor_counts[type] >
-          pool->max.descriptor_counts[type]) {
+      const uint32_t type_index = vn_descriptor_type_index(type);
+      pool->used.descriptor_counts[type_index] += count;
+
+      if (pool->used.descriptor_counts[type_index] >
+          pool->max.descriptor_counts[type_index]) {
          /* restore pool state before this allocation */
          pool->used = recovery;
          return false;
@@ -366,7 +432,11 @@ vn_descriptor_pool_free_descriptors(
                                 ? last_binding_descriptor_count
                                 : layout->bindings[i].count;
 
-      pool->used.descriptor_counts[layout->bindings[i].type] -= count;
+      pool->used.descriptor_counts[vn_descriptor_type_index(
+         layout->bindings[i].type)] -= count;
+
+      if (layout->bindings[i].type == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK)
+         --pool->used.iub_binding_count;
    }
 
    --pool->used.set_count;
@@ -386,6 +456,7 @@ vn_ResetDescriptorPool(VkDevice device,
                        VkDescriptorPool descriptorPool,
                        VkDescriptorPoolResetFlags flags)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    struct vn_descriptor_pool *pool =
       vn_descriptor_pool_from_handle(descriptorPool);
@@ -410,6 +481,7 @@ vn_AllocateDescriptorSets(VkDevice device,
                           const VkDescriptorSetAllocateInfo *pAllocateInfo,
                           VkDescriptorSet *pDescriptorSets)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    struct vn_descriptor_pool *pool =
       vn_descriptor_pool_from_handle(pAllocateInfo->descriptorPool);
@@ -528,6 +600,7 @@ vn_FreeDescriptorSets(VkDevice device,
                       uint32_t descriptorSetCount,
                       const VkDescriptorSet *pDescriptorSets)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    struct vn_descriptor_pool *pool =
       vn_descriptor_pool_from_handle(descriptorPool);
@@ -554,6 +627,7 @@ vn_update_descriptor_sets_alloc(uint32_t write_count,
                                 uint32_t image_count,
                                 uint32_t buffer_count,
                                 uint32_t view_count,
+                                uint32_t iub_count,
                                 const VkAllocationCallbacks *alloc,
                                 VkSystemAllocationScope scope)
 {
@@ -564,7 +638,11 @@ vn_update_descriptor_sets_alloc(uint32_t write_count,
       images_offset + sizeof(VkDescriptorImageInfo) * image_count;
    const size_t views_offset =
       buffers_offset + sizeof(VkDescriptorBufferInfo) * buffer_count;
-   const size_t alloc_size = views_offset + sizeof(VkBufferView) * view_count;
+   const size_t iubs_offset =
+      views_offset + sizeof(VkBufferView) * view_count;
+   const size_t alloc_size =
+      iubs_offset +
+      sizeof(VkWriteDescriptorSetInlineUniformBlock) * iub_count;
 
    void *storage = vk_alloc(alloc, alloc_size, VN_DEFAULT_ALIGN, scope);
    if (!storage)
@@ -576,6 +654,7 @@ vn_update_descriptor_sets_alloc(uint32_t write_count,
    update->images = storage + images_offset;
    update->buffers = storage + buffers_offset;
    update->views = storage + views_offset;
+   update->iubs = storage + iubs_offset;
 
    return update;
 }
@@ -602,7 +681,7 @@ vn_update_descriptor_sets_parse_writes(uint32_t write_count,
    }
 
    struct vn_update_descriptor_sets *update =
-      vn_update_descriptor_sets_alloc(write_count, img_count, 0, 0, alloc,
+      vn_update_descriptor_sets_alloc(write_count, img_count, 0, 0, 0, alloc,
                                       VK_SYSTEM_ALLOCATION_SCOPE_COMMAND);
    if (!update)
       return NULL;
@@ -668,6 +747,7 @@ vn_update_descriptor_sets_parse_writes(uint32_t write_count,
          write->pImageInfo = NULL;
          write->pTexelBufferView = NULL;
          break;
+      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
       default:
          write->pImageInfo = NULL;
          write->pBufferInfo = NULL;
@@ -686,6 +766,7 @@ vn_UpdateDescriptorSets(VkDevice device,
                         uint32_t descriptorCopyCount,
                         const VkCopyDescriptorSet *pDescriptorCopies)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    const VkAllocationCallbacks *alloc = &dev->base.base.alloc;
 
@@ -716,6 +797,7 @@ vn_update_descriptor_sets_parse_template(
    uint32_t img_count = 0;
    uint32_t buf_count = 0;
    uint32_t view_count = 0;
+   uint32_t iub_count = 0;
    for (uint32_t i = 0; i < create_info->descriptorUpdateEntryCount; i++) {
       const VkDescriptorUpdateTemplateEntry *entry =
          &create_info->pDescriptorUpdateEntries[i];
@@ -738,6 +820,9 @@ vn_update_descriptor_sets_parse_template(
       case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC:
          buf_count += entry->descriptorCount;
          break;
+      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
+         iub_count += 1;
+         break;
       default:
          unreachable("unhandled descriptor type");
          break;
@@ -746,13 +831,14 @@ vn_update_descriptor_sets_parse_template(
 
    struct vn_update_descriptor_sets *update = vn_update_descriptor_sets_alloc(
       create_info->descriptorUpdateEntryCount, img_count, buf_count,
-      view_count, alloc, VK_SYSTEM_ALLOCATION_SCOPE_OBJECT);
+      view_count, iub_count, alloc, VK_SYSTEM_ALLOCATION_SCOPE_OBJECT);
    if (!update)
       return NULL;
 
    img_count = 0;
    buf_count = 0;
    view_count = 0;
+   iub_count = 0;
    for (uint32_t i = 0; i < create_info->descriptorUpdateEntryCount; i++) {
       const VkDescriptorUpdateTemplateEntry *entry =
          &create_info->pDescriptorUpdateEntries[i];
@@ -795,6 +881,19 @@ vn_update_descriptor_sets_parse_template(
          write->pTexelBufferView = NULL;
          buf_count += entry->descriptorCount;
          break;
+      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
+         write->pImageInfo = NULL;
+         write->pBufferInfo = NULL;
+         write->pTexelBufferView = NULL;
+         VkWriteDescriptorSetInlineUniformBlock *iub_data =
+            &update->iubs[iub_count];
+         iub_data->sType =
+            VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_INLINE_UNIFORM_BLOCK;
+         iub_data->pNext = write->pNext;
+         iub_data->dataSize = entry->descriptorCount;
+         write->pNext = iub_data;
+         iub_count += 1;
+         break;
       default:
          break;
       }
@@ -810,6 +909,7 @@ vn_CreateDescriptorUpdateTemplate(
    const VkAllocationCallbacks *pAllocator,
    VkDescriptorUpdateTemplate *pDescriptorUpdateTemplate)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    const VkAllocationCallbacks *alloc =
       pAllocator ? pAllocator : &dev->base.base.alloc;
@@ -848,6 +948,7 @@ vn_DestroyDescriptorUpdateTemplate(
    VkDescriptorUpdateTemplate descriptorUpdateTemplate,
    const VkAllocationCallbacks *pAllocator)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    struct vn_descriptor_update_template *templ =
       vn_descriptor_update_template_from_handle(descriptorUpdateTemplate);
@@ -872,6 +973,7 @@ vn_UpdateDescriptorSetWithTemplate(
    VkDescriptorUpdateTemplate descriptorUpdateTemplate,
    const void *pData)
 {
+   VN_TRACE_FUNC();
    struct vn_device *dev = vn_device_from_handle(device);
    struct vn_descriptor_set *set =
       vn_descriptor_set_from_handle(descriptorSet);
@@ -936,6 +1038,12 @@ vn_UpdateDescriptorSetWithTemplate(
             *dst = *src;
          }
          break;
+      case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:;
+         VkWriteDescriptorSetInlineUniformBlock *iub_data =
+            (VkWriteDescriptorSetInlineUniformBlock *)vk_find_struct_const(
+               write->pNext, WRITE_DESCRIPTOR_SET_INLINE_UNIFORM_BLOCK);
+         iub_data->pData = pData + entry->offset;
+         break;
       default:
          unreachable("unhandled descriptor type");
          break;
